@@ -12,8 +12,10 @@
 #include <qsocketdevice.h>
 #include <qstringlist.h>
 #include <qnetwork.h> 
+#include <qmutex.h>
 
 #include <iostream>
+#include <sstream>
 #include <vector>
 
 using namespace std;
@@ -27,12 +29,12 @@ class QSqlDatabase;
 class QSqlQuery;
 class QSqlError;
 class QSocket;
-class LCD;
 class MythMainWindow;
 class MythPluginManager;
 class MediaMonitor;
 class MythMediaDevice;
 class DisplayRes;
+class MDBManager;
 
 enum VerboseMask {
     VB_IMPORTANT = 0x0001,
@@ -47,6 +49,8 @@ enum VerboseMask {
     VB_COMMFLAG  = 0x0200,
     VB_AUDIO     = 0x0400,
     VB_LIBAV     = 0x0800,
+    VB_JOBQUEUE  = 0x1000,
+    VB_SIPARSER  = 0x2000,
     VB_NONE      = 0x0000,
     VB_ALL       = 0xffff
 };
@@ -62,13 +66,61 @@ enum LogPriorities {
     LP_DEBUG     = 7
 };
 
+struct DatabaseParams {
+    QString dbHostName;         // database server
+    QString dbUserName;         // DB user name 
+    QString dbPassword;         // DB password
+    QString dbName;             // database name
+    QString dbType;             // database type (MySQL, Postgres, etc.)
+    
+    bool    localEnabled;       // true if localHostName is not default
+    QString localHostName;      // name used for loading/saving settings
+    
+    bool    wolEnabled;         // true if wake-on-lan params are used
+    int     wolReconnect;       // seconds to wait for reconnect
+    int     wolRetry;           // times to retry
+    QString wolCommand;         // command to use for wake-on-lan
+};
+    
+
+// The verbose_mutex lock is a recursive lock so it is possible (while
+// not recommended) to use a VERBOSE macro within another VERBOSE macro.
+// But waiting for another thread to do something is not safe within a 
+// VERBOSE macro, since those threads may wish to use the VERBOSE macro
+// and this will cause a deadlock.
+#ifdef DEBUG
+
 #define VERBOSE(mask,args...) \
 do { \
-if ((print_verbose_messages & mask) != 0) \
-    cout << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss") \
-         << " " << args << endl; \
+    if ((print_verbose_messages & mask) != 0) \
+    { \
+        QDateTime dtmp = QDateTime::currentDateTime(); \
+        QString dtime = dtmp.toString("yyyy-MM-dd hh:mm:ss.zzz"); \
+        MythContext::verbose_mutex.lock(); \
+        cout << dtime << " " << args << endl; \
+        MythContext::verbose_mutex.unlock(); \
+    } \
 } while (0)
 
+#else // if !DEBUG
+
+// use a slower non-deadlockable version in release builds
+#define VERBOSE(mask,args...) \
+do { \
+    if ((print_verbose_messages & mask) != 0) \
+    { \
+        QDateTime dtmp = QDateTime::currentDateTime(); \
+        QString dtime = dtmp.toString("yyyy-MM-dd hh:mm:ss.zzz"); \
+        ostringstream verbose_macro_tmp; \
+        verbose_macro_tmp << dtime << " " << args; \
+        MythContext::verbose_mutex.lock(); \
+        cout << verbose_macro_tmp.str() << endl; \
+        MythContext::verbose_mutex.unlock(); \
+    } \
+} while (0)
+
+#endif // DEBUG
+
 class MythEvent : public QCustomEvent
 {
   public:
@@ -110,8 +162,8 @@ class MythPrivRequest
     void *m_data;
 };
 
-#define MYTH_BINARY_VERSION "0.16.20040906-1"
-#define MYTH_PROTO_VERSION "13"
+#define MYTH_BINARY_VERSION "0.18.20050409-1"
+#define MYTH_PROTO_VERSION "15"
 
 extern int print_verbose_messages;
 
@@ -121,9 +173,11 @@ class MythContext : public QObject
 {
     Q_OBJECT
   public:
-    MythContext(const QString &binversion, bool gui = true, bool lcd = true);
+    MythContext(const QString &binversion);
     virtual ~MythContext();
 
+    bool Init(bool gui = true);
+
     QString GetMasterHostPrefix(void);
 
     QString GetHostName(void);
@@ -131,9 +185,13 @@ class MythContext : public QObject
     bool ConnectToMasterServer(void);
     bool ConnectServer(const QString &hostname, int port);
     bool IsConnectedToMaster(void);
+    void SetBackend(bool backend);
+    bool IsBackend(void);
 
     QString GetInstallPrefix(void);
     QString GetShareDir(void);
+    QString GetLibraryDir(void);
+    static QString GetConfDir(void);
 
     QString GetFilePrefix(void);
 
@@ -143,22 +201,48 @@ class MythContext : public QObject
 
     void RefreshBackendConfig(void);
 
+    // Note that these give the dimensions for the GUI,
+    // which the user may have set to be different from the raw screen size
     void GetScreenSettings(float &wmult, float &hmult);
     void GetScreenSettings(int &width, float &wmult,
                            int &height, float &hmult);
     void GetScreenSettings(int &xbase, int &width, float &wmult,
                            int &ybase, int &height, float &hmult);
-   
+
+    // This returns the raw (drawable) screen size
+    void GetScreenBounds(int &xbase, int &ybase, int &width, int &height);
+
+    // Parse an X11 style command line (-geometry) string
+    bool ParseGeometryOverride(const QString geometry);
+
     QString FindThemeDir(const QString &themename);
     QString GetThemeDir(void);
 
     QString GetMenuThemeDir(void);
 
-    int OpenDatabase(QSqlDatabase *db);
-    static void KickDatabase(QSqlDatabase *db);
-    static void DBError(const QString &where, const QSqlQuery& query);
+    QString GetThemesParentDir(void);
+
+    QString GetPluginsDir(void);
+    QString GetPluginsNameFilter(void);
+    QString FindPlugin(const QString &plugname);
+
+    QString GetTranslationsDir(void);
+    QString GetTranslationsNameFilter(void);
+    QString FindTranslation(const QString &translation);
+
+    QString GetFontsDir(void);
+    QString GetFontsNameFilter(void);
+    QString FindFont(const QString &fontname);
+
+    QString GetFiltersDir(void);
+
+    MDBManager *GetDBManager(void);
+    static void DBError(const QString &where, const QSqlQuery &query);
     static QString DBErrorMessage(const QSqlError& err);
 
+    DatabaseParams GetDatabaseParams(void);
+    bool SaveDatabaseParams(const DatabaseParams &params);
+    
     void LogEntry(const QString &module, int priority,
                   const QString &message, const QString &details);
 
@@ -168,7 +252,15 @@ class MythContext : public QObject
     void SaveSetting(const QString &key, int newValue);
     void SaveSetting(const QString &key, const QString &newValue);
     QString GetSetting(const QString &key, const QString &defaultval = "");
+
+    // Convenience setting query methods
     int GetNumSetting(const QString &key, int defaultval = 0);
+    double GetFloatSetting(const QString &key, double defaultval = 0.0);
+    void GetResolutionSetting(const QString &type, int &width, int &height,
+                              double& forced_aspect, short &refreshrate,
+                              int index=-1);
+    void GetResolutionSetting(const QString &type, int &width, int &height,
+                              int index=-1);
 
     QString GetSettingOnHost(const QString &key, const QString &host,
                              const QString &defaultval = "");
@@ -200,8 +292,6 @@ class MythContext : public QObject
     void SetMainWindow(MythMainWindow *mainwin);
     MythMainWindow *GetMainWindow(void);
 
-    LCD *GetLCDDevice(void);
-
     bool TestPopupVersion(const QString &name, const QString &libversion,
                           const QString &pluginversion);
 
@@ -229,9 +319,11 @@ class MythContext : public QObject
     bool GetScreensaverEnabled(void);
 
     void addPrivRequest(MythPrivRequest::Type t, void *data);
-    bool hasPrivRequest() const;
+    void waitPrivRequest() const;
     MythPrivRequest popPrivRequest();
 
+    static QMutex verbose_mutex;
+
   private slots:
     void EventSocketRead();
     void EventSocketConnected();
