@@ -12,7 +12,7 @@
 
 using namespace std;
 
-class QSqlDatabase;
+//class QSqlDatabase;
 class QWidget;
 class ConfigurationGroup;
 class QDir;
@@ -21,15 +21,15 @@ class Configurable: virtual public QObject {
     Q_OBJECT
 public:
     Configurable():
-        labelAboveWidget(false), visible(true) {};
+        labelAboveWidget(false), enabled(true), visible(true) {};
     virtual ~Configurable() {};
 
     // Create and return a widget for configuring this entity
     virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent,
                                   const char* widgetName = 0);
 
-    virtual void load(QSqlDatabase* db) = 0;
-    virtual void save(QSqlDatabase* db) = 0;
+    virtual void load() = 0;
+    virtual void save() = 0;
 
     // A name for looking up the setting
     void setName(QString str) {
@@ -51,8 +51,12 @@ public:
     void setVisible(bool b) { visible = b; };
     bool isVisible(void) const { return visible; };
 
+    virtual void setEnabled(bool b) { enabled = b; }
+    bool isEnabled() { return enabled; }
+
 protected:
     bool labelAboveWidget; 
+    bool enabled;
 
 private:
     QString configName;
@@ -99,8 +103,10 @@ class ConfigurationGroup: virtual public Configurable
 {
     Q_OBJECT
   public:
-    ConfigurationGroup(bool luselabel = true, bool luseframe = true) 
-             { uselabel = luselabel; useframe = luseframe; }
+    ConfigurationGroup(bool luselabel = true, bool luseframe = true,
+                       bool lzeroMargin = false, bool lzeroSpace = false) 
+              { uselabel = luselabel; useframe = luseframe; 
+               zeroMargin = lzeroMargin; zeroSpace = lzeroSpace; }
     virtual ~ConfigurationGroup();
 
 
@@ -110,9 +116,9 @@ class ConfigurationGroup: virtual public Configurable
 
     virtual Setting* byName(QString name);
 
-    virtual void load(QSqlDatabase* db);
+    virtual void load();
 
-    virtual void save(QSqlDatabase* db);
+    virtual void save();
 
     void setUseLabel(bool useit) { uselabel = useit; }
     void setUseFrame(bool useit) { useframe = useit; }
@@ -125,12 +131,15 @@ class ConfigurationGroup: virtual public Configurable
     childList children;
     bool uselabel;
     bool useframe;
+    bool zeroMargin;
+    bool zeroSpace;
 };
 
 class VerticalConfigurationGroup: virtual public ConfigurationGroup {
  public:
-    VerticalConfigurationGroup(bool uselabel = true, bool useframe = true) 
-                : ConfigurationGroup(uselabel, useframe) { }
+    VerticalConfigurationGroup(bool uselabel = true, bool useframe = true,
+                                 bool zeroMargin = false, bool zeroSpace = false) 
+                : ConfigurationGroup(uselabel, useframe, zeroMargin, zeroSpace) { }
 
     virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent,
                                   const char* widgetName = 0);
@@ -138,13 +147,27 @@ class VerticalConfigurationGroup: virtual public ConfigurationGroup {
 
 class HorizontalConfigurationGroup: virtual public ConfigurationGroup {
  public:
-    HorizontalConfigurationGroup(bool uselabel = true, bool useframe = true) 
-                : ConfigurationGroup(uselabel, useframe) { }
+    HorizontalConfigurationGroup(bool uselabel = true, bool useframe = true,
+                                 bool zeroMargin = false, bool zeroSpace = false) 
+                : ConfigurationGroup(uselabel, useframe, zeroMargin, zeroSpace) { }
 
     virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent,
                                   const char* widgetName = 0);
 };
 
+class GridConfigurationGroup: virtual public ConfigurationGroup {
+ public:
+    GridConfigurationGroup(uint col, bool uselabel = true, bool useframe = true,
+                           bool zeroMargin = false, bool zeroSpace = false) 
+                : ConfigurationGroup(uselabel, useframe, zeroMargin, zeroSpace), 
+                  columns(col) { }
+
+    virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent,
+                                  const char* widgetName = 0);
+ private:
+    uint columns;
+};
+
 class StackedConfigurationGroup: virtual public ConfigurationGroup {
     Q_OBJECT
 public:
@@ -155,7 +178,7 @@ public:
                                   const char* widgetName = 0);
 
     void raise(Configurable* child);
-    virtual void save(QSqlDatabase* db);
+    virtual void save();
 
     // save all children, or only the top?
     void setSaveAll(bool b) { saveAll = b; };
@@ -169,12 +192,17 @@ protected:
 };
 
 class ConfigurationDialogWidget: public MythDialog {
+    Q_OBJECT
 public:
     ConfigurationDialogWidget(MythMainWindow *parent, 
                               const char* widgetName = 0):
         MythDialog(parent, widgetName) {};
 
     virtual void keyPressEvent(QKeyEvent* e);
+
+signals:
+    void editButtonPressed();
+    void deleteButtonPressed();
 };
 
 class ConfigurationDialog: virtual public Configurable {
@@ -184,7 +212,7 @@ public:
                                      const char* widgetName = 0);
 
     // Show a dialogWidget, and save if accepted
-    int exec(QSqlDatabase* db, bool saveOnExec = true, bool doLoad = true);
+    int exec(bool saveOnExec = true, bool doLoad = true);
 
 protected:
     MythDialog *dialog;
@@ -209,7 +237,7 @@ public:
 
 class LineEditSetting: virtual public Setting {
 protected:
-    LineEditSetting(bool readwrite = true) { rw = readwrite; };
+    LineEditSetting(bool readwrite = true) : edit(NULL) { rw = readwrite; };
 public:
     virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent, 
                                   const char* widgetName = 0);
@@ -217,6 +245,9 @@ public:
     void setRW(bool readwrite = true) { rw = readwrite; edit->setRW(rw); };
     void setRO() { rw = false; edit->setRO(); };
 
+    virtual void setEnabled(bool b);
+    virtual void setVisible(bool b);
+
 private:
     MythLineEdit* edit;
     bool rw;
@@ -344,10 +375,12 @@ class ComboBoxSetting: public SelectSetting {
     Q_OBJECT;
 
 protected:
-    ComboBoxSetting(bool _rw = false) {
+    ComboBoxSetting(bool _rw = false, int _step = 1) {
         rw = _rw;
+        step = _step;
         widget = NULL;
     };
+
 public:
     virtual void setValue(QString newValue) {
         if (!rw)
@@ -365,6 +398,9 @@ public:
 
     void setFocus() { if (widget) widget->setFocus(); }
 
+    virtual void setEnabled(bool b);
+    virtual void setVisible(bool b);
+
 public slots:
     void addSelection(const QString& label,
                       QString value=QString::null,
@@ -379,16 +415,24 @@ protected slots:
 private:
     bool rw;
     MythComboBox *widget;
+
+protected:
+    int step;
 };
 
 class ListBoxSetting: public SelectSetting {
     Q_OBJECT
 public:
-    ListBoxSetting(): widget(NULL) { }
+    ListBoxSetting(): widget(NULL), selectionMode(MythListBox::Single) { }
     virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent, 
                                   const char* widgetName = 0);
 
     void setFocus() { if (widget) widget->setFocus(); }
+    void setSelectionMode(MythListBox::SelectionMode mode);
+    void setCurrentItem(int i) { if (widget) widget->setCurrentItem(i); }
+    void setCurrentItem(const QString& str)  { if (widget) widget->setCurrentItem(str); }
+
+    virtual void setEnabled(bool b);
 
 signals:
     void accepted(int);
@@ -399,13 +443,14 @@ protected slots:
     void addSelection(const QString& label,
                       QString value=QString::null,
                       bool select=false) {
+        SelectSetting::addSelection(label, value, select);
         if (widget != NULL)
             widget->insertItem(label);
-        SelectSetting::addSelection(label, value, select);
     };
     void widgetDestroyed() { widget=NULL; };
 protected:
     MythListBox* widget;
+    MythListBox::SelectionMode selectionMode;
 };
 
 class RadioSetting: public SelectSetting {
@@ -454,8 +499,12 @@ signals:
 
 class CheckBoxSetting: public BooleanSetting {
 public:
+    CheckBoxSetting() : widget(NULL) {}
     virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent,
                                   const char* widgetName = 0);
+    virtual void setEnabled(bool b);
+protected:
+    MythCheckBox *widget;
 };
 
 
@@ -520,9 +569,9 @@ public:
         setLabel("Channel");
     };
 
-    static void fillSelections(QSqlDatabase* db, SelectSetting* setting);
-    virtual void fillSelections(QSqlDatabase* db) {
-        fillSelections(db, this);
+    static void fillSelections(SelectSetting* setting);
+    virtual void fillSelections() {
+        fillSelections(this);
     };
 };
 
@@ -557,8 +606,8 @@ public:
     DBStorage(QString _table, QString _column):
         table(_table), column(_column) {};
 
-    virtual void load(QSqlDatabase* db) = 0;
-    virtual void save(QSqlDatabase* db) = 0;
+    virtual void load() = 0;
+    virtual void save() = 0;
 
 protected:
     QString getColumn(void) const { return column; };
@@ -575,8 +624,8 @@ public:
 
     virtual ~SimpleDBStorage() {};
 
-    virtual void load(QSqlDatabase* db);
-    virtual void save(QSqlDatabase* db);
+    virtual void load();
+    virtual void save();
 
 protected:
 
@@ -588,8 +637,8 @@ protected:
 
 class TransientStorage: virtual public Setting {
 public:
-    virtual void load(QSqlDatabase* db) { (void)db; }
-    virtual void save(QSqlDatabase* db) { (void)db; }
+    virtual void load() {  }
+    virtual void save() {  }
 };
 
 class AutoIncrementStorage: virtual public IntegerSetting, public DBStorage {
@@ -599,17 +648,23 @@ public:
         setValue(0);
     };
 
-    virtual void load(QSqlDatabase* db) { (void)db; };
-    virtual void save(QSqlDatabase* db);
+    virtual void load() { };
+    virtual void save();
 };
 
 class ButtonSetting: virtual public Setting {
     Q_OBJECT
 public:
+    ButtonSetting() : button(NULL) {}
     virtual QWidget* configWidget(ConfigurationGroup* cg, QWidget* parent,
                                   const char* widgetName=0);
+
+    virtual void setEnabled(bool b);
+
 signals:
     void pressed();
+protected:
+    MythPushButton *button;
 };
 
 class TransButtonSetting: public ButtonSetting, public TransientStorage {
@@ -618,6 +673,7 @@ public:
 };
 
 class ConfigPopupDialogWidget: public MythPopupBox {
+    Q_OBJECT
 public:
     ConfigPopupDialogWidget(MythMainWindow* parent, const char* widgetName=0):
         MythPopupBox(parent, widgetName) {};
@@ -635,12 +691,15 @@ public:
 
     virtual MythDialog* dialogWidget(MythMainWindow* parent,
                                      const char* widgetName=0);
-    int exec(QSqlDatabase* db, bool saveOnAccept=true);
+    int exec(bool saveOnAccept=true);
 
 public slots:
     void accept() { if (dialog) dialog->accept(); };
     void reject() { if (dialog) dialog->reject(); };
 
+signals:
+    void popupDone();
+
 protected:
     ConfigPopupDialogWidget* dialog;
 };
@@ -656,4 +715,157 @@ private:
     int totalSteps;
 };
 
+class TransLabelSetting: public LabelSetting, public TransientStorage {
+public:
+    TransLabelSetting() {};
+};
+
+class TransCheckBoxSetting: public CheckBoxSetting, public TransientStorage {
+public:
+    TransCheckBoxSetting() {};
+};
+
+class HostSetting: public SimpleDBStorage, virtual public Configurable {
+public:
+    HostSetting(QString name):
+        SimpleDBStorage("settings", "data") {
+        setName(name);
+    };
+
+protected:
+    virtual QString whereClause(void);
+    virtual QString setClause(void);
+};
+
+class GlobalSetting: public SimpleDBStorage, virtual public Configurable {
+public:
+    GlobalSetting(QString name):
+        SimpleDBStorage("settings", "data") {
+        setName(name);
+    };
+
+protected:
+    virtual QString whereClause(void);
+    virtual QString setClause(void);
+};
+
+class HostSlider: public SliderSetting, public HostSetting {
+  public:
+    HostSlider(const QString &name, int min, int max, int step) :
+        SliderSetting(min, max, step),
+        HostSetting(name) { }
+};
+
+class HostSpinBox: public SpinBoxSetting, public HostSetting {
+  public:
+    HostSpinBox(const QString &name, int min, int max, int step, 
+                  bool allow_single_step = false) :
+        SpinBoxSetting(min, max, step, allow_single_step),
+        HostSetting(name) { }
+};
+
+class HostCheckBox: public CheckBoxSetting, public HostSetting {
+  public:
+    HostCheckBox(const QString &name) :
+        HostSetting(name) { }
+};
+
+class HostComboBox: public ComboBoxSetting, public HostSetting {
+  public:
+    HostComboBox(const QString &name, bool rw = false) :
+        ComboBoxSetting(rw),
+        HostSetting(name) { }
+    virtual ~HostComboBox() { ; }
+};
+
+class HostRefreshRateComboBox: virtual public HostComboBox
+{
+    Q_OBJECT
+  public:
+    HostRefreshRateComboBox(const QString &name, bool rw = false) :
+        HostComboBox(name, rw) { ; }
+    virtual ~HostRefreshRateComboBox() { ; }
+  public slots:
+    virtual void ChangeResolution(const QString& resolution);
+  private:
+    static const vector<short> GetRefreshRates(const QString &resolution);
+};
+
+class HostTimeBox: public ComboBoxSetting, public HostSetting {
+  public:
+    HostTimeBox(const QString &name, const QString &defaultTime = "00:00",
+                const int interval = 1) :
+        ComboBoxSetting(false, 30 / interval),
+        HostSetting(name)
+    {
+        int hour;
+        int minute;
+        QString timeStr;
+
+        for (hour = 0; hour < 24; hour++)
+        {
+            for (minute = 0; minute < 60; minute += interval)
+            {
+                timeStr = timeStr.sprintf("%02d:%02d", hour, minute);
+                addSelection(timeStr, timeStr, timeStr == defaultTime);
+            }
+        }
+    }
+};
+
+class HostLineEdit: public LineEditSetting, public HostSetting {
+  public:
+    HostLineEdit(const QString &name, bool rw = true) :
+        LineEditSetting(rw),
+        HostSetting(name) { }
+};
+
+class HostImageSelect: public ImageSelectSetting, public HostSetting {
+  public:
+    HostImageSelect(const QString &name) :
+        HostSetting(name) { }
+};
+
+class GlobalSlider: public SliderSetting, public GlobalSetting {
+  public:
+    GlobalSlider(const QString &name, int min, int max, int step) :
+        SliderSetting(min, max, step),
+        GlobalSetting(name) { }
+};
+
+class GlobalSpinBox: public SpinBoxSetting, public GlobalSetting {
+  public:
+    GlobalSpinBox(const QString &name, int min, int max, int step,
+                   bool allow_single_step = false) :
+        SpinBoxSetting(min, max, step, allow_single_step),
+        GlobalSetting(name) { }
+};
+
+class GlobalCheckBox: public CheckBoxSetting, public GlobalSetting {
+  public:
+    GlobalCheckBox(const QString &name) :
+        GlobalSetting(name) { }
+};
+
+class GlobalComboBox: public ComboBoxSetting, public GlobalSetting {
+  public:
+    GlobalComboBox(const QString &name, bool rw = false) :
+        ComboBoxSetting(rw),
+        GlobalSetting(name) { }
+};
+
+class GlobalLineEdit: public LineEditSetting, public GlobalSetting {
+  public:
+    GlobalLineEdit(const QString &name, bool rw = true) :
+        LineEditSetting(rw),
+        GlobalSetting(name) { }
+};
+
+class GlobalImageSelect: public ImageSelectSetting, public GlobalSetting {
+  public:
+    GlobalImageSelect(const QString &name) :
+        GlobalSetting(name) { }
+};
+
+
 #endif
