@@ -20,6 +20,11 @@ using namespace std;
 #include "util.h"
 #include "mythplugin.h"
 #include "dialogbox.h"
+#include "lcddevice.h"
+
+
+
+extern QMap<QString, fontProp> globalFontMap;
 
 struct TextAttributes
 {
@@ -51,6 +56,7 @@ struct ThemedButton
 {
     QPoint pos;
     QRect  posRect;
+    QRect  paintRect;
 
     ButtonIcon *buttonicon;
     QPoint iconPos;
@@ -106,6 +112,8 @@ class ThemedMenuPrivate
     void parseText(TextAttributes &attributes, QDomElement &element);
     void parseOutline(TextAttributes &attributes, QDomElement &element);
     void parseShadow(TextAttributes &attributes, QDomElement &element);
+    
+    void parseFont(QDomElement &element);
 
     void setDefaults(void);
 
@@ -122,7 +130,7 @@ class ThemedMenuPrivate
     QString getFirstText(QDomElement &element);
     QPoint parsePoint(const QString &text);
     QRect parseRect(const QString &text);
-
+    
     QRect menuRect() const;
 
     void paintLogo(QPainter *p);
@@ -151,8 +159,6 @@ class ThemedMenuPrivate
     int screenwidth;
     int screenheight;
     
-    QString prefix;
-
     QRect buttonArea;
 
     QRect logoRect;
@@ -175,7 +181,7 @@ class ThemedMenuPrivate
 
     QString selection;
     bool foundtheme;
-
+    bool balancerows;
     int menulevel;
     vector<MenuState> menufiles;
 
@@ -211,11 +217,14 @@ class ThemedMenuPrivate
     QPoint watermarkPos;
     QRect watermarkRect;
 
-    LCD *lcddev;
-
     bool allowreorder;
+    int maxColumns;
+    QString fontSizeType;
+    static bool parseFonts;
 };
 
+bool ThemedMenuPrivate::parseFonts = true;
+
 ThemedMenuPrivate::ThemedMenuPrivate(ThemedMenu *lparent, float lwmult, 
                                      float lhmult, int lscreenwidth, 
                                      int lscreenheight)
@@ -225,7 +234,8 @@ ThemedMenuPrivate::ThemedMenuPrivate(ThemedMenu *lparent, float lwmult,
     hmult = lhmult;
     screenwidth = lscreenwidth;
     screenheight = lscreenheight;
-
+    if (parseFonts)
+        fontSizeType = gContext->GetSetting("ThemeFontSizeType", "default");
     allowreorder = true;
 
     logo = NULL;
@@ -235,8 +245,6 @@ ThemedMenuPrivate::ThemedMenuPrivate(ThemedMenu *lparent, float lwmult,
     downarrow = NULL;
 
     ignorekeys = false;
-
-    lcddev = gContext->GetLCDDevice();
 }
 
 ThemedMenuPrivate::~ThemedMenuPrivate()
@@ -275,7 +283,7 @@ QString ThemedMenuPrivate::getFirstText(QDomElement &element)
     }
     return "";
 }
-	
+
 void ThemedMenuPrivate::parseBackground(const QString &dir, 
                                         QDomElement &element)
 {
@@ -286,7 +294,8 @@ void ThemedMenuPrivate::parseBackground(const QString &dir,
     bool hasarea = false;
 
     spreadbuttons = true;
-    visiblerowlimit = 6;	// the old default
+    maxColumns = 20;        // Arbitrary number
+    visiblerowlimit = 6;    // the old default
 
     QString type = element.attribute("style", "");
     if (type == "tiled")
@@ -321,13 +330,25 @@ void ThemedMenuPrivate::parseBackground(const QString &dir,
                 if (val == "no")
                     spreadbuttons = false;
             }
+            else if (info.tagName() == "balancerows")
+            {
+                QString val = getFirstText(info);
+                if (val == "no")
+                    balancerows = false;
+            }
+            else if (info.tagName() == "columns")
+            {
+                QString val = getFirstText(info);
+                maxColumns = val.toInt();
+            }
             else if (info.tagName() == "visiblerowlimit")
             {
                 visiblerowlimit = atoi(getFirstText(info).ascii());
             }
             else
             {
-                cerr << "Unknown tag " << info.tagName() << " in background\n";
+                VERBOSE(VB_GENERAL, QString("ThemedMenuPrivate: Unknown tag %1 in "
+                                            "background").arg(info.tagName()));
                 return;
             }
         }
@@ -335,7 +356,7 @@ void ThemedMenuPrivate::parseBackground(const QString &dir,
 
     if (!hasarea)
     {
-        cerr << "Missing buttonaread in background\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing buttonaread in background");
         return;
     }
 
@@ -397,7 +418,8 @@ void ThemedMenuPrivate::parseShadow(TextAttributes &attributes,
             }
             else
             {
-                cerr << "Unknown tag " << info.tagName() << " in text/shadow\n";
+                VERBOSE(VB_GENERAL, QString("ThemedMenuPrivate: Unknown tag %1 in "
+                                            "text/shadow").arg(info.tagName()));
                 return;
             }
         }
@@ -405,23 +427,268 @@ void ThemedMenuPrivate::parseShadow(TextAttributes &attributes,
 
     if (!hascolor)
     {
-        cerr << "Missing color tag in shadow\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing color tag in shadow");
         return;
     }
 
     if (!hasalpha)
     {
-        cerr << "Missing alpha tag in shadow\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing alpha tag in shadow");
         return;
     }
 
     if (!hasoffset)
     {
-        cerr << "Missing offset tag in shadow\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing offset tag in shadow");
         return;
     }
 }
 
+
+void ThemedMenuPrivate::parseFont(QDomElement &element)
+{
+    QString name;
+    QString face;
+    QString bold;
+    QString ital;
+    QString under;
+    
+    int size = -1;
+    int sizeSmall = -1;
+    int sizeBig = -1;
+    
+    QPoint shadowOffset = QPoint(0, 0);
+    QString color = "#ffffff";
+    QString dropcolor = "#000000";
+    QString hint;
+    QFont::StyleHint styleHint = QFont::AnyStyle;
+    
+    bool haveSizeSmall = false;
+    bool haveSizeBig = false;
+    bool haveSize = false;
+    bool haveFace = false;
+    bool haveColor = false;
+    bool haveDropColor = false;
+    bool haveBold = false;
+    bool haveShadow = false;
+    bool haveItal = false;
+    bool haveUnder = false;
+    
+    
+    
+    fontProp *baseFont = NULL;
+    
+    
+    
+    name = element.attribute("name", "");
+    if (name.isNull() || name.isEmpty())
+    {
+        VERBOSE(VB_IMPORTANT, "Font needs a name");
+        return;
+    }
+
+    
+
+    QString base =  element.attribute("base", "");
+    if (!base.isNull() && !base.isEmpty())
+    {
+        if (globalFontMap.contains(base))
+            baseFont = &globalFontMap[base];
+        
+        if (!baseFont)
+        {
+            VERBOSE(VB_IMPORTANT,
+                    QString("ThemedMenuPrivate: Specified base font '%1' does not "
+                            "exist for font %2").arg(base).arg(face));
+            return;
+        }
+    }
+    
+    
+    
+    face = element.attribute("face", "");
+    if (face.isNull() || face.isEmpty())
+    {
+        if (!baseFont)
+        {
+            VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Font needs a face");
+            return;
+        }
+    }
+    else
+    {
+        haveFace = true;
+    }
+    
+    hint = element.attribute("stylehint", "");
+    if (!hint.isNull() && !hint.isEmpty())
+    {
+        styleHint = (QFont::StyleHint)hint.toInt();
+    }
+    
+    for (QDomNode child = element.firstChild(); !child.isNull();
+         child = child.nextSibling())
+    {
+        QDomElement info = child.toElement();
+        if (!info.isNull())
+        {
+            if (info.tagName() == "size")
+            {
+                haveSize = true;
+                size = getFirstText(info).toInt();
+            }
+            else if (info.tagName() == "size:small")
+            {
+                haveSizeSmall = true;
+                sizeSmall = getFirstText(info).toInt();
+            }
+            else if (info.tagName() == "size:big")
+            {
+                haveSizeBig = true;
+                sizeBig = getFirstText(info).toInt();
+            }
+            else if (info.tagName() == "color")
+            {
+                haveColor = true;
+                color = getFirstText(info);
+            }
+            else if (info.tagName() == "dropcolor")
+            {
+                haveDropColor = true;
+                dropcolor = getFirstText(info);
+            }
+            else if (info.tagName() == "shadow")
+            {
+                haveShadow = true;
+                shadowOffset = parsePoint(getFirstText(info));
+                shadowOffset.setX((int)(shadowOffset.x() * wmult));
+                shadowOffset.setY((int)(shadowOffset.y() * hmult));
+            }
+            else if (info.tagName() == "bold")
+            {
+                haveBold = true;
+                bold = getFirstText(info);
+            }
+            else if (info.tagName() == "italics")
+            {
+                haveItal = true;
+                ital = getFirstText(info);
+            }
+            else if (info.tagName() == "underline")
+            {
+                haveUnder = true;
+                under = getFirstText(info);
+            }
+            else
+            {
+                VERBOSE(VB_IMPORTANT, QString("ThemedMenuPrivate: Unknown tag %1 in "
+                                              "font").arg(info.tagName()));
+                return;
+            }
+        }
+    }
+    
+    if (globalFontMap.contains(name))
+    {
+        VERBOSE(VB_IMPORTANT, QString("ThemedMenuPrivate: Error, already have a "
+                                      "global font called: %1").arg(name));
+        return;
+    }
+    
+    fontProp newFont;
+    
+    if(baseFont)
+        newFont = *baseFont;
+
+    if ( haveSizeSmall && fontSizeType == "small")
+    {
+        if (sizeSmall > 0)
+            size = sizeSmall;
+    }
+    else if( haveSizeBig && fontSizeType == "big")
+    {
+        if (sizeBig > 0)
+            size = sizeBig;
+    }
+
+    if (size < 0 && !baseFont)
+    {
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Error, font size must be > 0");
+        return;
+    }
+
+    
+    
+    if (baseFont && !haveSize)
+        size = baseFont->face.pointSize();
+    else    
+        size = (int)ceil(size * hmult);
+    
+    // If we don't have to, don't load the font.
+    if (!haveFace && baseFont)
+    {
+        newFont.face = baseFont->face;
+        if(haveSize)
+            newFont.face.setPointSize(size);
+    }
+    else
+    {
+        QFont temp(face, size);
+        temp.setStyleHint(styleHint);
+        newFont.face = temp;
+    }
+    
+    if (baseFont && !haveBold)
+        newFont.face.setBold(baseFont->face.bold());
+    else        
+    {
+        if (bold.lower() == "yes")
+            newFont.face.setBold(true);
+        else
+            newFont.face.setBold(false);
+    }
+    
+    if (baseFont && !haveItal)
+        newFont.face.setItalic(baseFont->face.italic());
+    else        
+    {
+        if (ital.lower() == "yes")
+            newFont.face.setItalic(true);
+        else
+            newFont.face.setItalic(false);
+    }
+
+    if (baseFont && !haveUnder)
+        newFont.face.setUnderline(baseFont->face.underline());
+    else        
+    {
+        if (under.lower() == "yes")
+            newFont.face.setUnderline(true);
+        else
+            newFont.face.setUnderline(false);
+    }    
+    
+    if (haveColor)
+    {
+        QColor foreColor(color);
+        newFont.color = foreColor; 
+    }
+    
+    if (haveDropColor)
+    {
+        QColor dropColor(dropcolor);
+        newFont.dropColor = dropColor;
+    }
+    
+    if (haveShadow)
+        newFont.shadowOffset = shadowOffset;
+    
+    globalFontMap[name] = newFont;
+}
+
+
+
+
 void ThemedMenuPrivate::parseOutline(TextAttributes &attributes, 
                                      QDomElement &element)
 {
@@ -449,7 +716,8 @@ void ThemedMenuPrivate::parseOutline(TextAttributes &attributes,
             }
             else
             {
-                cerr << "Unknown tag " << info.tagName() << " in text/shadow\n";
+                VERBOSE(VB_GENERAL, QString("ThemedMenuPrivate: Unknown tag %1 in "
+                                            "text/shadow").arg(info.tagName()));
                 return;
             }
         }
@@ -457,13 +725,13 @@ void ThemedMenuPrivate::parseOutline(TextAttributes &attributes,
 
     if (!hassize)
     {
-        cerr << "Missing size in outline\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing size in outline");
         return;
     }
 
     if (!hascolor)
     {
-        cerr << "Missing color in outline\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing color in outline");
         return;
     }
 }
@@ -553,7 +821,8 @@ void ThemedMenuPrivate::parseText(TextAttributes &attributes,
             }
             else
             {
-                cerr << "Unknown tag " << info.tagName() << " in text\n";
+                VERBOSE(VB_GENERAL, QString("ThemedMenuPrivate: Unknown tag %1 "
+                                            "in text").arg(info.tagName()));
                 return;
             }
         }
@@ -564,7 +833,8 @@ void ThemedMenuPrivate::parseText(TextAttributes &attributes,
 
     if (!hasarea)
     {
-        cerr << "Missing 'area' tag in 'text' element of 'genericbutton'\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing 'area' "
+                "tag in 'text' element of 'genericbutton'");
         return;
     }
 }
@@ -602,8 +872,8 @@ void ThemedMenuPrivate::parseButtonDefinition(const QString &dir,
             {
                 if (!hasnormal)
                 {
-                    cerr << "The 'normal' tag needs to come before the "
-                         << "'normaltext' tag\n";
+                    VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: The 'normal' "
+                            "tag needs to come before the 'normaltext' tag");
                     return;
                 }
                 parseText(normalAttributes, info);
@@ -612,8 +882,8 @@ void ThemedMenuPrivate::parseButtonDefinition(const QString &dir,
             {
                 if (!hasactive)
                 {
-                    cerr << "The 'active' tag needs to come before the "
-                         << "'activetext' tag\n";
+                    VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: The 'active' "
+                            "tag needs to come before the 'activetext' tag");
                     return;
                 }
                 parseText(activeAttributes, info);
@@ -625,8 +895,9 @@ void ThemedMenuPrivate::parseButtonDefinition(const QString &dir,
             }
             else
             {
-                cerr << "Unknown tag " << info.tagName() 
-                     << " in genericbutton\n";
+                VERBOSE(VB_GENERAL,
+                        QString("ThemedMenuPrivate: Unknown tag %1 in "
+                                "genericbutton").arg(info.tagName()));
                 return;
             }
         }
@@ -634,13 +905,13 @@ void ThemedMenuPrivate::parseButtonDefinition(const QString &dir,
 
     if (!hasnormal)
     {
-        cerr << "No normal button image defined\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: No normal button image defined");
         return;
     }
 
     if (!hasactive)
     {
-        cerr << "No active button image defined\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: No active button image defined");
         return;
     }
 
@@ -678,11 +949,12 @@ void ThemedMenuPrivate::parseLogo(const QString &dir, QDomElement &element)
             else if (info.tagName() == "position")
             {
                 logopos = parsePoint(getFirstText(info));
-		hasposition = true;
-	    }
+                hasposition = true;
+            }
             else
             {
-                cerr << "Unknown tag " << info.tagName() << " in logo\n";
+                VERBOSE(VB_GENERAL, QString("ThemedMenuPrivate: Unknown tag %1 "
+                                            "in logo").arg(info.tagName()));
                 return;
             }
         }
@@ -690,13 +962,13 @@ void ThemedMenuPrivate::parseLogo(const QString &dir, QDomElement &element)
 
     if (!hasimage)
     {
-        cerr << "Missing image tag in logo\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing image tag in logo");
         return;
     }
 
     if (!hasposition)
     {
-        cerr << "Missing position tag in logo\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing position tag in logo");
         return;
     }
 
@@ -732,7 +1004,8 @@ void ThemedMenuPrivate::parseTitle(const QString &dir, QDomElement &element)
                 }
                 else
                 {
-                    cerr << "Missing mode in titles/image\n";
+                    VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: "
+                            "Missing mode in titles/image");
                     return;
                 }
 
@@ -747,7 +1020,8 @@ void ThemedMenuPrivate::parseTitle(const QString &dir, QDomElement &element)
             }
             else
             {
-                cerr << "Unknown tag " << info.tagName() << " in logo\n";
+                VERBOSE(VB_GENERAL, QString("ThemedMenuPrivate: Unknown tag %1 "
+                                            "in logo").arg(info.tagName()));
                 return;
             }
         }
@@ -755,13 +1029,13 @@ void ThemedMenuPrivate::parseTitle(const QString &dir, QDomElement &element)
 
     if (!hasimage)
     {
-        cerr << "Missing image tag in titles\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing image tag in titles");
         return;
     }
 
     if (!hasposition)
     {
-        cerr << "Missing position tag in titles\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing position tag in titles");
         return;
     }
 
@@ -799,7 +1073,8 @@ void ThemedMenuPrivate::parseArrow(const QString &dir, QDomElement &element,
             }
             else
             {
-                cerr << "Unknown tag " << info.tagName() << " in arrow\n";
+                VERBOSE(VB_GENERAL, QString("ThemedMenuPrivate: Unknown tag %1 "
+                                            "in arrow").arg(info.tagName()));
                 return;
             }
         }
@@ -807,13 +1082,13 @@ void ThemedMenuPrivate::parseArrow(const QString &dir, QDomElement &element,
 
     if (!hasimage)
     {
-        cerr << "Missing image tag in arrow\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing image tag in arrow");
         return;
     }
 
     if (!hasposition)
     {
-        cerr << "Missing position tag in arrow\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing position tag in arrow");
         return;
     }
 
@@ -882,7 +1157,8 @@ void ThemedMenuPrivate::parseButton(const QString &dir, QDomElement &element)
             }    
             else
             {
-                cerr << "Unknown tag " << info.tagName() << " in buttondef\n";
+                VERBOSE(VB_GENERAL, QString("ThemedMenuPrivate: Unknown tag %1 "
+                                            "in buttondef").arg(info.tagName()));
                 return;
             }
         }
@@ -890,19 +1166,21 @@ void ThemedMenuPrivate::parseButton(const QString &dir, QDomElement &element)
 
     if (!hasname)
     {
-        cerr << "Missing name in button\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing name in button");
         return;
     }
 
     if (!hasoffset)
     {
-        cerr << "Missing offset in buttondef " << name << endl;
+        VERBOSE(VB_IMPORTANT, QString("ThemedMenuPrivate: Missing offset "
+                                      "in buttondef %1").arg(name));
         return;
     }
 
     if (!hasicon) 
     {
-        cerr << "Missing image in buttondef " << name << endl;
+        VERBOSE(VB_IMPORTANT, QString("ThemedMenuPrivate: Missing image "
+                                      "in buttondef %1").arg(name));
         return;
     }
 
@@ -930,7 +1208,7 @@ void ThemedMenuPrivate::setDefaults(void)
 {
     logo = NULL;
     buttonnormal = buttonactive = NULL;
-
+    balancerows = true;
     normalAttributes.textflags = Qt::AlignTop | Qt::AlignLeft | Qt::WordBreak;
     normalAttributes.textColor = QColor(QWidget::white);
     normalAttributes.hasoutline = false;
@@ -961,7 +1239,8 @@ void ThemedMenuPrivate::parseSettings(const QString &dir,
 
     if (!f.open(IO_ReadOnly))
     {
-        cerr << "ThemedMenuPrivate::parseSettings(): Can't open: " << filename << endl;
+        VERBOSE(VB_IMPORTANT, QString("ThemedMenuPrivate::parseSettings(): "
+                                      "Can't open: %1").arg(filename));
         return;
     }
 
@@ -971,9 +1250,9 @@ void ThemedMenuPrivate::parseSettings(const QString &dir,
 
     if (!doc.setContent(&f, false, &errorMsg, &errorLine, &errorColumn))
     {
-        cerr << "Error parsing: " << filename << endl;
-        cerr << "at line: " << errorLine << "  column: " << errorColumn << endl;
-        cerr << errorMsg << endl;
+        VERBOSE(VB_IMPORTANT, QString("ThemedMenuPrivate: Error, parsing %1\n"
+                                      "at line: %2  column: %3 msg: %4").
+                arg(filename).arg(errorLine).arg(errorColumn).arg(errorMsg));
         f.close();
         MythPopupBox::showOkPopup(gContext->GetMainWindow(), 
                                   QObject::tr("Bad Menu File"),
@@ -1030,9 +1309,15 @@ void ThemedMenuPrivate::parseSettings(const QString &dir,
             {
                 parseArrow(dir, e, false);
             }
+            else if (e.tagName() == "font")
+            {
+                if (parseFonts)
+                    parseFont(e);
+            }
             else
             {
-                cerr << "Unknown element " << e.tagName() << endl;
+                VERBOSE(VB_GENERAL, QString("ThemedMenuPrivate: Unknown "
+                                            "element %1").arg(e.tagName()));
                 return;
             }
         }
@@ -1041,16 +1326,18 @@ void ThemedMenuPrivate::parseSettings(const QString &dir,
 
     if (!setbackground)
     {
-        cerr << "Missing background element\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing background element");
         return;
     }
 
     if (!setbuttondef)
     {
-        cerr << "Missing genericbutton definition\n";
+        VERBOSE(VB_IMPORTANT,
+                "ThemedMenuPrivate: Missing genericbutton definition");
         return;
     }
 
+    parseFonts = false;
     return;
 }
 
@@ -1117,7 +1404,8 @@ void ThemedMenuPrivate::parseThemeButton(QDomElement &element)
             }
             else
             {
-                cerr << "Unknown tag " << info.tagName() << " in button\n";
+                VERBOSE(VB_GENERAL, QString("ThemedMenuPrivate: Unknown tag %1 "
+                                            "in button").arg(info.tagName()));
                 return;
             }
         }
@@ -1125,13 +1413,13 @@ void ThemedMenuPrivate::parseThemeButton(QDomElement &element)
 
     if (text == "")
     {
-        cerr << "Missing 'text' in button\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing 'text' in button");
         return;
     }
    
     if (action.empty())
     {
-        cerr << "Missing 'action' in button\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing 'action' in button");
         return;
     }
 
@@ -1148,16 +1436,19 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
 
     if (!f.open(IO_ReadOnly))
     {
-        cerr << "Couldn't read menu file " << menuname << endl;
+        VERBOSE(VB_IMPORTANT, QString("ThemedMenuPrivate: Couldn't read "
+                                      "menu file %1").arg(menuname));
         if (menuname == "mainmenu.xml" )
         {
             exit(-38);
         }
         else
         {
-            MythPopupBox::showOkPopup(gContext->GetMainWindow(), QObject::tr("No Menu File"),
-                                      QObject::tr(QString("Myth could not locate the menu file %1.\n\n"
-                                      "We will now return to the main menu.").arg(menuname)));
+            MythPopupBox::showOkPopup(
+                gContext->GetMainWindow(), QObject::tr("No Menu File"),
+                QObject::tr(QString("Myth could not locate the menu file %1.\n\n"
+                                    "We will now return to the main menu.").
+                            arg(menuname)));
             menulevel = 0;
             parseMenu("mainmenu.xml");
             return;
@@ -1170,9 +1461,9 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
 
     if (!doc.setContent(&f, false, &errorMsg, &errorLine, &errorColumn))
     {
-        cerr << "Error parsing: " << filename << endl;
-        cerr << "at line: " << errorLine << "  column: " << errorColumn << endl;
-        cerr << errorMsg << endl;
+        VERBOSE(VB_IMPORTANT,
+                QString("Error parsing: %1\nat line: %2  column: %3 msg: %4").
+                arg(filename).arg(errorLine).arg(errorColumn).arg(errorMsg));
         f.close();
 
         if (menuname == "mainmenu.xml" )
@@ -1212,7 +1503,8 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
             }
             else
             {
-                cerr << "Unknown element " << e.tagName() << endl;
+                VERBOSE(VB_IMPORTANT, QString("ThemedMenuPrivate: Unknown "
+                                              "element %1").arg(e.tagName()));
                 exit(-40);
             }
         }
@@ -1221,7 +1513,8 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
 
     if (buttonList.size() == 0)
     {
-        cerr << "No buttons for menu " << menuname << endl;
+        VERBOSE(VB_IMPORTANT, QString("ThemedMenuPrivate: No buttons "
+                                      "for menu %1").arg(menuname));
         exit(-41);
     }
 
@@ -1270,10 +1563,9 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
     {
         drawTitle = false;
     }
-
     drawInactiveButtons();
 
-    if (lcddev)
+    if (LCD::Get())
     {
         titleText = "MYTH-";
         titleText += menumode;
@@ -1286,7 +1578,8 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
 
 void ThemedMenuPrivate::updateLCD()
 {
-    if (!lcddev)
+    class LCD * lcddev = LCD::Get();
+    if (lcddev == NULL)
         return;
 
     // Build a list of the menu items
@@ -1326,6 +1619,8 @@ QRect ThemedMenuPrivate::parseRect(const QString &text)
     if (sscanf(text.data(), "%d,%d,%d,%d", &x, &y, &w, &h) == 4)
         retval = QRect(x, y, w, h);
 
+    retval = retval.normalize();
+
     return retval;
 }
 
@@ -1353,31 +1648,38 @@ void ThemedMenuPrivate::layoutButtons(void)
     int numbuttons = buttonList.size();
   
     columns = buttonArea.width() / buttonnormal->width();
+    columns = columns > maxColumns ? maxColumns : columns;
+    
     maxrows = buttonArea.height() / buttonnormal->height();
 
-    if (maxrows < 2)
+    if (maxrows < 1)
     {
-        cerr << "Must have room for at least 2 rows of buttons\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Must have "
+                "room for at least 1 row of buttons");
         exit(-43);
     }
     
     if (columns < 1)
     {
-        cerr << "Must have room for at least 1 column of buttons\n";
+        VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Must have "
+                "room for at least 1 column of buttons");
         exit(-44);
     }
 
-    // keep the rows balanced
-    if (numbuttons <= 4)
+    if (balancerows)
     {
-        if (columns > 2)
-            columns = 2;
+        // keep the rows balanced
+        if (numbuttons <= 4)
+        {
+            if (columns > 2)
+                columns = 2;
+        }
+        else
+        {
+            if (columns > 3)
+                columns = 3;
+        }    
     }
-    else
-    {
-        if (columns > 3)
-            columns = 3;
-    }    
 
     // limit it to 6 items displayed at one time
     if (columns * maxrows > visiblerowlimit)
@@ -1460,6 +1762,7 @@ void ThemedMenuPrivate::positionButtons(bool resetpos)
         for (; biter != (*menuiter).buttons.end(); biter++)
         {
             int xpos = xspacing * col + (buttonnormal->width() * (col - 1));
+            xpos = (maxColumns == 1) ? 0 : xpos;
             xpos += buttonArea.x();
 
             ThemedButton *tbutton = (*biter);
@@ -1471,7 +1774,8 @@ void ThemedMenuPrivate::positionButtons(bool resetpos)
             tbutton->posRect = QRect(tbutton->pos.x(), tbutton->pos.y(),
                                      buttonnormal->width(),
                                      buttonnormal->height());
-
+                
+                
             if (tbutton->buttonicon)
             {
                 tbutton->iconPos = tbutton->buttonicon->offset + tbutton->pos;
@@ -1479,6 +1783,12 @@ void ThemedMenuPrivate::positionButtons(bool resetpos)
                                           tbutton->iconPos.y(),
                                           tbutton->buttonicon->icon->width(),
                                           tbutton->buttonicon->icon->height());
+                
+                tbutton->paintRect = tbutton->posRect.unite(tbutton->iconRect);
+            }
+            else
+            {
+                tbutton->paintRect = tbutton->posRect;
             }
             
             col++;
@@ -1641,6 +1951,7 @@ void ThemedMenuPrivate::paintButton(unsigned int button, QPainter *p,
     else
         cr = tbutton->posRect;
 
+    
     if (!erased)
     {
         if (tbutton->status == 1 && activebutton == tbutton)
@@ -1817,6 +2128,9 @@ void ThemedMenuPrivate::drawText(QPainter *p, const QRect &rect,
 
 void ThemedMenuPrivate::ReloadTheme(void)
 {
+    
+    globalFontMap.clear();
+    parseFonts = true;
     buttonList.clear();
     buttonRows.clear();
 
@@ -1856,15 +2170,16 @@ void ThemedMenuPrivate::ReloadTheme(void)
         delete downarrow;
     downarrow = NULL;
 
+    QString themedir = gContext->GetThemeDir();
+    
     gContext->GetScreenSettings(screenwidth, wmult, screenheight, hmult);
     parent->setFixedSize(QSize(screenwidth, screenheight));
 
     parent->setFont(gContext->GetMediumFont());
-    parent->setCursor(QCursor(Qt::BlankCursor));
 
     gContext->ThemeWidget(parent);
 
-    QString themedir = gContext->GetThemeDir();
+    
     parseSettings(themedir, "theme.xml");
 
     QString file = menufiles.back().name;
@@ -1900,13 +2215,20 @@ bool ThemedMenuPrivate::keyPressHandler(QKeyEvent *e)
 
         if (action == "UP")
         { 
-            if (currentrow > 0)
-                currentrow--;
-            else if (columns == 1)
-                currentrow = buttonRows.size() - 1;
-
-            if (currentcolumn >= buttonRows[currentrow].numitems)
-                currentcolumn = buttonRows[currentrow].numitems - 1;
+            if( maxrows > 1)
+            {
+                if (currentrow > 0)
+                    currentrow--;
+                else if (columns == 1)
+                    currentrow = buttonRows.size() - 1;
+    
+                if (currentcolumn >= buttonRows[currentrow].numitems)
+                    currentcolumn = buttonRows[currentrow].numitems - 1;
+            }
+            else
+            {
+                handled = false;
+            }
         }
         else if (action == "PAGEUP")
         {
@@ -1919,16 +2241,26 @@ bool ThemedMenuPrivate::keyPressHandler(QKeyEvent *e)
         {
             if (currentcolumn > 0)
                 currentcolumn--;
+            else
+                currentcolumn = buttonRows[currentrow].numitems - 1;
+            
         }
         else if (action == "DOWN")
         {
-            if (currentrow < (int)buttonRows.size() - 1)
-                currentrow++;
-            else if (columns == 1)
-                currentrow = 0;
-
-            if (currentcolumn >= buttonRows[currentrow].numitems)
-                currentcolumn = buttonRows[currentrow].numitems - 1;
+            if( maxrows > 1)
+            {
+                if (currentrow < (int)buttonRows.size() - 1)
+                    currentrow++;
+                else if (columns == 1)
+                    currentrow = 0;
+    
+                if (currentcolumn >= buttonRows[currentrow].numitems)
+                    currentcolumn = buttonRows[currentrow].numitems - 1;
+            }
+            else
+            {
+                handled = false;
+            }
         }
         else if (action == "PAGEDOWN")
         {
@@ -1942,10 +2274,13 @@ bool ThemedMenuPrivate::keyPressHandler(QKeyEvent *e)
         {
             if (currentcolumn < buttonRows[currentrow].numitems - 1)
                 currentcolumn++;
+            else
+                currentcolumn = 0;
+            
         }
         else if (action == "SELECT")
         {
-            if (lcddev)
+            if (class LCD * lcddev = LCD::Get())
                 lcddev->stopAll();
 
             lastbutton = activebutton;
@@ -1988,15 +2323,15 @@ bool ThemedMenuPrivate::keyPressHandler(QKeyEvent *e)
 
     parent->update(watermarkRect);
     if (lastbutton)
-        parent->update(lastbutton->posRect);
-    parent->update(activebutton->posRect);
+        parent->update(lastbutton->paintRect);
+    parent->update(activebutton->paintRect);
 
     return true;
 } 
 
 QString ThemedMenuPrivate::findMenuFile(const QString &menuname)
 {
-    QString testdir = QDir::homeDirPath() + "/.mythtv/" + menuname;
+    QString testdir = MythContext::GetConfDir() + "/" + menuname;
     QFile file(testdir);
     if (file.exists())
         return testdir;
@@ -2012,7 +2347,7 @@ QString ThemedMenuPrivate::findMenuFile(const QString &menuname)
     if (file.exists())
         return testdir;
         
-    testdir = prefix + "/share/mythtv/" + menuname;
+    testdir = gContext->GetShareDir() + menuname;
     file.setName(testdir);
     if (file.exists())
         return testdir;
@@ -2057,7 +2392,8 @@ bool ThemedMenuPrivate::handleAction(const QString &action)
         else
         {
             if (cardid == -2)
-                cerr << "themedmenu.o: Tuner already locked" << endl;
+                VERBOSE(VB_IMPORTANT, QString("ThemedMenuPrivate: Card %1 is "
+                                              "already locked").arg(cardid));
             
             DialogBox *error_dialog = new DialogBox(gContext->GetMainWindow(),
                     "\n\nAll tuners are currently in use. If you want to watch "
@@ -2142,8 +2478,7 @@ bool ThemedMenuPrivate::findDepends(const QString &fileList)
         if (filename != "")
             return true;
     
-        QString newname = QString(PREFIX) + "/lib/mythtv/plugins/lib" + *it +
-                          ".so";
+        QString newname = gContext->FindPlugin(*it);
     
         QFile checkFile(newname);
         if (checkFile.exists())
@@ -2166,8 +2501,9 @@ bool ThemedMenuPrivate::checkPinCode(const QString &timestamp_setting,
 
     if (last_time_stamp.length() < 1)
     {
-        cerr << "themedmenu.o: Could not read password/pin time stamp. "
-             << "This is only an issue if it happens repeatedly. " << endl;
+        VERBOSE(VB_IMPORTANT,
+                "ThemedMenuPrivate: Could not read password/pin time stamp.\n"
+                "This is only an issue if it happens repeatedly.");
     }
     else
     {
@@ -2237,7 +2573,6 @@ void ThemedMenu::Init(const char *cdir, const char *menufile)
         return;
     }
 
-    d->prefix = gContext->GetInstallPrefix();
     d->menulevel = 0;
 
     d->callback = NULL;
@@ -2301,7 +2636,7 @@ void ThemedMenu::paintEvent(QPaintEvent *e)
 
     for (unsigned int i = 0; i < d->buttonList.size(); i++)
     {
-        if (r.intersects(d->buttonList[i].posRect))
+        if (r.intersects(d->buttonList[i].paintRect))
             d->paintButton(i, &p, e->erased());
     }
 
