@@ -19,13 +19,14 @@
 using namespace std;
 
 #include "mythcontext.h"
+#include "mythdbcon.h"
 #include "guidegrid.h"
-#include "infodialog.h"
 #include "infostructs.h"
 #include "programinfo.h"
 #include "oldsettings.h"
 #include "tv.h"
 #include "progfind.h"
+#include "proglist.h"
 #include "util.h"
 #include "remoteutil.h"
 
@@ -43,16 +44,6 @@ QString RunProgramGuide(QString startchannel, bool thread, TV *player,
     GuideGrid *gg = new GuideGrid(gContext->GetMainWindow(), startchannel, 
                                   player, allowsecondaryepg, "guidegrid");
 
-    bool switchMode = gContext->GetNumSetting("UseVideoModes", 0);
-    if (switchMode) 
-    {
-        // xrandr to gui size
-        char buf[128];
-        int screenwidth = gContext->GetNumSetting("GuiVidModeWidth", 0);
-        int screenheight = gContext->GetNumSetting("GuiVidModeHeight", 0);
-        sprintf(buf, "xrandr -s %dx%d", screenwidth, screenheight);
-        system(buf);
-    }
     gg->Show();
 
     if (thread)
@@ -88,12 +79,12 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
     m_currentStartChannel = 0;
 
     m_player = player;
-    m_db = QSqlDatabase::database();
 
     m_context = 0;
 
     fullRect = QRect(0, 0, size().width(), size().height());
     dateRect = QRect(0, 0, 0, 0);
+    jumpToChannelRect = QRect(0, 0, 0, 0);
     channelRect = QRect(0, 0, 0, 0);
     timeRect = QRect(0, 0, 0, 0);
     programRect = QRect(0, 0, 0, 0);
@@ -101,8 +92,12 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
     curInfoRect = QRect(0, 0, 0, 0);
     videoRect = QRect(0, 0, 0, 0);
 
-    MythContext::KickDatabase(m_db);
-
+    jumpToChannelEnabled = gContext->GetNumSetting("EPGEnableJumpToChannel", 0);
+    jumpToChannelActive = false;
+    jumpToChannelHasRect = false; // by default, we assume there is no specific region for jumpToChannel in the theme
+    jumpToChannelTimer = new QTimer(this);
+    connect(jumpToChannelTimer, SIGNAL(timeout()), SLOT(jumpToChannelTimeout()));
+    
     theme = new XMLParse();
     theme->SetWMult(wmult);
     theme->SetHMult(hmult);
@@ -132,7 +127,7 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
 
     selectChangesChannel = gContext->GetNumSetting("SelectChangesChannel", 0);
     selectRecThreshold = gContext->GetNumSetting("SelChangeRecThreshold", 16);
-    
+
     LayerSet *container = NULL;
     container = theme->GetSet("guide");
     if (container)
@@ -184,6 +179,10 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
     container = theme->GetSet("chanbar");
 
     int dNum = gContext->GetNumSetting("chanPerPage", 8);
+
+    if (m_player && m_player->IsRunning() && allowsecondaryepg)
+        dNum = dNum * 2 / 3 + 1;
+
     desiredDisplayChans = DISPLAY_CHANS = dNum;
     if (container)
     {
@@ -241,7 +240,7 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
 
     //int fillchannels = clock.elapsed();
     //clock.restart();
-    fillRecordInfos();
+    m_recList.FromScheduler();
     fillProgramInfos();
     //int fillprogs = clock.elapsed();
 
@@ -278,13 +277,6 @@ GuideGrid::~GuideGrid()
 
     m_channelInfos.clear();
 
-    while (!m_recList.empty())
-    {
-        ProgramInfo *p = m_recList.back();
-        delete p;
-        m_recList.pop_back();
-    }
-
     delete theme;
 }
 
@@ -357,6 +349,15 @@ void GuideGrid::keyPressEvent(QKeyEvent *e)
                 dayLeft();
             else if (action == "DAYRIGHT")
                 dayRight();
+            else if (jumpToChannelEnabled &&
+                     ((action[0] >= '0') && (action[0] <= '9')))
+                jumpToChannelDigitPress(action.toInt());
+            else if (jumpToChannelEnabled && jumpToChannelActive &&
+                     (action == "ESCAPE"))
+                jumpToChannelCancel();
+            else if (jumpToChannelEnabled && jumpToChannelActive &&
+                     (action == "DELETE"))
+                jumpToChannelDeleteLastDigit();
             else if (action == "NEXTFAV" || action == "4")
                 toggleGuideListing();
             else if (action == "6")
@@ -367,7 +368,7 @@ void GuideGrid::keyPressEvent(QKeyEvent *e)
                 escape();
             else if (action == "SELECT")
             {                         
-                if(m_player && selectChangesChannel)
+                if (m_player && selectChangesChannel)
                 {
                     // See if this show is far enough into the future that it's probable
                     // that the user wanted to schedule it to record instead of changing the channel.
@@ -387,6 +388,10 @@ void GuideGrid::keyPressEvent(QKeyEvent *e)
             }
             else if (action == "INFO")
                 editScheduled();
+            else if (action == "UPCOMING")
+                upcoming();
+            else if (action == "DETAILS")
+                details();
             else if (action == "TOGGLERECORD")
                 quickRecord();
             else if (action == "TOGGLEFAV")
@@ -468,6 +473,10 @@ void GuideGrid::parseContainer(QDomElement &element)
         timeRect = area;
     if (name.lower() == "date_info")
         dateRect = area;
+    if (name.lower() == "jumptochannel") {
+        jumpToChannelRect = area;
+        jumpToChannelHasRect = true;
+    }
     if (name.lower() == "current_info")
         curInfoRect = area;
     if (name.lower() == "current_video")
@@ -508,6 +517,8 @@ void GuideGrid::timeout()
                                   videoRect.width(), videoRect.height());
     }
 
+    fillProgramInfos();
+    repaint(programRect, false);
     repaint(curInfoRect, false);
 }
 
@@ -516,7 +527,8 @@ void GuideGrid::fillChannelInfos(int &maxchannel, bool gotostartchannel)
     m_channelInfos.clear();
 
     QString queryfav;
-    QSqlQuery query;
+   
+    MSqlQuery query(MSqlQuery::InitCon());
 
     QString queryall = "SELECT channel.channum, channel.callsign, "
                        "channel.icon, channel.chanid, favorites.favid, "
@@ -533,24 +545,27 @@ void GuideGrid::fillChannelInfos(int &maxchannel, bool gotostartchannel)
                    "channel.chanid = favorites.chanid and visible = 1 "
                    "ORDER BY " + channelOrdering + ";";
 
-        query.exec(queryfav);   
+        query.prepare(queryfav);
+        query.exec();   
 
         // If we don't have any favorites, then just show regular listings.
-        if (!query.isActive() || query.numRowsAffected() == 0)
+        if (!query.isActive() || query.size() == 0)
         {
             showFavorites = (!showFavorites);
-            query.exec(queryall);
+            query.prepare(queryall);
+            query.exec();
         }
     }
     else
     {
-        query.exec(queryall);
+        query.prepare(queryall);
+        query.exec();
     }
  
     bool set = false;
     maxchannel = 0;
     
-    if (query.isActive() && query.numRowsAffected() > 0)
+    if (query.isActive() && query.size() > 0)
     {
         while (query.next())
         {
@@ -558,12 +573,12 @@ void GuideGrid::fillChannelInfos(int &maxchannel, bool gotostartchannel)
             val.chanstr = query.value(0).toString();
             if (val.chanstr != "")
             {
-                val.callsign = query.value(1).toString();
+                val.callsign = QString::fromUtf8(query.value(1).toString());
                 val.iconpath = query.value(2).toString();
                 val.chanstr = query.value(0).toString();
                 val.chanid = query.value(3).toInt();
                 val.favid = query.value(4).toInt();
-                val.channame = query.value(5).toString();
+                val.channame = QString::fromUtf8(query.value(5).toString());
                 val.icon = NULL;
         
                 if (gotostartchannel && val.chanstr == m_startChanStr && !set)
@@ -630,18 +645,6 @@ void GuideGrid::fillTimeInfos()
     m_currentEndTime = t;
 }
 
-void GuideGrid::fillRecordInfos(void)
-{
-    while (!m_recList.empty())
-    {
-        ProgramInfo *p = m_recList.back();
-        delete p;
-        m_recList.pop_back();
-    }
-
-    RemoteGetAllPendingRecordings(m_recList);
-}
-
 void GuideGrid::fillProgramInfos(void)
 {
     LayerSet *container = NULL;
@@ -676,7 +679,7 @@ void GuideGrid::fillProgramRowInfos(unsigned int row)
     if (type)
         type->ResetRow(row);
  
-    QPtrList<ProgramInfo> *proglist;
+    ProgramList *proglist;
     ProgramInfo *program;
     ProgramInfo *proginfo = NULL;
 
@@ -695,35 +698,38 @@ void GuideGrid::fillProgramRowInfos(unsigned int row)
     if (chanNum < 0)
         chanNum = 0;
 
-    m_programs[row] = proglist = new QPtrList<ProgramInfo>;
-    m_programs[row]->setAutoDelete(true);
+    m_programs[row] = proglist = new ProgramList();
 
-    QString chanid = QString("%1").arg(m_channelInfos[chanNum].chanid);
+    QString querystr = QString(
+        "WHERE program.chanid = %1 AND program.endtime >= %2 "
+        "    AND program.starttime <= %3 AND program.manualid = 0 ")
+        .arg(m_channelInfos[chanNum].chanid)
+        .arg(m_currentStartTime.toString("yyyyMMddhhmm00"))
+        .arg(m_currentEndTime.toString("yyyyMMddhhmm00"));
 
-    char temp[16];
-    sprintf(temp, "%4d%02d%02d%02d%02d00",
-            m_currentStartTime.date().year(),
-            m_currentStartTime.date().month(),
-            m_currentStartTime.date().day(),
-            m_currentStartTime.time().hour(),
-            m_currentStartTime.time().minute());
-    QString starttime = temp;
-    sprintf(temp, "%4d%02d%02d%02d%02d00",
-            m_currentEndTime.date().year(),
-            m_currentEndTime.date().month(),
-            m_currentEndTime.date().day(),
-            m_currentEndTime.time().hour(),
-            m_currentEndTime.time().minute());
-    QString endtime = temp;
-
-    ProgramInfo::GetProgramRangeDateTime(m_db, proglist, chanid, starttime, 
-                                         endtime);
-
-    for (program = proglist->first(); program; program = proglist->next())
-            program->FillInRecordInfo(m_recList);
+    proglist->FromProgram(querystr, m_recList);
 
     QDateTime ts = m_currentStartTime;
 
+    if (type)
+    {
+        QDateTime tnow = QDateTime::currentDateTime();
+        int progPast = 0;
+        if (tnow > m_currentEndTime)
+            progPast = 100;
+        else if (tnow < m_currentStartTime)
+            progPast = 0;
+        else
+        {
+            int played = m_currentStartTime.secsTo(tnow);
+            int length = m_currentStartTime.secsTo(m_currentEndTime);
+            if (length)
+                progPast = played * 100 / length;
+        }
+
+        type->SetProgPast(progPast);
+    }
+
     program = proglist->first();
     QPtrList<ProgramInfo> unknownlist;
     bool unknown = false;
@@ -860,6 +866,8 @@ void GuideGrid::fillProgramRowInfos(unsigned int row)
                     recFlag = 5;
                     break;
                 case kFindOneRecord:
+                case kFindDailyRecord:
+                case kFindWeeklyRecord:
                     recFlag = 6;
                     break;
                 case kOverrideRecord:
@@ -901,7 +909,7 @@ void GuideGrid::customEvent(QCustomEvent *e)
 
         if (message == "SCHEDULE_CHANGE")
         {
-            fillRecordInfos();
+            m_recList.FromScheduler();
             fillProgramInfos();
             update(fullRect);
         }
@@ -917,7 +925,7 @@ void GuideGrid::paintEvent(QPaintEvent *e)
 
     if (r.intersects(infoRect))
         paintInfo(&p);
-    if (r.intersects(dateRect))
+    if (r.intersects(dateRect) && (jumpToChannelHasRect || (!jumpToChannelHasRect && !jumpToChannelActive)))
         paintDate(&p);
     if (r.intersects(channelRect))
         paintChannels(&p);
@@ -928,6 +936,16 @@ void GuideGrid::paintEvent(QPaintEvent *e)
     if (r.intersects(curInfoRect))
         paintCurrentInfo(&p);
 
+    // if jumpToChannel has its own rect, use that; otherwise use the date's rect
+    if ((jumpToChannelHasRect && r.intersects(jumpToChannelRect)) ||
+        (!jumpToChannelHasRect && r.intersects(dateRect)))
+        paintJumpToChannel(&p);
+
+    if (r.intersects(videoRect) && m_player)
+    {
+        timeCheck->changeInterval((int)(200));
+    }
+
     qApp->unlock();
 }
 
@@ -962,6 +980,49 @@ void GuideGrid::paintDate(QPainter *p)
     p->drawPixmap(dr.topLeft(), pix);
 }
 
+void GuideGrid::paintJumpToChannel(QPainter *p)
+{
+    if (!jumpToChannelEnabled || !jumpToChannelActive)
+        return;
+
+    QRect jtcr;
+    LayerSet *container = NULL;
+
+    if (jumpToChannelHasRect) {
+        jtcr = jumpToChannelRect;
+        container = theme->GetSet("jumptochannel");
+    } else {
+        jtcr = dateRect;
+        container = theme->GetSet("date_info");
+    }
+
+    QPixmap pix(jtcr.size());
+    pix.fill(this, jtcr.topLeft());
+    QPainter tmp(&pix);
+
+    if (container)
+    {
+        UITextType *type = (UITextType *)container->GetType((jumpToChannelHasRect) ? "channel" : "date");
+        if (type) {
+            type->SetText(QString::number(jumpToChannel));
+        }
+    }
+
+    if (container)
+    {
+        container->Draw(&tmp, 1, m_context);
+        container->Draw(&tmp, 2, m_context);
+        container->Draw(&tmp, 3, m_context);
+        container->Draw(&tmp, 4, m_context);
+        container->Draw(&tmp, 5, m_context);
+        container->Draw(&tmp, 6, m_context);
+        container->Draw(&tmp, 7, m_context);
+        container->Draw(&tmp, 8, m_context);
+    }
+    tmp.end();
+    p->drawPixmap(jtcr.topLeft(), pix);
+}
+
 void GuideGrid::paintCurrentInfo(QPainter *p)
 {
     QRect dr = curInfoRect;
@@ -1139,7 +1200,7 @@ void GuideGrid::paintInfo(QPainter *p)
 
     ChannelInfo *chinfo = &(m_channelInfos[chanNum]);
 
-    pginfo->ToMap(m_db, infoMap);
+    pginfo->ToMap(infoMap);
 
     LayerSet *container = theme->GetSet("program_info");
     if (container)
@@ -1191,7 +1252,7 @@ void GuideGrid::generateListings()
     if (DISPLAY_CHANS > maxchannel)
         DISPLAY_CHANS = maxchannel;
 
-    fillRecordInfos();
+    m_recList.FromScheduler();
     fillProgramInfos();
     update(fullRect);
 }
@@ -1199,7 +1260,7 @@ void GuideGrid::generateListings()
 void GuideGrid::toggleChannelFavorite()
 {
     QString thequery;
-    QSqlQuery query;
+    MSqlQuery query(MSqlQuery::InitCon());
 
     // Get current channel id, and make sure it exists...
     int chanNum = m_currentRow + m_currentStartChannel;
@@ -1216,7 +1277,8 @@ void GuideGrid::toggleChannelFavorite()
         thequery = QString("DELETE FROM favorites WHERE favid = '%1'")
                            .arg(favid);
 
-        query.exec(thequery);
+        query.prepare(thequery);
+        query.exec(); 
     }
     else
     {
@@ -1224,7 +1286,8 @@ void GuideGrid::toggleChannelFavorite()
         thequery = QString("INSERT INTO favorites (chanid) VALUES ('%1')")
                            .arg(chanid);
 
-        query.exec(thequery);
+        query.prepare(thequery);
+        query.exec(); 
     }
 
     if (showFavorites)
@@ -1360,6 +1423,7 @@ void GuideGrid::scrollLeft()
     repaint(programRect, false);
     repaint(infoRect, false);
     repaint(dateRect, false);
+    repaint(jumpToChannelRect, false);
     repaint(timeRect, false);
 }
 
@@ -1381,6 +1445,7 @@ void GuideGrid::scrollRight()
     repaint(programRect, false);
     repaint(infoRect, false);
     repaint(dateRect, false);
+    repaint(jumpToChannelRect, false);
     repaint(timeRect, false);
 }
 
@@ -1398,16 +1463,6 @@ void GuideGrid::scrollDown()
 {
     setStartChannel(m_currentStartChannel + 1);
 
-    QPtrList<ProgramInfo> *proglist = m_programs[0];
-    for (int y = 0; y < DISPLAY_CHANS - 1; y++)
-    {
-        m_programs[y] = m_programs[y + 1];
-        for (int x = 0; x < DISPLAY_TIMES; x++)
-        {
-            m_programInfos[y][x] = m_programInfos[y + 1][x];
-        }
-    } 
-    m_programs[DISPLAY_CHANS - 1] = proglist;
     fillProgramInfos();
 
     repaint(programRect, false);
@@ -1419,16 +1474,6 @@ void GuideGrid::scrollUp()
 {
     setStartChannel((int)(m_currentStartChannel) - 1);
 
-    QPtrList<ProgramInfo> *proglist = m_programs[DISPLAY_CHANS - 1];
-    for (int y = DISPLAY_CHANS - 1; y > 0; y--)
-    {
-        m_programs[y] = m_programs[y - 1];
-        for (int x = 0; x < DISPLAY_TIMES; x++)
-        {
-            m_programInfos[y][x] = m_programInfos[y - 1][x];
-        }
-    } 
-    m_programs[0] = proglist;
     fillProgramInfos();
 
     repaint(programRect, false);
@@ -1535,16 +1580,6 @@ void GuideGrid::enter()
 
     unsetCursor();
     selectState = 1;
-    bool switchMode = gContext->GetNumSetting("UseVideoModes", 0);
-    if (m_player && switchMode) 
-    {
-        // xrandr back to video size
-        char buf[128];
-        int screenwidth = gContext->GetNumSetting("TVVidModeWidth", 0);
-        int screenheight = gContext->GetNumSetting("TVVidModeHeight", 0);
-        sprintf(buf, "xrandr -s %dx%d", screenwidth, screenheight);
-        system(buf);
-    }
     accept();
 }
 
@@ -1558,16 +1593,6 @@ void GuideGrid::escape()
     }
 
     unsetCursor();
-    bool switchMode = gContext->GetNumSetting("UseVideoModes", 0);
-    if (m_player && switchMode) 
-    {
-        // xrandr back to video size
-        char buf[128];
-        int screenwidth = gContext->GetNumSetting("TVVidModeWidth", 0);
-        int screenheight = gContext->GetNumSetting("TVVidModeHeight", 0);
-        sprintf(buf, "xrandr -s %dx%d", screenwidth, screenheight);
-        system(buf);
-    }
     accept();
 }
 
@@ -1581,9 +1606,9 @@ void GuideGrid::quickRecord()
     if (pginfo->title == unknownTitle)
         return;
 
-    pginfo->ToggleRecord(m_db);
+    pginfo->ToggleRecord();
 
-    fillRecordInfos();
+    m_recList.FromScheduler();
     fillProgramInfos();
     repaint(programRect, false);
     repaint(infoRect, false);
@@ -1603,7 +1628,7 @@ void GuideGrid::editRecording()
     setFocusPolicy(QWidget::NoFocus);
 
     ProgramInfo *temppginfo = new ProgramInfo(*pginfo);
-    temppginfo->EditRecording(m_db);
+    temppginfo->EditRecording();
     delete temppginfo;
 
     setFocusPolicy(storeFocus);
@@ -1611,7 +1636,7 @@ void GuideGrid::editRecording()
     setActiveWindow();
     setFocus();
 
-    fillRecordInfos();
+    m_recList.FromScheduler();
     fillProgramInfos();
     repaint(fullRect, false);
 }
@@ -1630,7 +1655,7 @@ void GuideGrid::editScheduled()
     setFocusPolicy(QWidget::NoFocus);
 
     ProgramInfo *temppginfo = new ProgramInfo(*pginfo);
-    temppginfo->EditScheduled(m_db);
+    temppginfo->EditScheduled();
     delete temppginfo;
 
     setFocusPolicy(storeFocus);
@@ -1638,11 +1663,41 @@ void GuideGrid::editScheduled()
     setActiveWindow();
     setFocus();
 
-    fillRecordInfos();
+    m_recList.FromScheduler();
     fillProgramInfos();
+
     repaint(fullRect, false);
 }
 
+void GuideGrid::upcoming()
+{
+    ProgramInfo *pginfo = m_programInfos[m_currentRow][m_currentCol];
+
+    if (!pginfo)
+        return;
+
+    if (pginfo->title == unknownTitle)
+        return;
+
+    ProgLister *pl = new ProgLister(plTitle, pginfo->title,
+                                   gContext->GetMainWindow(), "proglist");
+    pl->exec();
+    delete pl;
+}
+
+void GuideGrid::details()
+{
+    ProgramInfo *pginfo = m_programInfos[m_currentRow][m_currentCol];
+
+    if (!pginfo)
+        return;
+
+    if (pginfo->title == unknownTitle)
+        return;
+
+    pginfo->showDetails();
+}
+
 void GuideGrid::channelUpdate(void)
 {
     if (!m_player)
@@ -1657,3 +1712,110 @@ void GuideGrid::channelUpdate(void)
     m_player->EPGChannelUpdate(m_channelInfos[chanNum].chanstr);
 }
 
+//
+// jumpToChannel - Jason Parekh <jasonparekh@gmail.com> - 06/23/2004
+//
+// The way this works is if you enter a digit, it will set a timer for 2.5 seconds and enter
+// jump to channel mode where as you enter digits, it will find the closest matching channel
+// and show that on the program guide.  Within the 2.5 seconds, if you press the button bound
+// to END, it will cancel channel jump mode and revert to the highlighted channel before entering
+// this mode.  If you let the timer expire, it will stay on the selected channel.
+//
+// The positioning for the channel display can be in either one of two ways.  To ensure compatibility
+// with themes, I made it so if it can't find a specific jumptochannel container, it will use the date's
+// and just overwrite the date whenever it is in channel jump mode.  However, if there is a jumptochannel
+// container, it will use that.  An example container is as follows:
+//
+//     <container name="jumptochannel">
+//     <area>20,20,115,25</area>
+//     <textarea name="channel" draworder="4" align="right">
+//         <font>info</font>
+//         <area>0,0,115,25</area>
+//         <cutdown>no</cutdown>
+//     </textarea>
+//     </container>
+//
+
+void GuideGrid::jumpToChannelDigitPress(int digit)
+{
+    // if this is the first digit press (coming from inactive mode)
+    if (jumpToChannelActive == false) {
+        jumpToChannelActive = true;
+        jumpToChannel = 0;
+        jumpToChannelPreviousStartChannel = m_currentStartChannel;
+        jumpToChannelPreviousRow = m_currentRow;
+    }
+    
+    // setup the timeout timer for jump mode
+    jumpToChannelTimer->stop();
+    jumpToChannelTimer->start(3500, true);
+
+    jumpToChannel = (jumpToChannel * 10) + digit;
+
+    // So it will move to the closest channels while they enter the digits
+    jumpToChannelShowSelection();
+}
+
+void GuideGrid::jumpToChannelDeleteLastDigit()
+{
+    jumpToChannel /= 10;
+    
+    if (jumpToChannel == 0) {
+        jumpToChannelCancel();
+    }
+}
+
+void GuideGrid::jumpToChannelShowSelection()
+{
+    unsigned int i;
+    
+    // Not the best method, but the channel list is small and this isn't time critical
+    // Go through until the ith channel is equal to or greater than the one we're looking for
+    for (i=0; (i < m_channelInfos.size()-1) &&
+             (m_channelInfos[i].chanstr.toInt() < jumpToChannel); i++);
+    
+    if ((i > 0) &&
+        ((m_channelInfos[i].chanstr.toInt() - jumpToChannel) > (jumpToChannel - m_channelInfos[i-1].chanstr.toInt()))) {
+        i--;
+    }
+
+    // DISPLAY_CHANS to center
+    setStartChannel(i-DISPLAY_CHANS/2);
+    m_currentRow = DISPLAY_CHANS/2;
+    
+    fillProgramInfos();
+
+    repaint(fullRect, false);
+}
+
+void GuideGrid::jumpToChannelCommit()
+{
+    jumpToChannelResetAndHide();
+}
+
+void GuideGrid::jumpToChannelCancel()
+{
+    setStartChannel(jumpToChannelPreviousStartChannel);
+    m_currentRow = jumpToChannelPreviousRow;
+    
+    fillProgramInfos();
+    
+    repaint(fullRect, false);
+    
+    jumpToChannelResetAndHide();
+}
+
+void GuideGrid::jumpToChannelResetAndHide()
+{
+    jumpToChannelActive = false;
+
+    jumpToChannelTimer->stop();
+
+    repaint(fullRect, false);
+}
+
+void GuideGrid::jumpToChannelTimeout()
+{
+    jumpToChannelCommit();
+}
+
