@@ -10,6 +10,7 @@ using namespace std;
 #include "osdtypes.h"
 #include "ttfont.h"
 #include "osdsurface.h"
+#include "osdlistbtntype.h"
 
 #include "mythcontext.h"
 
@@ -37,6 +38,8 @@ OSDSet::OSDSet(const QString &name, bool cache, int screenwidth,
     m_screenheight = screenheight;
     m_wmult = wmult;
     m_hmult = hmult;
+    m_xoffsetbase = 0;
+    m_yoffsetbase = 0;
 
     m_notimeout = false;
     m_fadetime = -1;
@@ -51,6 +54,8 @@ OSDSet::OSDSet(const QString &name, bool cache, int screenwidth,
     m_priority = 5;
     currentOSDFunctionalType = 0;
 
+    m_showwith = QRegExp(".*");
+
     allTypes = new vector<OSDType *>;
 }
 
@@ -62,6 +67,8 @@ OSDSet::OSDSet(const OSDSet &other)
     m_frameint = other.m_frameint;
     m_wmult = other.m_wmult;
     m_hmult = other.m_hmult;
+    m_xoffsetbase = other.m_xoffsetbase;
+    m_yoffsetbase = other.m_yoffsetbase;
     m_cache = other.m_cache;
     m_name = other.m_name;
     m_notimeout = other.m_notimeout;
@@ -156,6 +163,8 @@ void OSDSet::Reinit(int screenwidth, int screenheight, int xoff, int yoff,
     m_screenheight = screenheight;
     m_wmult = wmult;
     m_hmult = hmult;
+    m_xoffsetbase = xoff;
+    m_yoffsetbase = yoff;
 
     vector<OSDType *>::iterator iter = allTypes->begin();
     for (;iter != allTypes->end(); iter++)
@@ -201,6 +210,10 @@ void OSDSet::Reinit(int screenwidth, int screenheight, int xoff, int yoff,
         {
             item->Reinit(xoff, yoff, displaywidth, displayheight);
         }
+        else if (OSDListTreeType *item = dynamic_cast<OSDListTreeType*>(type))
+        {
+            item->Reinit(wchange, hchange, wmult, hmult);
+        }
         else
         {
             cerr << "Unknown conversion\n";
@@ -252,16 +265,24 @@ void OSDSet::SetText(QMap<QString, QString> &infoMap)
                 new_text = infoMap[item->Name()];
             }
             else if (new_text.contains(QRegExp("%.*%")))
+            {
                 for (; riter != infoMap.end(); riter++)
                 {
-                   full_regex = "%" + riter.key().upper() + 
-                     "(\\|([^%|]*))?" + "(\\|([^%|]*))?" + "(\\|([^%]*))?%";
-                   if (riter.data() != "")
-                       new_text.replace(QRegExp(full_regex), 
-                                        "\\2" + riter.data() + "\\4");
-                   else
-                       new_text.replace(QRegExp(full_regex), "\\6");
+                    QString key = riter.key().upper();
+                    QString data = riter.data();
+
+                    if (new_text.contains(key))
+                    {
+                        full_regex = QString("%") + key + QString("(\\|([^%|]*))?") + 
+                                     QString("(\\|([^%|]*))?") + QString("(\\|([^%]*))?%");
+                       if (riter.data() != "")
+                           new_text.replace(QRegExp(full_regex), 
+                                            QString("\\2") + data + "\\4");
+                       else
+                           new_text.replace(QRegExp(full_regex), "\\6");
+                    }
                 }
+            }
 
             if (new_text != "")
                 item->SetText(new_text);
@@ -346,7 +367,8 @@ void OSDSet::Draw(OSDSurface *surface, bool actuallydraw)
         for (; i != allTypes->end(); i++)
         {
             OSDType *type = (*i);
-            type->Draw(surface, m_fadetime, m_maxfade, m_xoff, m_yoff);
+            type->Draw(surface, m_fadetime, m_maxfade, m_xoff + m_xoffsetbase,
+                       m_yoff + m_yoffsetbase);
 
             if (m_wantsupdates)
                 m_lastupdate = (m_timeleft + 999999) / 1000000;
@@ -813,33 +835,33 @@ void OSDTypeImage::LoadImage(const QString &filename, float wmult, float hmult,
     if (scaleh > 0 && m_scaleh > 0)
         scaleh = m_scaleh;
 
-    int width = 0, height = 0;
+    int imwidth = 0, imheight = 0;
 
     if (scalew > 0) 
-        width = ((int)(scalew * wmult) / 2) * 2;
+        imwidth = ((int)(scalew * wmult) / 2) * 2;
     else
-        width = ((int)(tmpimage.width() * wmult) / 2) * 2;
+        imwidth = ((int)(tmpimage.width() * wmult) / 2) * 2;
 
     if (scaleh > 0)
-        height = ((int)(scaleh * hmult) / 2) * 2;
+        imheight = ((int)(scaleh * hmult) / 2) * 2;
     else
-        height = ((int)(tmpimage.height() * hmult) / 2) * 2;
+        imheight = ((int)(tmpimage.height() * hmult) / 2) * 2;
 
-    QImage tmp2 = tmpimage.smoothScale(width, height);
+    QImage tmp2 = tmpimage.smoothScale(imwidth, imheight);
 
     m_isvalid = true;
 
-    m_yuv = new unsigned char[width * height * 3 / 2];
+    m_yuv = new unsigned char[imwidth * imheight * 3 / 2];
     m_ybuffer = m_yuv;
-    m_ubuffer = m_yuv + (width * height);
-    m_vbuffer = m_yuv + (width * height * 5 / 4);
+    m_ubuffer = m_yuv + (imwidth * imheight);
+    m_vbuffer = m_yuv + (imwidth * imheight * 5 / 4);
 
-    m_alpha = new unsigned char[width * height];
+    m_alpha = new unsigned char[imwidth * imheight];
 
     rgb32_to_yuv420p(m_ybuffer, m_ubuffer, m_vbuffer, m_alpha, tmp2.bits(),
-                     width, height, tmp2.width());
+                     imwidth, imheight, tmp2.width());
 
-    m_imagesize = QRect(0, 0, width, height);
+    m_imagesize = QRect(0, 0, imwidth, imheight);
 }
 
 void OSDTypeImage::LoadFromQImage(const QImage &img)
@@ -858,20 +880,20 @@ void OSDTypeImage::LoadFromQImage(const QImage &img)
 
     m_isvalid = true;
 
-    int width = (img.width() / 2) * 2;
-    int height = (img.height() / 2) * 2;
+    int imwidth = (img.width() / 2) * 2;
+    int imheight = (img.height() / 2) * 2;
 
-    m_yuv = new unsigned char[width * height * 3 / 2];
+    m_yuv = new unsigned char[imwidth * imheight * 3 / 2];
     m_ybuffer = m_yuv;
-    m_ubuffer = m_yuv + (width * height);
-    m_vbuffer = m_yuv + (width * height * 5 / 4);
+    m_ubuffer = m_yuv + (imwidth * imheight);
+    m_vbuffer = m_yuv + (imwidth * imheight * 5 / 4);
 
-    m_alpha = new unsigned char[width * height];
+    m_alpha = new unsigned char[imwidth * imheight];
 
     rgb32_to_yuv420p(m_ybuffer, m_ubuffer, m_vbuffer, m_alpha, img.bits(),
-                     width, height, img.width());
+                     imwidth, imheight, img.width());
 
-    m_imagesize = QRect(0, 0, width, height);
+    m_imagesize = QRect(0, 0, imwidth, imheight);
 }
 
 void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff, 
@@ -882,13 +904,13 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
 
     unsigned char *dest, *destalpha, *src, *srcalpha;
     unsigned char *udest, *vdest, *usrc, *vsrc;
-    int alpha, iwidth, width;
+    int alpha, iwidth, drawwidth;
 
-    iwidth = width = m_imagesize.width();
-    int height = m_imagesize.height();
+    iwidth = drawwidth = m_imagesize.width();
+    int drawheight = m_imagesize.height();
 
     if (m_drawwidth >= 0)
-        width = m_drawwidth;
+        drawwidth = m_drawwidth;
 
     int ystart = m_displaypos.y();
     int xstart = m_displaypos.x();
@@ -914,15 +936,15 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
         xstart = 0;
     }
 
-    if (height + ystart > surface->height)
-        height = surface->height - ystart - 1;
-    if (width + xstart > surface->width)
-        width = surface->width - xstart - 1;
+    if (drawheight + ystart > surface->height)
+        drawheight = surface->height - ystart - 1;
+    if (drawwidth + xstart > surface->width)
+        drawwidth = surface->width - xstart - 1;
 
-    if (width == 0 || height == 0)
+    if (drawwidth == 0 || drawheight == 0)
         return;
 
-    QRect destRect = QRect(xstart, ystart, width, height);
+    QRect destRect = QRect(xstart, ystart, drawwidth, drawheight);
     bool needblend = false;
 
     if (m_onlyusefirst || surface->IntersectsDrawn(destRect))
@@ -942,17 +964,17 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
 
     if (!needblend)
     {
-        for (int y = startline; y < height; y++)
+        for (int y = startline; y < drawheight; y++)
         {
             ysrcwidth = y * iwidth;
             ydestwidth = (y + ystart - startline) * surface->width;
 
             memcpy(surface->y + xstart + ydestwidth,
-                   m_ybuffer + startcol + ysrcwidth, width);
+                   m_ybuffer + startcol + ysrcwidth, drawwidth);
 
             destalpha = surface->alpha + xstart + ydestwidth;
 
-            for (int x = startcol; x < width; x++)
+            for (int x = startcol; x < drawwidth; x++)
             {
                 alpha = *(m_alpha + x + ysrcwidth);
   
@@ -966,8 +988,8 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
         }
 
         iwidth /= 2;
-        width /= 2;
-        height /= 2;
+        drawwidth /= 2;
+        drawheight /= 2;
 
         ystart /= 2;
         xstart /= 2;
@@ -975,15 +997,15 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
         startline /= 2;
         startcol /= 2;
 
-        for (int y = startline; y < height; y++)
+        for (int y = startline; y < drawheight; y++)
         {
             uvsrcwidth = y * iwidth;
             uvdestwidth = (y + ystart - startline) * (surface->width / 2);
 
             memcpy(surface->u + xstart + uvdestwidth,
-                   m_ubuffer + startcol + uvsrcwidth, width);
+                   m_ubuffer + startcol + uvsrcwidth, drawwidth);
             memcpy(surface->v + xstart + uvdestwidth,
-                   m_vbuffer + startcol + uvsrcwidth, width);
+                   m_vbuffer + startcol + uvsrcwidth, drawwidth);
         }
 
         return;
@@ -1014,13 +1036,15 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
     if (m_onlyusefirst)
         (surface->blendcolumnfunc) (src, usrc, vsrc, srcalpha, iwidth, dest,
                                     udest, vdest, destalpha, surface->width,
-                                    width - startcol, height - startline,
+                                    drawwidth - startcol, 
+                                    drawheight - startline,
                                     alphamod, 1, surface->rec_lut,
                                     surface->pow_lut);
     else
         (surface->blendregionfunc) (src, usrc, vsrc, srcalpha, iwidth, dest,
                                     udest, vdest, destalpha, surface->width,
-                                    width - startcol, height - startline,
+                                    drawwidth - startcol, 
+                                    drawheight - startline,
                                     alphamod, 1, surface->rec_lut,
                                     surface->pow_lut);
 }
@@ -1516,10 +1540,8 @@ void OSDTypePositionRectangle::Reinit(float wchange, float hchange)
         int x = (int)(tmp.x() * wchange);
         int y = (int)(tmp.y() * hchange);
 
-        positions[i].setX(x);
-        positions[i].setY(y);
-        positions[i].setWidth(width);
-        positions[i].setHeight(height);
+        tmp = QRect(x, y, width, height);
+        positions[i] = tmp;
     }
 }
 
@@ -1543,10 +1565,10 @@ void OSDTypePositionRectangle::Draw(OSDSurface *surface, int fade, int maxfade,
     QRect rect = positions[m_curposition];
 
     unsigned char *src;
-    int ystart = rect.top();
-    int yend = rect.bottom();
-    int xstart = rect.left();
-    int xend = rect.right();
+    int ystart = rect.top() + yoff;
+    int yend = rect.bottom() + yoff;
+    int xstart = rect.left() + xoff;
+    int xend = rect.right() + xoff;
 
     int height = yend - ystart + 1, width = xend - xstart + 1;
 
@@ -1672,6 +1694,7 @@ OSDTypeCC::OSDTypeCC(const QString &name, TTFFont *font, int xoff, int yoff,
 
     QRect rect = QRect(0, 0, 0, 0);
     m_box = new OSDTypeBox("cc_background", rect);
+    m_ccbackground = gContext->GetNumSetting("CCBackground", 0);
 }
 
 OSDTypeCC::~OSDTypeCC()
@@ -1856,10 +1879,27 @@ void OSDTypeCC::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
             if (maxy > surface->height)
                 maxy = surface->height;
 
-            QRect rect = QRect(0, 0, textlength + 4, 
-                               (m_font->Size() * 3 / 2) + 3);
-            m_box->SetRect(rect);
-            m_box->Draw(surface, 0, 0, x - 2, y - 2);
+            if (m_ccbackground && !cc->teletextmode)
+            {
+                QRect rect = QRect(0, 0, textlength + 4, 
+                                   (m_font->Size() * 3 / 2) + 3);
+                m_box->SetRect(rect);
+                m_box->Draw(surface, 0, 0, x - 2, y - 2);
+            }
+
+            m_font->setOutline(TRUE);
+
+            switch (cc->color)
+            {
+                case 1: m_font->setColor((Qt::red), kTTF_Normal); break;
+                case 2: m_font->setColor((Qt::green), kTTF_Normal); break;
+                case 3: m_font->setColor((Qt::yellow), kTTF_Normal); break;
+                case 4: m_font->setColor((Qt::blue), kTTF_Normal); break;
+                case 5: m_font->setColor((Qt::magenta), kTTF_Normal); break;
+                case 6: m_font->setColor((Qt::cyan), kTTF_Normal); break;
+                case 7: m_font->setColor((Qt::white), kTTF_Normal); break;
+                default: m_font->setColor((Qt::white), kTTF_Normal); break;
+            }
 
             m_font->DrawString(surface, x, y + 2, cc->text, maxx, maxy, 255); 
         }
