@@ -21,14 +21,14 @@ using namespace std;
 #include "dialogbox.h"
 #include "mythcontext.h"
 #include "remoteutil.h"
+#include "mythdbcon.h"
 
 ProgLister::ProgLister(ProgListType pltype, const QString &view,
-                       QSqlDatabase *ldb, MythMainWindow *parent,
+                       MythMainWindow *parent,
                        const char *name)
             : MythDialog(parent, name)
 {
     type = pltype;
-    db = ldb;
     startTime = QDateTime::currentDateTime();
     searchTime = startTime;
 
@@ -44,6 +44,8 @@ ProgLister::ProgLister(ProgListType pltype, const QString &view,
         case plTitleSearch:   searchtype = kTitleSearch;   break;
         case plKeywordSearch: searchtype = kKeywordSearch; break;
         case plPeopleSearch:  searchtype = kPeopleSearch;  break;
+        case plPowerSearch:   searchtype = kPowerSearch;   break;
+        case plSQLSearch:     searchtype = kPowerSearch;   break;
         default:              searchtype = kNoSearch;      break;
     }
 
@@ -51,6 +53,8 @@ ProgLister::ProgLister(ProgListType pltype, const QString &view,
     allowUpdates = true;
     updateAll = false;
     refillAll = false;
+    titleSort = false;
+    reverseSort = false;
 
     fullRect = QRect(0, 0, size().width(), size().height());
     viewRect = QRect(0, 0, 0, 0);
@@ -84,12 +88,21 @@ ProgLister::ProgLister(ProgListType pltype, const QString &view,
     choosePopup = NULL;
     chooseListBox = NULL;
     chooseLineEdit = NULL;
+    chooseEditButton = NULL;
     chooseOkButton = NULL;
     chooseDeleteButton = NULL;
     chooseRecordButton = NULL;
     chooseDay = NULL;
     chooseHour = NULL;
 
+    powerPopup = NULL;
+    powerTitleEdit = NULL;
+    powerSubtitleEdit = NULL;
+    powerDescEdit = NULL;
+    powerCatType = NULL;
+    powerCategory = NULL;
+    powerStation = NULL;
+
     curView = -1;
     fillViewList(view);
 
@@ -108,6 +121,7 @@ ProgLister::ProgLister(ProgListType pltype, const QString &view,
 
 ProgLister::~ProgLister()
 {
+    itemList.clear();
     gContext->removeListener(this);
     delete theme;
 }
@@ -154,6 +168,32 @@ void ProgLister::keyPressEvent(QKeyEvent *e)
             details();
         else if (action == "TOGGLERECORD")
             quickRecord();
+        else if (action == "1")
+        {
+            if (titleSort == true)
+            {
+                titleSort = false;
+                reverseSort = false;
+            }
+            else
+            {
+                reverseSort = !reverseSort;
+            }
+            refillAll = true;
+        }
+        else if (action == "2")
+        {
+            if (titleSort == false)
+            {
+                titleSort = true;
+                reverseSort = false;
+            }
+            else
+            {
+                reverseSort = !reverseSort;
+            }
+            refillAll = true;
+        }
         else
             handled = false;
     }
@@ -230,8 +270,10 @@ void ProgLister::updateBackground(void)
                 case plTitleSearch: value = tr("Title Search"); break;
                 case plKeywordSearch: value = tr("Keyword Search"); break;
                 case plPeopleSearch: value = tr("People Search"); break;
-                case plChannel: value = tr("Channel Search"); break;
+                case plPowerSearch: value = tr("Power Search"); break;
+                case plSQLSearch: value = tr("Power Search"); break;
                 case plCategory: value = tr("Category Search"); break;
+                case plChannel: value = tr("Channel Search"); break;
                 case plMovies: value = tr("Movie Search"); break;
                 case plTime: value = tr("Time Search"); break;
                 default: value = tr("Unknown Search"); break;
@@ -291,6 +333,16 @@ void ProgLister::cursorUp(bool page)
 
 void ProgLister::prevView(void)
 {
+    if (type == plTime)
+    {
+        searchTime = searchTime.addSecs(-3600);
+        curView = 0;
+        viewList[curView] = searchTime.toString(fullDateFormat);
+        viewTextList[curView] = viewList[curView];
+        refillAll = true;
+        return;
+    }
+
     if (viewList.count() < 2)
         return;
 
@@ -304,6 +356,15 @@ void ProgLister::prevView(void)
 
 void ProgLister::nextView(void)
 {
+    if (type == plTime)
+    {
+        searchTime = searchTime.addSecs(3600);
+        curView = 0;
+        viewList[curView] = searchTime.toString(fullDateFormat);
+        viewTextList[curView] = viewList[curView];
+        refillAll = true;
+        return;
+    }
     if (viewList.count() < 2)
         return;
 
@@ -317,7 +378,7 @@ void ProgLister::nextView(void)
 
 void ProgLister::setViewFromList(void)
 {
-    if (!choosePopup || !chooseListBox)
+    if (!choosePopup || (!chooseListBox && !chooseEditButton))
         return;
 
     int view = chooseListBox->currentItem();
@@ -333,6 +394,16 @@ void ProgLister::setViewFromList(void)
             return;
         }
     }
+    if (type == plPowerSearch)
+    {
+        view--;
+        if (view < 0)
+        {
+            if (chooseEditButton)
+                powerEdit();
+            return;
+        }
+    }
 
     choosePopup->done(0);
 
@@ -358,17 +429,25 @@ void ProgLister::chooseEditChanged(void)
 
 void ProgLister::chooseListBoxChanged(void)
 {
-    if (!chooseListBox || !chooseLineEdit)
+    if (!chooseListBox)
         return;
 
     int view = chooseListBox->currentItem() - 1;
 
-    if (view < 0)
-        chooseLineEdit->setText("");
-    else
-        chooseLineEdit->setText(viewList[view]);
+    if (chooseLineEdit)
+    {
+        if (view < 0)
+            chooseLineEdit->setText("");
+        else
+            chooseLineEdit->setText(viewList[view]);
 
-    chooseDeleteButton->setEnabled(view >= 0);
+        chooseDeleteButton->setEnabled(view >= 0);
+    }
+    else if (chooseEditButton)
+    {
+        chooseDeleteButton->setEnabled(view >= 0);
+        chooseRecordButton->setEnabled(view >= 0);
+    }
 }
 
 void ProgLister::setViewFromEdit(void)
@@ -394,22 +473,24 @@ void ProgLister::setViewFromEdit(void)
             qphrase = viewList[oldview].utf8();
             qphrase.replace("\'", "\\\'");
 
+            MSqlQuery query(MSqlQuery::InitCon());
             querystr = QString("DELETE FROM keyword "
                                "WHERE phrase = '%1' AND searchtype = '%2';")
                                .arg(qphrase).arg(searchtype);
-            QSqlQuery query;
-            query.exec(querystr);
+            query.prepare(querystr);
+            query.exec();
         }
         if (newview < 0)
         {
             qphrase = text.utf8();
             qphrase.replace("\'", "\\\'");
 
+            MSqlQuery query(MSqlQuery::InitCon());
             querystr = QString("REPLACE INTO keyword (phrase, searchtype)"
                                "VALUES('%1','%2');")
                                .arg(qphrase).arg(searchtype);
-            QSqlQuery query;
-            query.exec(querystr);
+            query.prepare(querystr);
+            query.exec();
         }
     }
 
@@ -421,12 +502,86 @@ void ProgLister::setViewFromEdit(void)
     refillAll = true;
 }
 
-void ProgLister::addSearchRecord(void)
+void ProgLister::setViewFromPowerEdit()
 {
-    if (!choosePopup || !chooseListBox || !chooseLineEdit)
+    if (!powerPopup || !choosePopup || !chooseListBox)
         return;
 
-    QString text = chooseLineEdit->text();
+    QString text = "";
+    text =     powerTitleEdit->text().replace(":","%").replace("*","%") + ":";
+    text += powerSubtitleEdit->text().replace(":","%").replace("*","%") + ":";
+    text +=     powerDescEdit->text().replace(":","%").replace("*","%") + ":";
+
+    if (powerCatType->currentItem() > 0)
+        text += typeList[powerCatType->currentItem()];
+    text += ":";
+    if (powerCategory->currentItem() > 0)
+        text += categoryList[powerCategory->currentItem()];
+    text += ":";
+    if (powerStation->currentItem() > 0)
+        text += stationList[powerStation->currentItem()];
+
+    if (text == ":::::")
+        return;
+
+    int oldview = chooseListBox->currentItem() - 1;
+    int newview = viewList.findIndex(text);
+
+    QString querystr = NULL;
+    QString qphrase = NULL;
+
+    if (newview < 0 || newview != oldview)
+    {
+        if (oldview >= 0)
+        {
+            qphrase = viewList[oldview].utf8();
+            qphrase.replace("\'", "\\\'");
+
+            MSqlQuery query(MSqlQuery::InitCon());
+            querystr = QString("DELETE FROM keyword "
+                               "WHERE phrase = '%1' AND searchtype = '%2';")
+                               .arg(qphrase).arg(searchtype);
+            query.prepare(querystr);
+            query.exec();
+        }
+        if (newview < 0)
+        {
+            qphrase = text.utf8();
+            qphrase.replace("\'", "\\\'");
+
+            MSqlQuery query(MSqlQuery::InitCon());
+            querystr = QString("REPLACE INTO keyword (phrase, searchtype)"
+                               "VALUES('%1','%2');")
+                               .arg(qphrase).arg(searchtype);
+            query.prepare(querystr);
+            query.exec();
+        }
+    }
+    powerPopup->done(0);
+
+    fillViewList(text);
+
+    curView = viewList.findIndex(text);
+
+    curItem = -1;
+    refillAll = true;
+}
+
+void ProgLister::addSearchRecord(void)
+{
+    if (!choosePopup || !chooseListBox)
+        return;
+
+    QString text = "";
+
+    if (chooseLineEdit)
+        text = chooseLineEdit->text();
+    else if (chooseEditButton)
+        text = chooseListBox->currentText();
+    else
+        return;
+
+    QString what = text;
 
     if (text.stripWhiteSpace().length() == 0)
         return;
@@ -436,11 +591,23 @@ void ProgLister::addSearchRecord(void)
         VERBOSE(VB_IMPORTANT, "Unknown search in ProgLister");
         return;
     }
+    if (searchtype == kPowerSearch)
+    {
+        if (text == "" || text == ":::::")
+            return;
+
+        what = powerStringToSQL(text);
+
+        if (what == "")
+            return;
+    }
+
 
     ScheduledRecording record;
-    record.loadBySearch(db, searchtype, text);
-    record.exec(db);
+    record.loadBySearch(searchtype, text, what);
+    record.exec();
 
+    chooseListBox->setFocus();
     setViewFromEdit();
 }
 
@@ -458,11 +625,12 @@ void ProgLister::deleteKeyword(void)
     QString qphrase = text.utf8();
     qphrase.replace("\'", "\\\'");
 
+    MSqlQuery query(MSqlQuery::InitCon());
     QString querystr = QString("DELETE FROM keyword "
                                "WHERE phrase = '%1' AND searchtype = '%2';")
                                .arg(qphrase).arg(searchtype);
-    QSqlQuery query;
-    query.exec(querystr);
+    query.prepare(querystr);
+    query.exec();
 
     chooseListBox->removeItem(view + 1);
     viewList.remove(text);
@@ -477,7 +645,8 @@ void ProgLister::deleteKeyword(void)
         view = chooseListBox->count() - 2;
 
     chooseListBox->setSelected(view + 1, true);
-    if (viewList.count() < 1)
+
+    if (viewList.count() < 1 && chooseLineEdit)
         chooseLineEdit->setFocus();
     else
         chooseListBox->setFocus();
@@ -507,16 +676,24 @@ void ProgLister::setViewFromTime(void)
 
 void ProgLister::chooseView(void)
 {
-    if (type == plChannel || type == plCategory)
+    if (type == plChannel || type == plCategory || 
+        type == plMovies || type == plNewListings)
     {
         if (viewList.count() < 2)
             return;
 
         choosePopup = new MythPopupBox(gContext->GetMainWindow(), "");
-        if (type == plChannel)
-            choosePopup->addLabel(tr("Select Channel"));
-        else if (type == plCategory)
-            choosePopup->addLabel(tr("Select Category"));
+
+        QString msg;
+        switch (type)
+        {
+            case plMovies: msg = tr("Select Rating"); break;
+            case plChannel: msg = tr("Select Channel"); break;
+            case plCategory: msg = tr("Select Category"); break;
+            case plNewListings: msg = tr("Select List"); break;
+            default: msg = tr("Select"); break;
+        }
+        choosePopup->addLabel(msg);
 
         chooseListBox = new MythListBox(choosePopup);
         chooseListBox->setScrollBar(false);
@@ -618,6 +795,78 @@ void ProgLister::chooseView(void)
             refillAll = true;
         }
     }
+    else if (type == plPowerSearch)
+    {
+        int oldView = curView;
+
+        choosePopup = new MythPopupBox(gContext->GetMainWindow(), "");
+        choosePopup->addLabel(tr("Select Search"));
+
+        chooseListBox = new MythListBox(choosePopup);
+        chooseListBox->setScrollBar(false);
+        chooseListBox->setBottomScrollBar(false);
+        chooseListBox->insertItem(tr("<New Search>"));
+        chooseListBox->insertStringList(viewTextList);
+        if (curView < 0)
+            chooseListBox->setCurrentItem(0);
+        else
+            chooseListBox->setCurrentItem(curView + 1);
+        choosePopup->addWidget(chooseListBox);
+
+        chooseEditButton = new MythPushButton(choosePopup);
+        chooseEditButton->setText(tr("Edit"));
+        choosePopup->addWidget(chooseEditButton);
+
+        chooseDeleteButton = new MythPushButton(choosePopup);
+        chooseDeleteButton->setText(tr("Delete"));
+        choosePopup->addWidget(chooseDeleteButton);
+
+        chooseRecordButton = new MythPushButton(choosePopup);
+        chooseRecordButton->setText(tr("Record"));
+        choosePopup->addWidget(chooseRecordButton);
+
+        chooseDeleteButton->setEnabled(curView >= 0);
+        chooseRecordButton->setEnabled(curView >= 0);
+
+        connect(chooseListBox, SIGNAL(accepted(int)), this,
+                               SLOT(setViewFromList()));
+        connect(chooseListBox, SIGNAL(menuButtonPressed(int)),chooseEditButton,
+                               SLOT(setFocus()));
+        connect(chooseListBox, SIGNAL(selectionChanged()), this,
+                               SLOT(chooseListBoxChanged()));
+        connect(chooseEditButton, SIGNAL(clicked()), this, 
+                                  SLOT(powerEdit()));
+        connect(chooseDeleteButton, SIGNAL(clicked()), this,
+                                    SLOT(deleteKeyword()));
+        connect(chooseRecordButton, SIGNAL(clicked()), this,
+                                    SLOT(addSearchRecord()));
+
+        if (viewList.count() < 1)
+            chooseEditButton->setFocus();
+        else
+            chooseListBox->setFocus();
+        choosePopup->ExecPopup();
+
+        delete chooseEditButton;
+        chooseEditButton = NULL;
+        delete chooseDeleteButton;
+        chooseDeleteButton = NULL;
+        delete chooseRecordButton;
+        chooseRecordButton = NULL;
+        delete chooseListBox;
+        chooseListBox = NULL;
+        delete choosePopup;
+        choosePopup = NULL;
+
+        if (viewList.count() < 1 || (oldView < 0 && curView < 0))
+            reject();
+        else if (curView < 0)
+        {
+            curView = 0;
+            curItem = -1;
+            refillAll = true;
+        }
+    }
     else if (type == plTime)
     {
         choosePopup = new MythPopupBox(gContext->GetMainWindow(), "");
@@ -668,6 +917,206 @@ void ProgLister::chooseView(void)
     }
 }
 
+void ProgLister::powerEdit()
+{
+    int view = chooseListBox->currentItem() - 1;
+    QString text = ":::::";
+
+    if (view >= 0)
+        text = viewList[view];
+
+    QStringList field = QStringList::split( ":", text, true);
+
+    if ( field.count() != 6)
+    {
+        VERBOSE(VB_IMPORTANT, QString("Error. PowerSearch %1 has %2 fields")
+                .arg(text).arg(field.count()));
+    }
+
+    powerPopup = new MythPopupBox(gContext->GetMainWindow(), "");
+    powerPopup->addLabel(tr("Edit Power Search Fields"));
+
+    powerPopup->addLabel(tr("Optional title phrase:"));
+    powerTitleEdit = new MythRemoteLineEdit(powerPopup);
+    powerPopup->addWidget(powerTitleEdit);
+
+    powerPopup->addLabel(tr("Optional subtitle phrase:"));
+    powerSubtitleEdit = new MythRemoteLineEdit(powerPopup);
+    powerPopup->addWidget(powerSubtitleEdit);
+
+    powerPopup->addLabel(tr("Optional description phrase:"));
+    powerDescEdit = new MythRemoteLineEdit(powerPopup);
+    powerPopup->addWidget(powerDescEdit);
+
+    powerCatType = new MythComboBox(false, powerPopup);
+    powerCatType->insertItem(tr("(Any Program Type)"));
+    typeList.clear();
+    typeList << "";
+    powerCatType->insertItem(tr("Movies"));
+    typeList << "movie";
+    powerCatType->insertItem(tr("Series"));
+    typeList << "series";
+    powerCatType->insertItem(tr("Show"));
+    typeList << "tvshow";
+    powerCatType->insertItem(tr("Sports"));
+    typeList << "sports";
+    powerCatType->setCurrentItem(typeList.findIndex(field[3]));
+    powerPopup->addWidget(powerCatType);
+
+    powerCategory = new MythComboBox(false, powerPopup);
+    powerCategory->insertItem(tr("(Any Category)"));
+    categoryList.clear();
+    categoryList << "";
+
+    MSqlQuery query(MSqlQuery::InitCon());
+
+    QString querystr = "SELECT category FROM program GROUP BY category;";
+    query.prepare(querystr);
+    query.exec();
+
+    if (query.isActive() && query.size())
+    {
+        while (query.next())
+        {
+            QString category = query.value(0).toString();
+            if (category <= " " || category == NULL)
+                continue;
+            category = QString::fromUtf8(query.value(0).toString());
+            powerCategory->insertItem(category);
+            categoryList << category;
+            if (category == field[4])
+                powerCategory->setCurrentItem(powerCategory->count() - 1);
+        }
+    }
+    powerPopup->addWidget(powerCategory);
+
+    powerStation = new MythComboBox(false, powerPopup);
+    powerStation->insertItem(tr("(Any Station)"));
+    stationList.clear();
+    stationList << "";
+
+    querystr = "SELECT channel.chanid, channel.channum, "
+               "channel.callsign, channel.name FROM channel "
+               "WHERE channel.visible = 1 "
+               "GROUP BY callsign "
+               "ORDER BY " + channelOrdering + ";";
+    query.prepare(querystr);
+    query.exec();
+
+    if (query.isActive() && query.numRowsAffected())
+    {
+        while (query.next())
+        {
+            QString chanid = query.value(0).toString();
+            QString channum = query.value(1).toString();
+            QString chansign = QString::fromUtf8(query.value(2).toString());
+            QString channame = QString::fromUtf8(query.value(3).toString());
+
+            QString chantext = channelFormat;
+            chantext.replace("<num>", channum)
+                .replace("<sign>", chansign)
+                .replace("<name>", channame);
+
+            viewList << chanid;
+            viewTextList << chantext;
+
+            powerStation->insertItem(chantext);
+            stationList << chansign;
+            if (chansign == field[5])
+                powerStation->setCurrentItem(powerStation->count() - 1);
+        }
+    }
+    powerPopup->addWidget(powerStation);
+
+    powerOkButton = new MythPushButton(powerPopup);
+    powerOkButton->setText(tr("OK"));
+    powerPopup->addWidget(powerOkButton);
+
+    connect(powerOkButton, SIGNAL(clicked()), this, 
+                           SLOT(setViewFromPowerEdit()));
+
+    powerTitleEdit->setText(field[0]);
+    powerSubtitleEdit->setText(field[1]);
+    powerDescEdit->setText(field[2]);
+
+    powerTitleEdit->setFocus();
+    choosePopup->done(0);
+    powerPopup->ExecPopup();
+
+    delete powerTitleEdit;
+    powerTitleEdit = NULL;
+    delete powerSubtitleEdit;
+    powerSubtitleEdit = NULL;
+    delete powerDescEdit;
+    powerDescEdit = NULL;
+
+    delete powerCatType;
+    powerCatType = NULL;
+    delete powerCategory;
+    powerCategory = NULL;
+    delete powerStation;
+    powerStation = NULL;
+
+    delete powerOkButton;
+    powerOkButton = NULL;
+    delete powerPopup;
+    powerPopup = NULL;
+}
+
+QString ProgLister::powerStringToSQL(QString &qphrase)
+{
+    QString str = "";
+
+    qphrase.replace("\"","\\\"");
+    QStringList field = QStringList::split( ":", qphrase, true);
+
+    if ( field.count() != 6)
+    {
+        VERBOSE(VB_IMPORTANT, QString("Error. PowerSearch %1 has %2 fields")
+                .arg(qphrase).arg(field.count()));
+        return str;
+    }
+    if (field[0])
+        str += QString("program.title LIKE \"\%%1\%\" ")
+                       .arg(field[0]);
+    if (field[1])
+    {
+        if (str > "")
+            str += "AND ";
+        str += QString("program.subtitle LIKE \"\%%1\%\" ")
+                       .arg(field[1]);
+    }
+    if (field[2])
+    {
+        if (str > "")
+            str += "AND ";
+        str += QString("program.description LIKE \"\%%1\%\" ")
+                       .arg(field[2]);
+    }
+    if (field[3])
+    {
+        if (str > "")
+            str += "AND ";
+        str += QString("program.category_type=\"%1\" ")
+                       .arg(field[3]);
+    }
+    if (field[4])
+    {
+        if (str > "")
+            str += "AND ";
+        str += QString("program.category=\"%1\" ")
+                       .arg(field[4]);
+    }
+    if (field[5])
+    {
+        if (str > "")
+            str += "AND ";
+        str += QString("channel.callsign=\"%1\" ")
+                       .arg(field[5]);
+    }
+    return str;
+}
+
 void ProgLister::quickRecord()
 {
     ProgramInfo *pi = itemList.at(curItem);
@@ -675,7 +1124,7 @@ void ProgLister::quickRecord()
     if (!pi)
         return;
 
-    pi->ToggleRecord(db);
+    pi->ToggleRecord();
 }
 
 void ProgLister::select()
@@ -685,7 +1134,7 @@ void ProgLister::select()
     if (!pi)
         return;
 
-    pi->EditRecording(db);
+    pi->EditRecording();
 }
 
 void ProgLister::edit()
@@ -695,7 +1144,7 @@ void ProgLister::edit()
     if (!pi)
         return;
 
-    pi->EditScheduled(db);
+    pi->EditScheduled();
 }
 
 void ProgLister::upcoming()
@@ -706,7 +1155,6 @@ void ProgLister::upcoming()
         return;
 
     ProgLister *pl = new ProgLister(plTitle, pi->title,
-                                   QSqlDatabase::database(),
                                    gContext->GetMainWindow(), "proglist");
     pl->exec();
     delete pl;
@@ -717,7 +1165,7 @@ void ProgLister::details()
     ProgramInfo *pi = itemList.at(curItem);
 
     if (pi)
-        pi->showDetails(QSqlDatabase::database());
+        pi->showDetails();
 }
 
 void ProgLister::fillViewList(const QString &view)
@@ -727,14 +1175,16 @@ void ProgLister::fillViewList(const QString &view)
 
     if (type == plChannel) // list by channel
     {
+        MSqlQuery query(MSqlQuery::InitCon()); 
         QString querystr = "SELECT channel.chanid, channel.channum, "
             "channel.callsign, channel.name FROM channel "
             "WHERE channel.visible = 1 "
             "GROUP BY channum, callsign "
             "ORDER BY " + channelOrdering + ";";
-        QSqlQuery query;
-        query.exec(querystr);
-        if (query.isActive() && query.numRowsAffected())
+        query.prepare(querystr);
+        query.exec();
+
+        if (query.isActive() && query.size())
         {
             while (query.next())
             {
@@ -757,10 +1207,12 @@ void ProgLister::fillViewList(const QString &view)
     }
     else if (type == plCategory) // list by category
     {
+        MSqlQuery query(MSqlQuery::InitCon()); 
         QString querystr = "SELECT category FROM program GROUP BY category;";
-        QSqlQuery query;
-        query.exec(querystr);
-        if (query.isActive() && query.numRowsAffected())
+        query.prepare(querystr);
+        query.exec();
+
+        if (query.isActive() && query.size())
         {
             while (query.next())
             {
@@ -776,14 +1228,16 @@ void ProgLister::fillViewList(const QString &view)
             curView = viewList.findIndex(view);
     }
     else if (type == plTitleSearch || type == plKeywordSearch ||
-             type == plPeopleSearch)
+             type == plPeopleSearch || type == plPowerSearch)
     {
+        MSqlQuery query(MSqlQuery::InitCon()); 
         QString querystr = QString("SELECT phrase FROM keyword "
                                    "WHERE searchtype = '%1';")
                                    .arg(searchtype);
-        QSqlQuery query;
-        query.exec(querystr);
-        if (query.isActive() && query.numRowsAffected())
+        query.prepare(querystr);
+        query.exec();
+
+        if (query.isActive() && query.size())
         {
             while (query.next())
             {
@@ -796,7 +1250,27 @@ void ProgLister::fillViewList(const QString &view)
             }
         }
         if (view != "")
+        {
             curView = viewList.findIndex(view);
+
+            if (curView < 0)
+            {
+                QString qphrase = view.utf8();
+                qphrase.replace("\'", "\\\'");
+
+                MSqlQuery query(MSqlQuery::InitCon()); 
+                querystr = QString("REPLACE INTO keyword (phrase, searchtype)"
+                                   "VALUES('%1','%2');")
+                                    .arg(qphrase).arg(searchtype);
+                query.prepare(querystr);
+                query.exec();
+
+                viewList << qphrase;
+                viewTextList << qphrase;
+
+                curView = viewList.count() - 1;
+            }
+        }
         else
             curView = -1;
     }
@@ -811,10 +1285,33 @@ void ProgLister::fillViewList(const QString &view)
         else
             curView = -1;
     }
-    else if (type == plNewListings || type == plMovies)
+    else if (type == plNewListings)
     {
-        viewList << "";
-        viewTextList << "";
+        viewList << "all";
+        viewTextList << tr("All");
+
+        viewList << "movies";
+        viewTextList << tr("Movies");
+
+        viewList << "series";
+        viewTextList << tr("Series");
+
+        viewList << "specials";
+        viewTextList << tr("Specials");
+        curView = 0;
+    }
+    else if (type == plMovies)
+    {
+        viewList << "0.0";
+        viewTextList << tr("All");
+        viewList << "1.0";
+        viewTextList << tr("4 stars");
+        viewList << "0.875";
+        viewTextList << tr("At least 3 1/2 stars");
+        viewList << "0.75";
+        viewTextList << tr("At least 3 stars");
+        viewList << "0.5";
+        viewTextList << tr("At least 2 stars");
         curView = 0;
     }
     else if (type == plTime)
@@ -823,11 +1320,53 @@ void ProgLister::fillViewList(const QString &view)
         viewList[curView] = searchTime.toString(fullDateFormat);
         viewTextList[curView] = viewList[curView];
     }
-
+    else if (type == plSQLSearch)
+    {
+        curView = 0;
+        viewList << view;
+        viewTextList << tr("Power Recording Rule");
+    }
     if (curView >= (int)viewList.count())
         curView = viewList.count() - 1;
 }
 
+class plTitleSort
+{
+    public:
+        plTitleSort(bool reverseSort = false) {m_reverse = reverseSort;}
+
+        bool operator()(const ProgramInfo *a, const ProgramInfo *b) 
+        {
+            if (m_reverse)
+                return (a->sortTitle > b->sortTitle);
+            else
+                return (a->sortTitle < b->sortTitle);
+        }
+
+    private:
+        bool m_reverse;
+};
+
+class plTimeSort
+{
+    public:
+        plTimeSort(bool reverseSort = false) {m_reverse = reverseSort;}
+
+        bool operator()(const ProgramInfo *a, const ProgramInfo *b) 
+        {
+            if (a->startts == b->startts)
+                return (a->chanid < b->chanid);
+
+            if (m_reverse)
+                return (a->startts > b->startts);
+            else
+                return (a->startts < b->startts);
+        }
+
+    private:
+        bool m_reverse;
+};
+
 void ProgLister::fillItemList(void)
 {
     if (curView < 0)
@@ -852,11 +1391,26 @@ void ProgLister::fillItemList(void)
                         "WHERE channel.visible = 1 "
                         "  AND program.endtime > %1 "
                         "  AND oldprogram.oldtitle IS NULL "
-                        "  AND (program.category_type <> 'movie' "
-                        "    OR program.airdate >= "
-                        "      YEAR(NOW() - INTERVAL 2 YEAR)) "
-                        "GROUP BY title ")
+                        "  AND program.manualid = 0 ")
                         .arg(startstr);
+
+        if (qphrase == "movies")
+        {
+            where += "AND program.category_type = 'movie' ";
+        }
+        else if (qphrase == "series")
+        {
+            where += "AND program.category_type = 'series' ";
+        }
+        else if (qphrase == "specials")
+        {
+            where += "AND program.category_type = 'tvshow' ";
+        }
+        else
+        {
+            where += "AND (program.category_type <> 'movie' ";
+            where += "OR program.airdate >= YEAR(NOW() - INTERVAL 2 YEAR)) ";
+        }
     }
     else if (type == plTitleSearch) // keyword search
     {
@@ -877,12 +1431,27 @@ void ProgLister::fillItemList(void)
     }
     else if (type == plPeopleSearch) // people search
     {
-        where = QString("LEFT JOIN credits ON credits.chanid = program.chanid "
-                        "  AND credits.starttime = program.starttime "
-                        "LEFT JOIN people ON people.person = credits.person "
-                        "WHERE channel.visible = 1 "
+        where = QString(", people, credits WHERE channel.visible = 1 "
                         "  AND program.endtime > %1 "
-                        "  AND people.name LIKE '\%%2\%' ")
+                        "  AND people.name LIKE '%2' "
+                        "  AND credits.person = people.person "
+                        "  AND program.chanid = credits.chanid "
+                        "  AND program.starttime = credits.starttime")
+                        .arg(startstr).arg(qphrase);
+    }
+    else if (type == plPowerSearch) // complex search
+    {
+        where = QString("WHERE channel.visible = 1 "
+                        "  AND program.endtime > %1 "
+                        "  AND ( %2 ) ")
+                        .arg(startstr).arg(powerStringToSQL(qphrase));
+    }
+    else if (type == plSQLSearch) // complex search
+    {
+        qphrase.remove(QRegExp("^\\s*AND\\s+", false));
+        where = QString("WHERE channel.visible = 1 "
+                        "  AND program.endtime > %1 "
+                        "  AND ( %2 ) ")
                         .arg(startstr).arg(qphrase);
     }
     else if (type == plChannel) // list by channel
@@ -903,18 +1472,57 @@ void ProgLister::fillItemList(void)
     {
         where = QString("WHERE channel.visible = 1 "
                         "  AND program.endtime > %1 "
-                        "  AND program.category_type = 'movie' ")
-                        .arg(startstr);
+                        "  AND program.category_type = 'movie' "
+                        "  AND program.stars >= '\%2' ")
+                        .arg(startstr).arg(qphrase);
     }
     else if (type == plTime) // list by time
     {
         where = QString("WHERE channel.visible = 1 "
                         "  AND program.starttime >= %1 ")
                         .arg(searchTime.toString("yyyyMMddhh0000"));
+        if (titleSort)
+            where += QString("  AND program.starttime < DATE_ADD(%1, "
+                             "INTERVAL 1 HOUR) ")
+                             .arg(searchTime.toString("yyyyMMddhh0000"));
     }
 
+    if (titleSort && type == plTitle)
+        where += " GROUP BY subtitle ";
+    else if (type == plNewListings || titleSort)
+        where += " GROUP BY title ";
+
     schedList.FromScheduler();
-    itemList.FromProgram(db, where, schedList);
+    itemList.FromProgram(where, schedList);
+
+    if (titleSort || reverseSort)
+    {
+        ProgramInfo *s;
+
+        vector<ProgramInfo *> sortedList;
+        while (itemList.count())
+        {
+            s = itemList.take();
+            if (type == plTitle)
+                s->sortTitle = s->subtitle;
+            else
+                s->sortTitle = s->title;
+
+            s->sortTitle.remove(QRegExp("^(The |A |An )"));
+            sortedList.push_back(s);
+        }
+
+        if (titleSort)
+            sort(sortedList.begin(), sortedList.end(),
+                 plTitleSort(reverseSort));
+        else
+            sort(sortedList.begin(), sortedList.end(),
+                 plTimeSort(reverseSort));
+
+        vector<ProgramInfo *>::iterator i = sortedList.begin();
+        for (; i != sortedList.end(); i++)
+            itemList.append(*i);
+    }
 
     if (curItem < 0 && itemList.count() > 0)
         curItem = 0;
@@ -934,9 +1542,9 @@ void ProgLister::updateView(QPainter *p)
     container = theme->GetSet("view");
     if (container)
     {  
-        UITextType *type = (UITextType *)container->GetType("curview");
-        if (type && curView >= 0)
-            type->SetText(viewTextList[curView]);
+        UITextType *uitype = (UITextType *)container->GetType("curview");
+        if (uitype && curView >= 0)
+            uitype->SetText(viewTextList[curView]);
 
         container->Draw(&tmp, 4, 0);
         container->Draw(&tmp, 5, 0);
@@ -957,7 +1565,7 @@ void ProgLister::updateList(QPainter *p)
     QPainter tmp(&pix);
 
     QString tmptitle;
-    
+
     LayerSet *container = theme->GetSet("selector");
     if (container)
     {
@@ -967,6 +1575,15 @@ void ProgLister::updateList(QPainter *p)
             ltype->ResetList();
             ltype->SetActive(true);
 
+            QStringList starMap;
+            QString starstr = "";
+            for (int i = 0; i <= 4; i++)
+            {
+                starMap << starstr;
+                starMap << starstr + "/";
+                starstr += "*";
+            }
+
             int skip;
             if ((int)itemList.count() <= listsize || curItem <= listsize/2)
                 skip = 0;
@@ -988,7 +1605,11 @@ void ProgLister::updateList(QPainter *p)
                 ltype->SetItemText(i, 1, pi->startts.toString(timeFormat));
                 ltype->SetItemText(i, 2, pi->ChannelText(channelFormat));
 
-                if (pi->subtitle == "")
+                if (pi->stars > 0.0)
+                    tmptitle = QString("%1 (%2, %3 )")
+                                       .arg(pi->title).arg(pi->year)
+                                       .arg(starMap[(int) (pi->stars * 8)]);
+                else if (pi->subtitle == "")
                     tmptitle = pi->title;
                 else
                 {
@@ -1004,8 +1625,10 @@ void ProgLister::updateList(QPainter *p)
 
                 if (pi->recstatus == rsConflict)
                     ltype->EnableForcedFont(i, "conflicting");
-                else if (pi->recstatus <= rsWillRecord)
+                else if (pi->recstatus == rsRecording)
                     ltype->EnableForcedFont(i, "recording");
+                else if (pi->recstatus == rsWillRecord)
+                    ltype->EnableForcedFont(i, "record");
 
                 if (i + skip == curItem)
                     ltype->SetItemCurrent(i);
@@ -1049,7 +1672,7 @@ void ProgLister::updateInfo(QPainter *p)
         if (container)
         {  
             QMap<QString, QString> infoMap;
-            pi->ToMap(db, infoMap);
+            pi->ToMap(infoMap);
             container->ClearAllText();
             container->SetText(infoMap);
         }
@@ -1108,4 +1731,3 @@ void ProgLister::customEvent(QCustomEvent *e)
 
     allowEvents = true;
 }
-
