@@ -6,6 +6,8 @@
 #include "sr_items.h"
 #include "sr_root.h"
 #include "sr_dialog.h"
+#include "jobqueue.h"
+#include "mythdbcon.h"
 
 #include <qlayout.h>
 #include <qlabel.h>
@@ -24,6 +26,12 @@ ScheduledRecording::ScheduledRecording()
     dupin = NULL;
     dupmethod = NULL;
     autoexpire = NULL;
+    autotranscode = NULL;
+    autocommflag = NULL;
+    autouserjob1 = NULL;
+    autouserjob2 = NULL;
+    autouserjob3 = NULL;
+    autouserjob4 = NULL;
     startoffset = NULL;
     endoffset = NULL;
     maxepisodes = NULL;
@@ -31,6 +39,7 @@ ScheduledRecording::ScheduledRecording()
     m_dialog = NULL;
     recpriority = NULL;
     recgroup = NULL;
+    inactive = NULL;
     searchType = "";
     searchForWhat = "";
         
@@ -54,17 +63,21 @@ ScheduledRecording::ScheduledRecording()
     category = new SRCategory(*this);
     seriesid = new SRSeriesid(*this);
     programid = new SRProgramid(*this);
+    findday = new SRFindDay(*this);
+    findtime = new SRFindTime(*this);
+    findid = new SRFindId(*this);
+    parentid = new SRParentId(*this);
     search = new SRRecSearchType(*this);
     
     rootGroup = new RootSRGroup(*this);
 }
 
 
-void ScheduledRecording::load(QSqlDatabase *db)
+void ScheduledRecording::load()
 {
     if (getRecordID())
     {
-        ConfigurationGroup::load(db);
+        ConfigurationGroup::load();
         
         QString tmpType = type->getValue();
         type->clearSelections();
@@ -72,45 +85,44 @@ void ScheduledRecording::load(QSqlDatabase *db)
             tmpType.toInt() == kDontRecord)
             type->addOverrideSelections();
         else
-            type->addNormalSelections(!station->getValue().isEmpty());
+            type->addNormalSelections(!station->getValue().isEmpty(),
+                                      search->intValue() == kManualSearch);
         
         type->setValue(tmpType);
         type->setUnchanged();
-        fetchChannelInfo(db);
+        fetchChannelInfo();
     }
 }
 
 
 
-void ScheduledRecording::loadByProgram(QSqlDatabase* db, ProgramInfo* proginfo) 
+void ScheduledRecording::loadByProgram(ProgramInfo* proginfo) 
 {
     m_pginfo = proginfo;
     
-    //query.exec( "SELECT count(previouslyshown) FROM program WHERE previouslyshown = 1;");
-    //if (query.isActive() && query.numRowsAffected() > 0)
-    
     if (proginfo->recordid)
-        loadByID(db, proginfo->recordid);
+        loadByID(proginfo->recordid);
     else
-        setDefault(db, true);
-    
-    if (search->intValue() == kNoSearch)
+        setDefault(true);
+
+    if (search->intValue() == kNoSearch ||
+        search->intValue() == kManualSearch)
         setProgram(proginfo);
-        
-    
 }
 
-void ScheduledRecording::loadBySearch(QSqlDatabase *db,
-                                      RecSearchType lsearch,
+void ScheduledRecording::loadBySearch(RecSearchType lsearch,
+                                      QString textname,
                                       QString forwhat)
 {
-    int rid = 0;
-    QString thequery = QString("SELECT recordid FROM record WHERE "
-                               "search = %1 AND description LIKE '%2'")
-        .arg(lsearch).arg(forwhat);
-    QSqlQuery query = db->exec(thequery);
+    MSqlQuery query(MSqlQuery::InitCon());
 
-    if (query.isActive())
+    int rid = 0;
+    query.prepare("SELECT recordid FROM record WHERE "
+                  "search = :SEARCH AND description LIKE :FORWHAT");
+    query.bindValue(":SEARCH", lsearch);
+    query.bindValue(":FORWHAT", forwhat);
+
+    if (query.exec() && query.isActive())
     {
         if (query.next())
             rid = query.value(0).toInt();
@@ -119,15 +131,18 @@ void ScheduledRecording::loadBySearch(QSqlDatabase *db,
         MythContext::DBError("loadBySearch", query);
 
     if (rid)
-        loadByID(db, rid);
+        loadByID(rid);
     else
     {
-        setDefault(db, false);
+        setDefault(false);
         search->setValue(lsearch);
         searchForWhat = forwhat;
         
         switch (lsearch)
         {
+        case kPowerSearch:
+            searchType = "(" +  QObject::tr("Power Search") + ")";
+            break;
         case kTitleSearch:
             searchType = "(" +  QObject::tr("Title Search") + ")";
             break;
@@ -141,26 +156,31 @@ void ScheduledRecording::loadBySearch(QSqlDatabase *db,
             searchType = "(" +  QObject::tr("Unknown Search") + ")";
             break;
         }
-        QString ltitle = QString("%1 %2").arg(forwhat).arg(searchType);
+        QString ltitle = QString("%1 %2").arg(textname).arg(searchType);
         title->setValue(ltitle);
         description->setValue(forwhat);
+        findday->setValue((startDate->dateValue().dayOfWeek() + 1) % 7);
     } 
 }
 
-void ScheduledRecording::fetchChannelInfo(QSqlDatabase *db)
+void ScheduledRecording::fetchChannelInfo()
 {
+    
     if (channel->getValue().toInt() > 0)
     {
-        QString queryStr(QString("SELECT channum, callsign, name FROM channel WHERE chanid = '%1';").arg(channel->getValue()));
+        MSqlQuery query(MSqlQuery::InitCon());
+
+        QString queryStr(QString("SELECT channum, callsign, name FROM channel "
+                                 "WHERE chanid = '%1';").arg(channel->getValue()));
+    
+        query.prepare(queryStr);
         
-        QSqlQuery result = db->exec(queryStr);
-        
-        if (result.isActive() && result.numRowsAffected() > 0)
+        if (query.exec() && query.isActive() && query.size() > 0)
         {
-            result.next();
-            chanstr =  result.value(0).toString();
-            chansign =  result.value(1).toString();
-            channame =  result.value(2).toString();
+            query.next();
+            chanstr =  query.value(0).toString();
+            chansign =  query.value(1).toString();
+            channame =  query.value(2).toString();
         }
     }
     else
@@ -187,10 +207,11 @@ void ScheduledRecording::ToMap(QMap<QString, QString>& progMap)
 {
     QString searchtitle = "";
 
-    if (m_pginfo && search->intValue() != kNoSearch)
+    if (m_pginfo && search->intValue() != kNoSearch &&
+                    search->intValue() != kManualSearch)
     {
         searchtitle = title->getValue();
-        m_pginfo->ToMap(NULL, progMap);
+        m_pginfo->ToMap(progMap);
     }
     else
     {
@@ -246,10 +267,25 @@ void ScheduledRecording::ToMap(QMap<QString, QString>& progMap)
                                    endts.time().toString(timeFormat);
     }
 
+    int recType = type->getValue().toInt();
+
+    if (recType == kFindDailyRecord || recType == kFindWeeklyRecord)
+    {
+        QString findfrom = findtime->timeValue().toString(timeFormat);
+        if (recType == kFindWeeklyRecord)
+        {
+            findfrom = QString("%1, %2")
+                               .arg(QDate::shortDayName(findday->intValue()-1))
+                               .arg(findfrom);
+        }
+        progMap["subtitle"] = QString("(%1 %2) %3")
+                                      .arg(findfrom).arg(tr("or later"))
+                                      .arg(progMap["subtitle"]);
+    }
+
     progMap["searchtype"] = searchType;
     progMap["searchforwhat"] = searchForWhat;
 
-        
     if (searchtitle != "")
     {
         progMap["banner"] = searchtitle;
@@ -269,9 +305,9 @@ void ScheduledRecording::ToMap(QMap<QString, QString>& progMap)
 }
 
 
-void ScheduledRecording::loadByID(QSqlDatabase* db, int recordID) {
+void ScheduledRecording::loadByID(int recordID) {
     id->setValue(recordID);
-    load(db);
+    load();
 }
 
 RecordingType ScheduledRecording::getRecordingType(void) const {
@@ -286,6 +322,20 @@ RecSearchType ScheduledRecording::getSearchType(void) const {
     return (RecSearchType)(search->intValue());
 }
 
+void ScheduledRecording::setSearchType(RecSearchType newType) {
+    if (type->getValue().toInt() == kOverrideRecord ||
+        type->getValue().toInt() == kDontRecord)
+    {
+        VERBOSE(VB_IMPORTANT, "Attempt to set search type for "
+                "override recording");
+        return;
+    }
+    search->setValue(newType);
+    type->clearSelections();
+    type->addNormalSelections(!station->getValue().isEmpty(),
+                              search->intValue() == kManualSearch);
+}
+
 bool ScheduledRecording::GetAutoExpire(void) const {
     return(autoexpire->getValue().toInt());
 }
@@ -294,6 +344,25 @@ void ScheduledRecording::SetAutoExpire(bool expire) {
     autoexpire->setValue(expire);
 }
 
+int ScheduledRecording::GetAutoRunJobs(void) const {
+    int result = 0;
+
+    if (autotranscode->getValue().toInt())
+        result |= JOB_TRANSCODE;
+    if (autocommflag->getValue().toInt())
+        result |= JOB_COMMFLAG;
+    if (autouserjob1->getValue().toInt())
+        result |= JOB_USERJOB1;
+    if (autouserjob2->getValue().toInt())
+        result |= JOB_USERJOB2;
+    if (autouserjob3->getValue().toInt())
+        result |= JOB_USERJOB3;
+    if (autouserjob4->getValue().toInt())
+        result |= JOB_USERJOB4;
+
+    return result;
+}
+
 int ScheduledRecording::GetMaxEpisodes(void) const {
     return(maxepisodes->getValue().toInt());
 }
@@ -302,105 +371,81 @@ bool ScheduledRecording::GetMaxNewest(void) const {
     return(maxnewest->getValue().toInt());
 }
 
-void ScheduledRecording::save(QSqlDatabase* db) 
+void ScheduledRecording::save() 
 {
     if (type->isChanged() && getRecordingType() == kNotRecording)
     {
-        remove(db);
+        remove();
     }
     else
     {
-        ConfigurationGroup::save(db);
+        ConfigurationGroup::save();
     }
-    signalChange(db);
+    signalChange(getRecordID());
 }
 
-void ScheduledRecording::remove(QSqlDatabase* db) {
-    if (!getRecordID())
+void ScheduledRecording::remove() 
+{
+    int rid = getRecordID();
+
+    if (!rid)
         return;
 
-    QString query = QString("DELETE FROM record WHERE recordid = %1")
-                            .arg(getRecordID());
-    db->exec(query);
-    query = QString("UPDATE recordid FROM recorded SET recordid = NULL "
-                    "WHERE recordid = %1")
-                    .arg(getRecordID());
+    QString querystr;
+    MSqlQuery query(MSqlQuery::InitCon());
+    querystr = QString("DELETE FROM record WHERE recordid = %1").arg(rid);
+    query.prepare(querystr);
+    query.exec();
+
+    querystr = QString("DELETE FROM oldfind WHERE recordid = %1").arg(rid);
+    query.prepare(querystr);
+    query.exec();
 }
 
-void ScheduledRecording::signalChange(QSqlDatabase* db) {
-    MythContext::KickDatabase(db);
-
-    QSqlQuery result = db->exec("SELECT NULL FROM settings WHERE value = \"RecordChanged\";");
-
-    QString query;
-    if (result.isActive() && result.numRowsAffected() > 0)
+void ScheduledRecording::signalChange(int recordid) 
+{
+    if (gContext->IsBackend())
     {
-        query = "UPDATE settings SET data = \"yes\" WHERE "
-                "value = \"RecordChanged\";";
+        MythEvent me(QString("RESCHEDULE_RECORDINGS %1").arg(recordid));
+        gContext->dispatch(me);
     }
     else
     {
-        query = "INSERT INTO settings (value,data) "
-                "VALUES(\"RecordChanged\", \"yes\");";
+        QStringList slist;
+        slist << QString("RESCHEDULE_RECORDINGS %1").arg(recordid);
+        if (!gContext->SendReceiveStringList(slist))
+            cerr << "error resceduling id " << recordid << 
+                " in ScheduledRecording::signalChange" << endl;
     }
-    db->exec(query);
-
 }
 
-bool ScheduledRecording::hasChanged(QSqlDatabase* db) {
-    QString thequery;
-    
-    bool retval = false;
-
-    thequery = "SELECT data FROM settings WHERE value = \"RecordChanged\";";
-    QSqlQuery query = db->exec(thequery);
-
-    if (query.isActive() && query.numRowsAffected() > 0)
-    {
-        query.next();
-
-        QString value = query.value(0).toString();
-        if (value == "yes")
-        {
-            thequery = "UPDATE settings SET data = \"no\" WHERE value = "
-                       "\"RecordChanged\";";
-            query = db->exec(thequery);
-
-            retval = true;
-        }
-    }
-
-    return retval;
-}
-
-void ScheduledRecording::doneRecording(QSqlDatabase* db, 
-                                       const ProgramInfo& proginfo) 
+void ScheduledRecording::doneRecording(const ProgramInfo& proginfo) 
 {
     if (getRecordingType() == kFindOneRecord)
-        remove(db);
+        remove();
 
     QString msg;
   
     msg = QString("Finished recording %1 on channel: %2")
-                  .arg(proginfo.title.utf8())
+                  .arg(proginfo.title)
                   .arg(proginfo.chanid);
 
-    VERBOSE(VB_GENERAL, msg);
-    gContext->LogEntry("scheduler", LP_NOTICE, "Finished recording", msg);
+    VERBOSE(VB_GENERAL, msg.local8Bit());
+    gContext->LogEntry("scheduler", LP_NOTICE, "Finished recording", 
+                       msg);
 
-    addHistory(db, proginfo);
+    addHistory(proginfo);
 }
 
-void ScheduledRecording::addHistory(QSqlDatabase* db, 
-                                    const ProgramInfo& proginfo) 
+void ScheduledRecording::addHistory(const ProgramInfo& proginfo) 
 {
-    QSqlQuery result(QString::null, db);
+    MSqlQuery result(MSqlQuery::InitCon());
 
-    result.prepare("INSERT INTO oldrecorded (chanid,starttime,"
+    result.prepare("REPLACE INTO oldrecorded (chanid,starttime,"
                    "endtime,title,subtitle,description,category,"
-                   "seriesid,programid) "
+                   "seriesid,programid,findid,recordid) "
                    "VALUES(:CHANID,:START,:END,:TITLE,:SUBTITLE,:DESC,"
-                   ":CATEGORY,:SERIESID,:PROGRAMID);");
+                   ":CATEGORY,:SERIESID,:PROGRAMID,:FINDID,:RECORDID);");
     result.bindValue(":CHANID", proginfo.chanid);
     result.bindValue(":START", proginfo.recstartts.toString(Qt::ISODate));
     result.bindValue(":END", proginfo.recendts.toString(Qt::ISODate));
@@ -408,8 +453,10 @@ void ScheduledRecording::addHistory(QSqlDatabase* db,
     result.bindValue(":SUBTITLE", proginfo.subtitle.utf8());
     result.bindValue(":DESC", proginfo.description.utf8());
     result.bindValue(":CATEGORY", proginfo.category.utf8());
-    result.bindValue(":SERIESID", proginfo.seriesid);
-    result.bindValue(":PROGRAMID", proginfo.programid);
+    result.bindValue(":SERIESID", proginfo.seriesid.utf8());
+    result.bindValue(":PROGRAMID", proginfo.programid.utf8());
+    result.bindValue(":FINDID", proginfo.findid);
+    result.bindValue(":RECORDID", proginfo.recordid);
 
     result.exec();
 
@@ -417,38 +464,61 @@ void ScheduledRecording::addHistory(QSqlDatabase* db,
     {
         MythContext::DBError("addHistory", result);
     }
-    else
+
+    if (proginfo.findid)
     {
-        // The addition of an entry to oldrecorded may affect near-future
-        // scheduling decisions, so recalculate
-        signalChange(db);
+        result.prepare("REPLACE INTO oldfind (recordid, findid) "
+                       "VALUES(:RECORDID,:FINDID);");
+        result.bindValue(":RECORDID", proginfo.recordid);
+        result.bindValue(":FINDID", proginfo.findid);
+    
+        result.exec();
+        if (!result.isActive())
+        {
+            MythContext::DBError("addFindHistory", result);
+        }
     }
+    // The removal of an entry from oldrecorded may affect near-future
+    // scheduling decisions, so recalculate
+    signalChange(0);
 }
 
-void ScheduledRecording::forgetHistory(QSqlDatabase* db,
-                                       const ProgramInfo& proginfo)
+void ScheduledRecording::forgetHistory(const ProgramInfo& proginfo)
 {
-    QSqlQuery result(QString::null, db);
+    MSqlQuery result(MSqlQuery::InitCon());
 
     result.prepare("DELETE FROM oldrecorded WHERE title = :TITLE AND "
                    "((subtitle = :SUBTITLE AND description = :DESC) OR "
-                   "(programid <> '' AND programid = :PROGRAMID))");
+                   "(programid <> '' AND programid = :PROGRAMID) OR "
+                   "(findid <> 0 AND findid = :FINDID))");
     result.bindValue(":TITLE", proginfo.title.utf8());
     result.bindValue(":SUBTITLE", proginfo.subtitle.utf8());
     result.bindValue(":DESC", proginfo.description.utf8());
     result.bindValue(":PROGRAMID", proginfo.programid);
+    result.bindValue(":FINDID", proginfo.findid);
     
     result.exec();
     if (!result.isActive())
     {
         MythContext::DBError("forgetHistory", result);
     }
-    else
+
+    if (proginfo.findid)
     {
-        // The removal of an entry from oldrecorded may affect near-future
-        // scheduling decisions, so recalculate
-        signalChange(db);
+        result.prepare("DELETE FROM oldfind WHERE "
+                       "recordid = :RECORDID AND findid = :FINDID");
+        result.bindValue(":RECORDID", proginfo.recordid);
+        result.bindValue(":FINDID", proginfo.findid);
+    
+        result.exec();
+        if (!result.isActive())
+        {
+            MythContext::DBError("forgetFindHistory", result);
+        }
     }
+    // The removal of an entry from oldrecorded may affect near-future
+    // scheduling decisions, so recalculate
+    signalChange(0);
 }
 
 
@@ -457,56 +527,60 @@ void ScheduledRecording::runProgList(void)
 {
     ProgLister *pl = NULL;
 
-    if (!m_pginfo)
+    if(search->intValue() && getRecordID())
     {
+        ScheduledRecording rule;
+        rule.loadByID(getRecordID());
+
         switch (search->intValue())
         {
         case kTitleSearch:
-            pl = new ProgLister(plTitleSearch, description->getValue(),
-                                QSqlDatabase::database(),
+            pl = new ProgLister(plTitleSearch, rule.description->getValue(),
                                 gContext->GetMainWindow(), "proglist");
             break;
         case kKeywordSearch:
-            pl = new ProgLister(plKeywordSearch, description->getValue(),
-                                QSqlDatabase::database(),
+            pl = new ProgLister(plKeywordSearch, rule.description->getValue(),
                                 gContext->GetMainWindow(), "proglist");
             break;
         case kPeopleSearch:
-            pl = new ProgLister(plPeopleSearch, description->getValue(),
-                                QSqlDatabase::database(),
+            pl = new ProgLister(plPeopleSearch, rule.description->getValue(),
+                                gContext->GetMainWindow(), "proglist");
+            break;
+        case kPowerSearch:
+            pl = new ProgLister(plSQLSearch, rule.description->getValue(),
                                 gContext->GetMainWindow(), "proglist");
             break;
         default:
             pl = new ProgLister(plTitle, title->getValue(),
-                                QSqlDatabase::database(),
                                 gContext->GetMainWindow(), "proglist");
             break;
         }
     }
     else
-        pl = new ProgLister(plTitle, m_pginfo->title,
-                            QSqlDatabase::database(),
+        pl = new ProgLister(plTitle, title->getValue(),
                             gContext->GetMainWindow(), "proglist");
-
     pl->exec();
     delete pl;
-    //proglistButton->setFocus();
 }
 
 void ScheduledRecording::runShowDetails(void)
 {
     if (m_pginfo)
-        m_pginfo->showDetails(QSqlDatabase::database());
+        m_pginfo->showDetails();
 }
 
-void ScheduledRecording::fillSelections(QSqlDatabase* db, SelectSetting* setting) {
-    QSqlQuery result = db->exec("SELECT recordid FROM record");
-    if (result.isActive() && result.numRowsAffected() > 0)
-        while (result.next()) {
-            int id = result.value(0).toInt();
+void ScheduledRecording::fillSelections(SelectSetting* setting) 
+{
+    MSqlQuery result(MSqlQuery::InitCon());
+    result.prepare("SELECT recordid FROM record");
+    if (result.exec() && result.isActive() && result.size() > 0)
+    {
+        while (result.next()) 
+        {
+            int recid = result.value(0).toInt();
 
             ScheduledRecording sr;
-            sr.loadByID(db, id);
+            sr.loadByID(recid);
 
             QString label;
             QString weekly = "";
@@ -518,6 +592,14 @@ void ScheduledRecording::fillSelections(QSqlDatabase* db, SelectSetting* setting
             case kFindOneRecord:
                 label = QString("%1").arg(sr.title->getValue());
                 break;
+            case kFindWeeklyRecord:
+                weekly = QDate(sr.startDate->dateValue()).toString("dddd")+"s ";
+            case kFindDailyRecord:
+                label = QString("%1 (after %2%3")
+                    .arg(sr.title->getValue())
+                    .arg(weekly)
+                    .arg(sr.findtime->timeValue().toString());
+                break;
             case kChannelRecord:
                 label = QString("%1 on channel %2")
                     .arg(sr.title->getValue())
@@ -547,18 +629,21 @@ void ScheduledRecording::fillSelections(QSqlDatabase* db, SelectSetting* setting
                 label = "You should not see this";
             }
 
-            setting->addSelection(label, QString::number(id));
+            setting->addSelection(label, QString::number(recid));
         }
+    }
 }
 
-void ScheduledRecordingEditor::load(QSqlDatabase* db) {
+void ScheduledRecordingEditor::load() 
+{
     clearSelections();
-    ScheduledRecording::fillSelections(db, this);
+    ScheduledRecording::fillSelections(this);
 }
 
 
-int ScheduledRecordingEditor::exec(QSqlDatabase* db) {
-    while (ConfigurationDialog::exec(db) == QDialog::Accepted)
+int ScheduledRecordingEditor::exec() 
+{
+    while (ConfigurationDialog::exec() == QDialog::Accepted)
         open(getValue().toInt());
 
     return QDialog::Rejected;
@@ -568,10 +653,10 @@ void ScheduledRecordingEditor::open(int id) {
     ScheduledRecording* sr = new ScheduledRecording();
 
     if (id != 0)
-        sr->loadByID(db,id);
+        sr->loadByID(id);
 
-    if (sr->exec(db) == QDialog::Accepted)
-        sr->save(db);
+    if (sr->exec() == QDialog::Accepted)
+        sr->save();
     delete sr;
 }
 
@@ -609,7 +694,7 @@ MythDialog* ScheduledRecording::dialogWidget(MythMainWindow *parent, const char
 
 
 
-void ScheduledRecording::setDefault(QSqlDatabase *db, bool haschannel)
+void ScheduledRecording::setDefault(bool haschannel)
 {
     id->setValue(0);
     title->setValue("");
@@ -625,6 +710,10 @@ void ScheduledRecording::setDefault(QSqlDatabase *db, bool haschannel)
     endTime->setValue(time);
     seriesid->setValue("");
     programid->setValue("");
+    findday->setValue(-1);
+    findtime->setValue(QTime::fromString("00:00:00", Qt::ISODate));
+    findid->setValue(0);
+    parentid->setValue(0);
     category->setValue("");
     search->setValue(kNoSearch);
 
@@ -637,10 +726,10 @@ void ScheduledRecording::setDefault(QSqlDatabase *db, bool haschannel)
     }
     
     type->clearSelections();
-    type->addNormalSelections(haschannel);
+    type->addNormalSelections(haschannel, search->intValue() == kManualSearch);
     type->setValue(kNotRecording);
     
-    profile->fillSelections(db);
+    profile->fillSelections();
     profile->setValue(QObject::tr("Default"));
     
     dupin->setValue(kDupsInAll);
@@ -653,14 +742,24 @@ void ScheduledRecording::setDefault(QSqlDatabase *db, bool haschannel)
     
     autoexpire->setValue(gContext->GetNumSetting("AutoExpireDefault", 0));
 
-    recgroup->fillSelections(db);    
+    autotranscode->setValue(0);
+    autocommflag->setValue(gContext->GetNumSetting("AutoCommercialFlag", 1));
+    autouserjob1->setValue(gContext->GetNumSetting("AutoRunUserJob1", 0));
+    autouserjob2->setValue(gContext->GetNumSetting("AutoRunUserJob2", 0));
+    autouserjob3->setValue(gContext->GetNumSetting("AutoRunUserJob3", 0));
+    autouserjob4->setValue(gContext->GetNumSetting("AutoRunUserJob4", 0));
+
+    recgroup->fillSelections();    
     recgroup->setValue("Default");
+
+    inactive->setValue(0);
 }
 
 void ScheduledRecording::setProgram(ProgramInfo *proginfo)
 {
     m_pginfo = proginfo;
-    
+    if (proginfo)
+    {
         title->setValue(proginfo->title);
         subtitle->setValue(proginfo->subtitle);
         description->setValue(proginfo->description);
@@ -672,8 +771,16 @@ void ScheduledRecording::setProgram(ProgramInfo *proginfo)
         endTime->setValue(proginfo->endts.time());
         seriesid->setValue(proginfo->seriesid);
         programid->setValue(proginfo->programid);
+        if (findday->intValue() < 0)
+        {
+            findday->setValue((proginfo->startts.date().dayOfWeek() + 1) % 7);
+            findtime->setValue(proginfo->startts.time());
+        }
+        findid->setValue(proginfo->findid);
         category->setValue(proginfo->category);
-
+        
+        fetchChannelInfo();
+    }
 }
 
 void ScheduledRecording::makeOverride(void)
@@ -685,14 +792,28 @@ void ScheduledRecording::makeOverride(void)
     id->setValue(0);
     type->clearSelections();
     type->addOverrideSelections();
-    search->setValue(kNoSearch);
+    type->setValue(kNotRecording);
+    inactive->setValue(0);
+
+    if (search->intValue() == kManualSearch)
+        search->setChanged();
+    else
+        search->setValue(kNoSearch);
 
     setProgram(m_pginfo);
 
+    parentid->setValue(m_pginfo->recordid);
+
     profile->setChanged();
     dupin->setChanged();
     dupmethod->setChanged();
     autoexpire->setChanged();
+    autotranscode->setChanged();
+    autocommflag->setChanged();
+    autouserjob1->setChanged();
+    autouserjob2->setChanged();
+    autouserjob3->setChanged();
+    autouserjob4->setChanged();
     maxepisodes->setChanged();
     maxnewest->setChanged();
     startoffset->setChanged();
