@@ -20,11 +20,6 @@ using namespace std;
 #include "../libmyth/util.h"
 #include "../libmyth/mythcontext.h"
 
-extern "C" {
-#include "../libavcodec/avcodec.h"
-#include "../libavcodec/xvmc_render.h"
-}
-
 #include <X11/keysym.h>
 #include <X11/Xatom.h>
 #include <X11/Xlib.h>
@@ -35,6 +30,11 @@ extern "C" {
 #define XMD_H 1
 #include <X11/extensions/xf86vmode.h>
 
+extern "C" {
+#include "../libavcodec/avcodec.h"
+#include "../libavcodec/xvmc_render.h"
+}
+
 /**********************************************************
  *  The DisplayRes module allows for the display resolution to be
  *  changed "on the fly", based on the video resolution.
@@ -73,7 +73,8 @@ extern "C" {
 }
 
 const int kNumBuffers = 8;
-const int kPrebufferFrames = 4;
+const int kPrebufferFramesNormal = 4;
+const int kPrebufferFramesSmall = 4;
 const int kNeedFreeFrames = 2;
 const int kKeepPrebuffer = 1;
 
@@ -121,8 +122,13 @@ struct XvMCData
     XShmSegmentInfo shminfo;
     XvImage *xvimage;
     unsigned char *palette;
+
+    int colorkey;
+    bool needdrawcolor;
+    bool initialdraw;
 };
 
+
 static int XJ_error_catcher(Display * d, XErrorEvent * xeev)
 {
     d = d; 
@@ -187,18 +193,20 @@ void VideoOutputXvMC::InputChanged(int width, int height, float aspect)
     {
         // Switching to custom display resolution succeeded
         // Make a note of the new size
-        dispx = dispy = 0;
-        dispw = display_res->Width();
-        disph = display_res->Height();
         w_mm =  display_res->Width_mm();
         h_mm =  display_res->Height_mm();
 
-        data->display_aspect = (float)(w_mm/h_mm);
+        data->display_aspect = display_res->GetAspectRatio();
 
-        // Resize X window to fill new resolution
-        XMoveResizeWindow(data->XJ_disp, data->XJ_win, 0, 0,
-                          display_res->Width(),
-                          display_res->Height());
+        if (!gContext->GetNumSetting("GuiSizeForTV", 0))
+        {
+            dispx = dispy = 0;
+            dispw = display_res->Width();
+            disph = display_res->Height();
+            // Resize X window to fill new resolution
+            XMoveResizeWindow(data->XJ_disp, data->XJ_win,
+                              dispx, dispy, dispw, disph);
+        }
     }
 
     CreateXvMCBuffers();
@@ -234,6 +242,9 @@ bool VideoOutputXvMC::Init(int width, int height, float aspect,
 {
     pthread_mutex_init(&lock, NULL);
 
+    needrepaint = true;
+    data->initialdraw = true;
+
     int (*old_handler)(Display *, XErrorEvent *);
     int i, ret;
     XJ_caught_error = 0;
@@ -247,7 +258,8 @@ bool VideoOutputXvMC::Init(int width, int height, float aspect,
     XvAdaptorInfo *ai;
 
     VideoOutput::InitBuffers(kNumBuffers, false, kNeedFreeFrames,
-                             kPrebufferFrames, kKeepPrebuffer);
+                             kPrebufferFramesNormal, kPrebufferFramesSmall, 
+                             kKeepPrebuffer);
     VideoOutput::Init(width, height, aspect, winid, winx, winy, winw, winh, 
                       embedid);
 
@@ -271,25 +283,29 @@ bool VideoOutputXvMC::Init(int width, int height, float aspect,
 
     if (display_res)
     {
-        if (display_res->Width() == 0)
-        {
-            // The very first Resize needs to be the maximum possible
-            // desired res, because X will mask off anything outside
-            // the initial dimensions
-            XMoveResizeWindow(data->XJ_disp, winid, 0, 0, 1920, 1080);
-        }
+        // The very first Resize needs to be the maximum possible
+        // desired res, because X will mask off anything outside
+        // the initial dimensions
+        XMoveResizeWindow(data->XJ_disp, data->XJ_win, 0, 0,
+                          display_res->GetMaxWidth(),
+                          display_res->GetMaxHeight());
 
         if (display_res->switchToVid(width, height))
         {
             // Switching to custom display resolution succeeded
             // Make a note of the new size
-            dispw = display_res->Width();
-            disph = display_res->Height();
             w_mm = display_res->Width_mm();
             h_mm = display_res->Height_mm();
             
-            // Resize X window to fill new resolution
-            XMoveResizeWindow(data->XJ_disp, winid, 0, 0, dispw, disph);
+            if (!gContext->GetNumSetting("GuiSizeForTV", 0))
+            {
+                dispx = dispy = 0;
+                dispw = display_res->Width();
+                disph = display_res->Height();
+                // Resize X window to fill new resolution
+                XMoveResizeWindow(data->XJ_disp, data->XJ_win,
+                                  dispx, dispy, dispw, disph);
+            }
         }
     }
     else
@@ -318,8 +334,8 @@ bool VideoOutputXvMC::Init(int width, int height, float aspect,
     {
         int w = DisplayWidth(data->XJ_disp, XJ_screen_num);
         int h = DisplayHeight(data->XJ_disp, XJ_screen_num);
-        int gui_w = gContext->GetNumSetting("GuiWidth", w);
-        int gui_h = gContext->GetNumSetting("GuiHeight", h);
+        int gui_w = w, gui_h = h;
+        gContext->GetResolutionSetting("Gui", gui_w,  gui_h);
 
         if (gui_w)
             w_mm = w_mm * gui_w / w;
@@ -331,6 +347,9 @@ bool VideoOutputXvMC::Init(int width, int height, float aspect,
     else
         data->display_aspect = (float)w_mm/h_mm;
 
+    if (display_res)
+        data->display_aspect = display_res->GetAspectRatio();
+
     XJ_white = XWhitePixel(data->XJ_disp, XJ_screen_num);
     XJ_black = XBlackPixel(data->XJ_disp, XJ_screen_num);
 
@@ -380,18 +399,35 @@ bool VideoOutputXvMC::Init(int width, int height, float aspect,
                 XvPortID p = 0;
                 int s;
                 XvMCSurfaceTypes::find(width, height, chroma,
-                                       true, 2, 0, 0,
+                                       true, false, 2, 0, 0,
                                        data->XJ_disp, ai[i].base_id,
                                        ai[i].base_id + ai[i].num_ports - 1,
                                        p, s);
+                if (p != 0)
+                    printf("XvMC found and using VLD surface\n");
+
+                if (0 == p)
+                {
+                    // No VLD surface found, try to find IDCT surface
+                    XvMCSurfaceTypes::find(width, height, chroma,
+                                           false, true, 2, 0, 0,
+                                           data->XJ_disp, ai[i].base_id,
+                                           ai[i].base_id + ai[i].num_ports - 1,
+                                           p, s);
+                    if (p != 0)
+                        printf("XvMC found and using IDCT surface\n");
+                }
+
                 if (0 == p) 
                 {
                     // No IDCT surface found, try to find MC surface
                     XvMCSurfaceTypes::find(width, height, chroma,
-                                           false, 2, 0, 0,
+                                           false, false, 2, 0, 0,
                                            data->XJ_disp, ai[i].base_id,
                                            ai[i].base_id + ai[i].num_ports - 1,
                                            p, s);
+                    if (p != 0)
+                        printf("XvMC found and using MC surface\n");
                 }
 
                 if (p != 0) 
@@ -423,14 +459,14 @@ bool VideoOutputXvMC::Init(int width, int height, float aspect,
         XJ_screenwidth = display_res->Width();
         XJ_screenheight = display_res->Height();
     }
-#ifndef QWS
     else
     {
-        GetMythTVGeometry(data->XJ_disp, XJ_screen_num,
-                          &XJ_screenx, &XJ_screeny, 
-                          &XJ_screenwidth, &XJ_screenheight);
+        // Get default (possibly user selected) screen resolution from context
+        float wmult, hmult;
+
+        gContext->GetScreenSettings(XJ_screenx, XJ_screenwidth, wmult,
+                                    XJ_screeny, XJ_screenheight, hmult);
     }
-#endif // QWS
 
     if (embedid > 0)
         data->XJ_curwin = data->XJ_win = embedid;
@@ -491,41 +527,7 @@ bool VideoOutputXvMC::Init(int width, int height, float aspect,
         return false;
     }
 
-    Atom xv_atom;  
-    XvAttribute *attributes;
-    int attrib_count;
-    bool needdrawcolor = true;
-
-    attributes = XvQueryPortAttributes(data->XJ_disp, xv_port, &attrib_count);
-    if (attributes)
-    {
-        for (int i = 0; i < attrib_count; i++)
-        {
-            if (!strcmp(attributes[i].name, "XV_AUTOPAINT_COLORKEY"))
-            {
-                xv_atom = XInternAtom(data->XJ_disp, "XV_AUTOPAINT_COLORKEY",
-                                      False);
-                if (xv_atom != None)
-                {
-                    ret = XvSetPortAttribute(data->XJ_disp, xv_port, xv_atom,
-                                             1);
-                    if (ret == Success)
-                        needdrawcolor = false;
-                }
-            }
-        }
-        XFree(attributes);
-    }
-
-    xv_atom = XInternAtom(data->XJ_disp, "XV_COLORKEY", False);
-    if (xv_atom != None)
-    {
-        ret = XvGetPortAttribute(data->XJ_disp, xv_port, xv_atom, &colorkey);
-        if (ret == Success)
-        {
-            needdrawcolor = true;
-        }
-    }
+    InitColorKey(true);
 
     MoveResize();
 
@@ -542,6 +544,60 @@ bool VideoOutputXvMC::Init(int width, int height, float aspect,
     return true;
 }
 
+void VideoOutputXvMC::InitColorKey(bool turnoffautopaint)
+{
+    int ret = Success;
+    data->needdrawcolor = true;
+
+    Atom xv_atom;
+    XvAttribute *attributes;
+    int attrib_count;
+
+    attributes = XvQueryPortAttributes(data->XJ_disp, xv_port, &attrib_count);
+    if (attributes)
+    {
+        for (int i = 0; i < attrib_count; i++)
+        {
+            if (!strcmp(attributes[i].name, "XV_AUTOPAINT_COLORKEY"))
+            {
+                xv_atom = XInternAtom(data->XJ_disp, "XV_AUTOPAINT_COLORKEY",
+                                      False);
+                if (xv_atom == None)
+                    continue;
+
+                if (turnoffautopaint)
+                    XvSetPortAttribute(data->XJ_disp, xv_port, xv_atom, 0);
+                else 
+                    XvSetPortAttribute(data->XJ_disp, xv_port, xv_atom, 1);
+            }
+        }
+        XFree(attributes);
+    }
+
+    if (data->needdrawcolor)
+    {
+        xv_atom = XInternAtom(data->XJ_disp, "XV_COLORKEY", False);
+        if (xv_atom != None)
+        {
+            ret = XvGetPortAttribute(data->XJ_disp, xv_port, xv_atom, 
+                                     &data->colorkey);
+            if (ret != Success)
+            {
+                VERBOSE(VB_IMPORTANT,
+                        "Couldn't get the color key color, and we need it.\n"
+                        "You likely won't get any video.");
+                data->colorkey = 0;
+            }
+        }
+    }
+}
+
+bool VideoOutputXvMC::SetupDeinterlace(bool interlaced)
+{
+    bool deint = VideoOutput::SetupDeinterlace(interlaced);
+    return deint;
+}
+
 bool VideoOutputXvMC::NeedsDoubleFramerate() const
 {
     return m_deinterlacing; // always doing bob deint
@@ -577,26 +633,29 @@ bool VideoOutputXvMC::CreateXvMCBuffers(void)
     int numblocks = ((XJ_width + 15) / 16) * ((XJ_height + 15) / 16);
     int blocks_per_macroblock = 6;
 
-    for (int i = 0; i < 8; i++)
+    if (!hasVLDAcceleration())
     {
-        ret = XvMCCreateBlocks(data->XJ_disp, &data->ctx,
-                               numblocks * blocks_per_macroblock,
-                               &(data->data_blocks[i]));
-        if (ret != Success)
+        for (int i = 0; i < 8; i++)
         {
-            cerr << "Unable to create XvMC Blocks\n";
-            XvMCDestroyContext(data->XJ_disp, &data->ctx);
-            return false;
-        }
+            ret = XvMCCreateBlocks(data->XJ_disp, &data->ctx,
+                                   numblocks * blocks_per_macroblock,
+                                   &(data->data_blocks[i]));
+            if (ret != Success)
+            {
+                cerr << "Unable to create XvMC Blocks\n";
+                XvMCDestroyContext(data->XJ_disp, &data->ctx);
+                return false;
+            }
 
-        ret = XvMCCreateMacroBlocks(data->XJ_disp, &data->ctx, numblocks,
-                                    &(data->mv_blocks[i]));
-        if (ret != Success)
-        {
-            cerr << "Unable to create XvMC Macro Blocks\n";
-            XvMCDestroyBlocks(data->XJ_disp,&(data->data_blocks[i]));
-            XvMCDestroyContext(data->XJ_disp, &data->ctx);
-            return false;
+            ret = XvMCCreateMacroBlocks(data->XJ_disp, &data->ctx, numblocks,
+                                        &(data->mv_blocks[i]));
+            if (ret != Success)
+            {
+                cerr << "Unable to create XvMC Macro Blocks\n";
+                XvMCDestroyBlocks(data->XJ_disp,&(data->data_blocks[i]));
+                XvMCDestroyContext(data->XJ_disp, &data->ctx);
+                return false;
+            }
         }
     }
 
@@ -628,6 +687,9 @@ bool VideoOutputXvMC::CreateXvMCBuffers(void)
         render->p_surface = surface;
         render->state = 0;
 
+        render->disp = data->XJ_disp;
+        render->ctx = &data->ctx;
+
         vbuffers[i].buf = (unsigned char *)render;
         vbuffers[i].priv[0] = (unsigned char *)&(data->data_blocks[i]);
         vbuffers[i].priv[1] = (unsigned char *)&(data->mv_blocks[i]);
@@ -710,6 +772,13 @@ void VideoOutputXvMC::Exit(void)
 {
     if (XJ_started) 
     {
+        XSetForeground(data->XJ_disp, data->XJ_gc, XJ_black);
+        XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc,
+                       dispx, dispy, dispw, disph);
+
+        m_deinterlacing = false;
+        InitColorKey(false);
+
         XJ_started = false;
 
         DeleteXvMCBuffers();
@@ -722,10 +791,13 @@ void VideoOutputXvMC::Exit(void)
 
 void VideoOutputXvMC::DeleteXvMCBuffers()
 {
-    for (int i = 0; i < 8; i++)
+    if (!hasVLDAcceleration()) 
     {
-        XvMCDestroyMacroBlocks(data->XJ_disp, &data->mv_blocks[i]);
-        XvMCDestroyBlocks(data->XJ_disp, &data->data_blocks[i]);
+        for (int i = 0; i < 8; i++)
+        {
+            XvMCDestroyMacroBlocks(data->XJ_disp, &data->mv_blocks[i]);
+            XvMCDestroyBlocks(data->XJ_disp, &data->data_blocks[i]);
+        }
     }
 
     for (int i = 0; i < numbuffers; i++)
@@ -765,10 +837,15 @@ void VideoOutputXvMC::DeleteXvMCBuffers()
 
 void VideoOutputXvMC::EmbedInWidget(WId wid, int x, int y, int w, int h)
 {
-    if (embedding)
-        return;
-
     pthread_mutex_lock(&lock);
+
+    if (embedding)
+    {
+        MoveResize();
+        pthread_mutex_unlock(&lock);
+        return;
+    }
+
     data->XJ_curwin = wid;
     VideoOutput::EmbedInWidget(wid, x, y, w, h);
 
@@ -831,6 +908,8 @@ void VideoOutputXvMC::PrepareFrame(VideoFrame *buffer, FrameScanType t)
     if (!buffer)
         return;
 
+    framesPlayed = buffer->frameNumber + 1;
+
     pthread_mutex_lock(&lock);
 
     xvmc_render_state_t *render = reinterpret_cast<xvmc_render_state_t *>(buffer->buf);
@@ -840,9 +919,13 @@ void VideoOutputXvMC::PrepareFrame(VideoFrame *buffer, FrameScanType t)
         (osdframe) ? (xvmc_render_state_t *)(osdframe->buf) : 0;
     Display *display = data->XJ_disp;
 
+    pthread_mutex_unlock(&lock);
+
     SyncSurface(display, render->p_surface);
     SyncSurface(display, (osdrender) ? osdrender->p_surface : 0);
 
+    pthread_mutex_lock(&lock);
+
     render->state |= MP_XVMC_STATE_DISPLAY_PENDING;
     data->p_render_surface_to_show = render;
 
@@ -850,7 +933,8 @@ void VideoOutputXvMC::PrepareFrame(VideoFrame *buffer, FrameScanType t)
 
     if (needrepaint)
     {
-        DrawUnusedRects();
+        data->initialdraw = false;
+        DrawUnusedRects(false);
         needrepaint = false;
     }
 
@@ -859,18 +943,26 @@ void VideoOutputXvMC::PrepareFrame(VideoFrame *buffer, FrameScanType t)
 
 void VideoOutputXvMC::Show(FrameScanType scan)
 {
-    int field;
-    switch (scan) 
+    int field = 3;
+    int src_h = imgh;
+    int halfLineSrc = 0;
+
+    VideoFrame *buffer = data->lastframe;
+
+    if (m_deinterlacing && buffer)
     {
-        case kScan_Interlaced:
+        if ((scan == kScan_Interlaced && buffer->top_field_first == 1) ||
+            (scan == kScan_Intr2ndField && buffer->top_field_first == 0))
+        {
             field = 1;
-            break;
-        case kScan_Intr2ndField:
+        }
+        else if ((scan == kScan_Interlaced && buffer->top_field_first == 0) ||
+                 (scan == kScan_Intr2ndField && buffer->top_field_first == 1))
+        {
             field = 2;
-            break;
-        default:
-            field = 3;
-            break;
+            halfLineSrc = (int) round(((float)disphoff)/imgh - 0.001f);
+            //src_h -= (halfLineSrc) ? 2 : 0;
+        }
     }
 
     pthread_mutex_lock(&lock);
@@ -895,8 +987,27 @@ void VideoOutputXvMC::Show(FrameScanType scan)
     if (data->p_render_surface_visible != NULL)
         data->p_render_surface_visible->state &= ~MP_XVMC_STATE_DISPLAY_PENDING;
 
-    XvMCPutSurface(data->XJ_disp, surf, data->XJ_curwin, imgx, imgy, imgw, 
-                   imgh, dispxoff, dispyoff, dispwoff, disphoff, field);
+    // XXX Ugly hack.  With bobdeint, holding this lock takes too much time
+    // away from the decoding process, and coming back from a pause can
+    // result in a very low buffer situation, causing jerky playback.
+    // Let's just hope that the XvMC driver's a bit threadsafe.
+    //if (field != 3)
+        pthread_mutex_unlock(&lock);
+
+    int yoff = dispyoff;
+    int hoff = disphoff;
+    if (!hasVLDAcceleration())
+    {
+        yoff += halfLineSrc;
+        hoff -= halfLineSrc;
+    }
+
+    XvMCPutSurface(data->XJ_disp, surf, data->XJ_curwin,
+                   imgx, imgy, imgw, src_h,
+                   dispxoff, yoff, dispwoff, hoff, field);
+
+    //if (field != 3)
+        pthread_mutex_lock(&lock);
 
     if (data->p_render_surface_visible && 
         (data->p_render_surface_visible != showingsurface))
@@ -916,14 +1027,13 @@ void VideoOutputXvMC::Show(FrameScanType scan)
     }
 
     data->p_render_surface_visible = data->p_render_surface_to_show;
-    data->p_render_surface_to_show = NULL;
 
     if (osdframe)
-    {
         data->p_render_surface_visible = osdren;
-        if (data->curosd)
-            DiscardFrame(data->curosd);
-        data->curosd = osdframe;
+ 
+    if (!m_deinterlacing || (m_deinterlacing && scan != kScan_Interlaced))
+    {
+        data->p_render_surface_to_show = NULL;
         render->p_osd_target_surface_render = NULL;
     }
 
@@ -971,24 +1081,39 @@ void VideoOutputXvMC::DrawSlice(VideoFrame *frame, int x, int y, int w, int h)
 
     pthread_mutex_lock(&lock);
 
-    if (render->p_past_surface && !render->p_future_surface)
-        SyncSurface(data->XJ_disp, render->p_past_surface); // Sync prev-I frame
-
-    int res = XvMCRenderSurface(data->XJ_disp, &data->ctx, 
-                                render->picture_structure, render->p_surface,
-                                render->p_past_surface, 
-                                render->p_future_surface,
-                                render->flags, render->filled_mv_blocks_num,
-                                render->start_mv_blocks_num,
-                                (XvMCMacroBlockArray *)frame->priv[1], 
-                                (XvMCBlockArray *)frame->priv[0]);
-    if (res != Success)
+    if (hasVLDAcceleration())
     {
-        cerr << "XvMCRenderSurface error\n";
+#ifdef USING_XVMC_VLD
+        Status status = XvMCPutSlice2(data->XJ_disp, &data->ctx, 
+                                      (char*)render->slice_data, 
+                                      render->slice_datalen, 
+                                      render->slice_code);
+        if (status)
+            printf("XvMCPutSlice: Error: %d\n", status);
+#endif
     }
+    else
+    {
+        if (render->p_past_surface && !render->p_future_surface)
+            SyncSurface(data->XJ_disp, render->p_past_surface); // Sync prev-I frame
 
-    FlushSlice(data->XJ_disp, render->p_surface, render->pict_type, 
-               render->p_past_surface, render->p_future_surface);
+        int res = XvMCRenderSurface(data->XJ_disp, &data->ctx, 
+                                    render->picture_structure, 
+                                    render->p_surface,
+                                    render->p_past_surface, 
+                                    render->p_future_surface,
+                                    render->flags, render->filled_mv_blocks_num,
+                                    render->start_mv_blocks_num,
+                                    (XvMCMacroBlockArray *)frame->priv[1], 
+                                    (XvMCBlockArray *)frame->priv[0]);
+        if (res != Success)
+        {
+            cerr << "XvMCRenderSurface error\n";
+        }
+
+        FlushSlice(data->XJ_disp, render->p_surface, render->pict_type, 
+                   render->p_past_surface, render->p_future_surface);
+    }
 
     pthread_mutex_unlock(&lock);
 
@@ -997,28 +1122,37 @@ void VideoOutputXvMC::DrawSlice(VideoFrame *frame, int x, int y, int w, int h)
     render->next_free_data_block_num = 0;
 }
 
-void VideoOutputXvMC::DrawUnusedRects(void)
+void VideoOutputXvMC::DrawUnusedRects(bool sync)
 {
-    XSetForeground(data->XJ_disp, data->XJ_gc, colorkey);
-    XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc, dispx, dispy,
-                   dispw, disph);
+    // boboff assumes the smallest interlaced resolution is 480 lines
+    int boboff = (int) round(((float)disphoff)/480 - 0.001f);
+    boboff = (m_deinterlacing && m_deintfiltername == "bobdeint") ? boboff : 0;
+
+    if (data->needdrawcolor && !data->initialdraw)
+    {
+        XSetForeground(data->XJ_disp, data->XJ_gc, data->colorkey);
+        XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc,
+                       dispx, dispy + boboff, dispw, disph - 2 * boboff);
+    }
 
     XSetForeground(data->XJ_disp, data->XJ_gc, XJ_black);
     if (dispxoff > dispx) // left
         XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc, 
-                       dispx, dispy, dispxoff-dispx, disph);
-    if (dispxoff+dispwoff < dispx+dispw) // right
+                       dispx, dispy, dispxoff - dispx, disph);
+    if (dispxoff + dispwoff < dispx + dispw) // right
         XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc, 
-                       dispxoff+dispwoff, dispy, 
-                       (dispx+dispw)-(dispxoff+dispwoff), disph);
-    if (dispyoff > dispy) // bottom
+                       dispxoff + dispwoff, dispy, 
+                       (dispx + dispw) - (dispxoff + dispwoff), disph);
+    if (dispyoff + boboff > dispy) // top of screen
         XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc, 
-                       dispx, dispy, dispw, dispyoff-dispy);
-    if (dispyoff+disphoff < dispy+disph) // top
+                       dispx, dispy, dispw, dispyoff + boboff - dispy);
+    if (dispyoff + disphoff < dispy + disph + boboff) // bottom of screen
         XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc, 
-                       dispx, dispyoff+disphoff, 
-                       dispw, (dispy+disph)-(dispyoff+disphoff));
-    XSync(data->XJ_disp, false);
+                       dispx, dispyoff + disphoff - boboff, 
+                       dispw, (dispy + disph + boboff) - (dispyoff + disphoff));
+
+    if (sync)
+        XSync(data->XJ_disp, false);
 }
 
 float VideoOutputXvMC::GetDisplayAspect(void)
@@ -1045,6 +1179,12 @@ void VideoOutputXvMC::ProcessFrame(VideoFrame *frame, OSD *osd,
     xvmc_render_state_t *render = (xvmc_render_state_t *)frame->buf;
     render->p_osd_target_surface_render = NULL;
 
+    if (data->curosd)
+    {
+        DiscardFrame(data->curosd);
+        data->curosd = NULL;
+    }
+
     if (osd)
     {
         if (data->subpicture_mode == BLEND_SUBPICTURE ||
@@ -1086,6 +1226,7 @@ void VideoOutputXvMC::ProcessFrame(VideoFrame *frame, OSD *osd,
                                      0, 0, XJ_width, XJ_height);
 
                 render->p_osd_target_surface_render = newframe;
+                data->curosd = newframe;
             }
             else if (data->subpicture_mode == BACKEND_SUBPICTURE)
             {
@@ -1154,6 +1295,8 @@ int VideoOutputXvMC::ChangePictureAttribute(int attributeType, int newValue)
 
             XvSetPortAttribute(data->XJ_disp, xv_port, attribute, value);
 
+            XvMCSetAttribute(data->XJ_disp, &data->ctx, attribute, value);
+
             pthread_mutex_unlock(&lock);
 
             return newValue;
@@ -1171,3 +1314,11 @@ bool VideoOutputXvMC::hasIDCTAcceleration() const
     return XVMC_IDCT == (data->surface_info.mc_type & XVMC_IDCT);
 }
 
+bool VideoOutputXvMC::hasVLDAcceleration() const
+{
+#if USING_XVMC_VLD
+    return XVMC_VLD == (data->surface_info.mc_type & XVMC_VLD);
+#endif
+    return 0;
+}
+
