@@ -2,6 +2,7 @@
 #include "videosource.h"
 #include "libmyth/mythwidgets.h"
 #include "libmyth/mythcontext.h"
+#include "libmyth/mythdbcon.h"
 #include "datadirect.h"
 
 #include <qapplication.h>
@@ -32,15 +33,18 @@
 
 const QString CardUtil::DVB = "DVB";
 
-bool CardUtil::isCardPresent(QSqlDatabase *db, const QString &strType)
+bool CardUtil::isCardPresent(const QString &strType)
 {
-    QSqlQuery query = db->exec(QString("SELECT count(cardtype)"
-                                       " FROM capturecard, cardinput"
-                                       " WHERE cardinput.cardid = capturecard.cardid"
-                                       " AND capturecard.cardtype=\"%1\""
-                                       " AND capturecard.hostname = \"%2\"")
-                                       .arg(strType).arg(gContext->GetHostName()));
-    if (query.isActive() && query.numRowsAffected() > 0)
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("SELECT count(cardtype)"
+                  " FROM capturecard, cardinput"
+                  " WHERE cardinput.cardid = capturecard.cardid"
+                  " AND capturecard.cardtype= :CARDTYPE "
+                  " AND capturecard.hostname = :HOSTNAME ;");
+    query.bindValue(":CARDTYPE", strType);
+    query.bindValue(":HOSTNAME", gContext->GetHostName());
+
+    if (query.exec() && query.isActive() && query.size() > 0)
     {
         query.next();
         int count = query.value(0).toInt();
@@ -99,14 +103,15 @@ enum CardUtil::DVB_TYPES CardUtil::cardDVBType(unsigned nVideoDev)
     return CardUtil::cardDVBType(nVideoDev,name);
 }
 
-int CardUtil::videoDeviceFromCardID(QSqlDatabase *db, unsigned nCardID)
+int CardUtil::videoDeviceFromCardID(unsigned nCardID)
 {
     int iRet = -1;
-    QSqlQuery query = db->exec(QString("SELECT videodevice FROM capturecard "
-                                       " WHERE capturecard.cardid=%1 ")
-                                       .arg(nCardID));
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("SELECT videodevice FROM capturecard "
+                  " WHERE capturecard.cardid = :CARDID ;");
+    query.bindValue(":CARDID", nCardID);
 
-    if (query.isActive() && query.numRowsAffected() > 0)
+    if (query.exec() && query.isActive() && query.size() > 0)
     {
         query.next();
         iRet = query.value(0).toInt();
@@ -114,14 +119,15 @@ int CardUtil::videoDeviceFromCardID(QSqlDatabase *db, unsigned nCardID)
     return iRet;
 }
 
-bool CardUtil::isDVB(QSqlDatabase *db, unsigned nCardID)
+bool CardUtil::isDVB(unsigned nCardID)
 {
     bool fRet = false;
-    QSqlQuery query = db->exec(QString("SELECT cardtype FROM capturecard "
-                                       " WHERE capturecard.cardid=%1")
-                                       .arg(nCardID));
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("SELECT cardtype FROM capturecard "
+                  " WHERE capturecard.cardid= :CARDID ;");
+    query.bindValue(":CARDID", nCardID);
 
-    if (query.isActive() && query.numRowsAffected() > 0)
+    if (query.exec() && query.isActive() && query.size() > 0)
     {
         query.next();
         if (query.value(0).toString() == "DVB")
@@ -130,16 +136,16 @@ bool CardUtil::isDVB(QSqlDatabase *db, unsigned nCardID)
     return fRet;
 }
 
-enum CardUtil::DISEQC_TYPES CardUtil::diseqcType(QSqlDatabase *db,
-                                                 unsigned nCardID)
+enum CardUtil::DISEQC_TYPES CardUtil::diseqcType(unsigned nCardID)
 {
     int iRet = 0;
-    QSqlQuery query = db->exec(QString("SELECT dvb_diseqc_type "
-                                       "FROM capturecard "
-                                       " WHERE capturecard.cardid=%1 ")
-                                       .arg(nCardID));
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("SELECT dvb_diseqc_type "
+                  "FROM capturecard "
+                  " WHERE capturecard.cardid= :CARDID ;");
+    query.bindValue(":CARDID", nCardID);
 
-    if (query.isActive() && query.numRowsAffected() > 0)
+    if (query.exec() && query.isActive() && query.size() > 0)
     {
         query.next();
         iRet = query.value(0).toInt();
@@ -254,9 +260,9 @@ void DataDirectLineupSelector::fillSelections(const QString &uid,
     pdlg.Close();
 }
 
-void DataDirect_config::load(QSqlDatabase* db) 
+void DataDirect_config::load() 
 {
-    VerticalConfigurationGroup::load(db);
+    VerticalConfigurationGroup::load();
     if ((userid->getValue() != lastloadeduserid) || 
         (password->getValue() != lastloadedpassword)) 
     {
@@ -380,8 +386,7 @@ XMLTV_uk_config::XMLTV_uk_config(const VideoSource& _parent)
             provider, SLOT(fillSelections(const QString&)));
 }
 
-void XMLTV_uk_config::save(QSqlDatabase* db) {
-    (void)db;
+void XMLTV_uk_config::save() {
 
     QString waitMsg(QObject::tr("Please wait while MythTV retrieves the "
                                 "list of available channels\n.  You "
@@ -393,8 +398,8 @@ void XMLTV_uk_config::save(QSqlDatabase* db) {
                                 "list of available channels"));
     pdlg.show();
 
-    QString filename = QString("%1/.mythtv/%2.xmltv")
-        .arg(QDir::homeDirPath()).arg(parent.getSourceName());
+    QString filename = QString("%1/%2.xmltv")
+        .arg(MythContext::GetConfDir()).arg(parent.getSourceName());
     QString command = QString("tv_grab_uk --config-file '%1' --configure --retry-limit %2 --retry-delay %3 --postalcode %4 --provider %5 --auto-new-channels add")
         .arg(filename)
         .arg(2)
@@ -431,8 +436,7 @@ XMLTV_generic_config::XMLTV_generic_config(const VideoSource& _parent,
     setValue(QObject::tr("Configuration will run in the terminal window"));
 }
 
-void XMLTV_generic_config::save(QSqlDatabase* db) {
-    (void)db;
+void XMLTV_generic_config::save() {
 
     QString waitMsg(QObject::tr("Please wait while MythTV retrieves the "
                                 "list of available channels.\nYou "
@@ -445,15 +449,11 @@ void XMLTV_generic_config::save(QSqlDatabase* db) {
     pdlg.show();
 
     QString command;
-    if (grabber == "tv_grab_de") {
-        command = "tv_grab_de --configure";
-    } else {
-        QString filename = QString("%1/.mythtv/%2.xmltv")
-            .arg(QDir::homeDirPath()).arg(parent.getSourceName());
+    QString filename = QString("%1/%2.xmltv")
+        .arg(MythContext::GetConfDir()).arg(parent.getSourceName());
 
-        command = QString("%1 --config-file '%2' --configure")
-            .arg(grabber).arg(filename);
-    }
+    command = QString("%1 --config-file '%2' --configure")
+        .arg(grabber).arg(filename);
 
     pdlg.setProgress(1);
 
@@ -473,10 +473,11 @@ void XMLTV_generic_config::save(QSqlDatabase* db) {
                                               "information"));
     }
 
-    if (grabber == "tv_grab_de" || grabber == "tv_grab_se_swedb" || 
+    if (grabber == "tv_grab_de_tvtoday" || grabber == "tv_grab_se_swedb" || 
         grabber == "tv_grab_fi" || grabber == "tv_grab_es" ||
         grabber == "tv_grab_nl" || grabber == "tv_grab_jp" ||
-        grabber == "tv_grab_no" || grabber == "tv_grab_pt")
+        grabber == "tv_grab_no" || grabber == "tv_grab_pt" ||
+        grabber == "tv_grab_ee")
     {
         cerr << "You _MUST_ run 'mythfilldatabase --manual the first time, "
              << "instead\n";
@@ -512,8 +513,8 @@ XMLTVConfig::XMLTVConfig(const VideoSource& parent)
     addTarget("datadirect", new DataDirect_config(parent));
     grabber->addSelection("North America (DataDirect)", "datadirect");
 
-    addTarget("tv_grab_de", new XMLTV_generic_config(parent, "tv_grab_de"));
-    grabber->addSelection("Germany/Austria", "tv_grab_de");
+    addTarget("tv_grab_de_tvtoday", new XMLTV_generic_config(parent, "tv_grab_de_tvtoday"));
+    grabber->addSelection("Germany (tvtoday)", "tv_grab_de_tvtoday");
 
     addTarget("tv_grab_se_swedb", new XMLTV_generic_config(parent, "tv_grab_se_swedb"));
     grabber->addSelection("Sweden (tv.swedb.se)","tv_grab_se_swedb");
@@ -527,9 +528,6 @@ XMLTVConfig::XMLTVConfig(const VideoSource& parent)
     addTarget("tv_grab_au", new XMLTV_generic_config(parent, "tv_grab_au"));
     grabber->addSelection("Australia", "tv_grab_au");
 
-    addTarget("tv_grab_nz", new XMLTV_generic_config(parent, "tv_grab_nz"));
-    grabber->addSelection("New Zealand", "tv_grab_nz");
-
     addTarget("tv_grab_fi", new XMLTV_generic_config(parent, "tv_grab_fi"));
     grabber->addSelection("Finland", "tv_grab_fi");
 
@@ -550,6 +548,9 @@ XMLTVConfig::XMLTVConfig(const VideoSource& parent)
 
     addTarget("tv_grab_pt", new XMLTV_generic_config(parent, "tv_grab_pt"));
     grabber->addSelection("Portugal", "tv_grab_pt");
+
+    addTarget("tv_grab_ee", new XMLTV_generic_config(parent, "tv_grab_ee"));
+    grabber->addSelection("Estonia", "tv_grab_ee");
 }
 
 VideoSource::VideoSource() 
@@ -565,21 +566,19 @@ VideoSource::VideoSource()
     addChild(group);
 }
 
-bool VideoSourceEditor::cardTypesInclude(QSqlDatabase *db, 
-                                         const int &sourceID, 
+bool VideoSourceEditor::cardTypesInclude(const int &sourceID, 
                                          const QString &thecardtype) 
 {
-    QString querystr = QString("SELECT count(cardtype)"
-                               " FROM cardinput,capturecard "
-                               " WHERE capturecard.cardid = cardinput.cardid "
-                               " AND cardinput.sourceid=%1 "
-                               " AND capturecard.cardtype=\"%2\"")
-                               .arg(sourceID)
-                               .arg(thecardtype);
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("SELECT count(cardtype)"
+                  " FROM cardinput,capturecard "
+                  " WHERE capturecard.cardid = cardinput.cardid "
+                  " AND cardinput.sourceid= :SOURCEID "
+                  " AND capturecard.cardtype= :CARDTYPE ;");
+    query.bindValue(":SOURCEID", sourceID);
+    query.bindValue(":CARDTYPE", thecardtype);
 
-    QSqlQuery query = db->exec(querystr);
-
-    if (query.isActive() && query.numRowsAffected() > 0)
+    if (query.exec() && query.isActive() && query.size() > 0)
     {
         query.next();
         int count = query.value(0).toInt();
@@ -591,20 +590,25 @@ bool VideoSourceEditor::cardTypesInclude(QSqlDatabase *db,
     return false;
 }
 
-void VideoSource::fillSelections(QSqlDatabase* db,
-                                 SelectSetting* setting) {
-    QString query = QString("SELECT name, sourceid FROM videosource;");
-    QSqlQuery result = db->exec(query);
+void VideoSource::fillSelections(SelectSetting* setting) 
+{
+    MSqlQuery result(MSqlQuery::InitCon());
+    result.prepare("SELECT name, sourceid FROM videosource;");
 
-    if (result.isActive() && result.numRowsAffected() > 0)
+    if (result.exec() && result.isActive() && result.size() > 0)
+    {
         while (result.next())
+        {
             setting->addSelection(result.value(0).toString(),
                                   result.value(1).toString());
+        }
+    }
 }
 
-void VideoSource::loadByID(QSqlDatabase* db, int sourceid) {
+void VideoSource::loadByID(int sourceid) 
+{
     id->setValue(sourceid);
-    load(db);
+    load();
 }
 
 class VideoDevice: public PathSetting, public CCSetting {
@@ -759,13 +763,14 @@ public:
     };
 };
 
-class DVBSwFilter: public CheckBoxSetting, public CCSetting {
+class DVBHwDecoder: public CheckBoxSetting, public CCSetting {
 public:
-    DVBSwFilter(const CaptureCard& parent):
-        CCSetting(parent, "dvb_swfilter") {
-        setLabel(QObject::tr("Do NOT use DVB driver for filtering."));
-        setHelpText(QObject::tr("(BROKEN) This option is used to get around "
-                    "filtering limitations on some DVB cards."));
+    DVBHwDecoder(const CaptureCard& parent):
+        CCSetting(parent, "dvb_hw_decoder") {
+        setLabel(QObject::tr("Use hardware MPEG decoder"));
+        setHelpText(QObject::tr("If your DVB card has a built in MPEG decoder "
+                    "you can activate it here. "
+                    "(Only for full featured cards & certain USB devices)."));
     };
 };
 
@@ -817,6 +822,7 @@ public:
         addSelection("DiSEqC v1.1 Switch (4-Way)","5");
         addSelection("DiSEqC v1.2 Positioner","6");
         addSelection("DiSEqC v1.3 Positioner (Goto X)","7");
+        addSelection("DiSEqC v1.1 or 2.1 (10-way method2)","8");
         setHelpText(QObject::tr("Select the input type for DVB-S cards. "
                     "Leave as Single LNB/Input for DVB-C or DVB-T. "
                     "The inputs are mapped from Input Connections option "
@@ -857,6 +863,16 @@ class FirewireModel: public ComboBoxSetting, public CCSetting {
         }
 };
 
+class FirewireConnection: public ComboBoxSetting, public CCSetting {
+    public:
+       FirewireConnection(const CaptureCard& parent):
+       CCSetting(parent, "firewire_connection") {
+            setLabel(QObject::tr("Firewire Connection Type"));
+            addSelection(QObject::tr("Point to Point"),"0");
+            addSelection(QObject::tr("Broadcast"),"1");
+        }
+};
+
 class FirewirePort: public LineEditSetting, public CCSetting {
     public:
 	FirewirePort(const CaptureCard& parent):
@@ -904,6 +920,7 @@ public:
         parent(a_parent) {
         setUseLabel(false);
         addChild(new FirewireModel(parent));
+        addChild(new FirewireConnection(parent));
         addChild(new FirewirePort(parent));
         addChild(new FirewireNode(parent));
         addChild(new FirewireSpeed(parent));
@@ -1001,7 +1018,7 @@ CaptureCardGroup::CaptureCardGroup(CaptureCard& parent)
 
 void CaptureCardGroup::triggerChanged(const QString& value) 
 {
-    QString own = (value == "MJPEG") ? "V4L" : value;
+    QString own = (value == "MJPEG" || value == "GO7007") ? "V4L" : value;
     TriggeredConfigurationGroup::triggerChanged(own);
 }
 
@@ -1016,31 +1033,42 @@ CaptureCard::CaptureCard()
     addChild(new Hostname(*this));
 }
 
-void CaptureCard::fillSelections(QSqlDatabase* db,
-                                 SelectSetting* setting) {
-    QString query = QString("SELECT cardtype, videodevice, cardid, firewire_port, firewire_node "
-                            "FROM capturecard WHERE hostname = \"%1\";")
-                            .arg(gContext->GetHostName());
-    QSqlQuery result = db->exec(query);
+void CaptureCard::fillSelections(SelectSetting* setting) 
+{
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("SELECT cardtype, videodevice, cardid, "
+                  " firewire_port, firewire_node "
+                  " FROM capturecard WHERE hostname = :HOSTNAME ;");
+    query.bindValue(":HOSTNAME", gContext->GetHostName());
 
-    if (result.isActive() && result.numRowsAffected() > 0)
-        while (result.next())
+    if (query.exec() && query.isActive() && query.size() > 0)
+    {
+        while (query.next())
+        { 
             // dont like doing this..
-            if(result.value(0).toString() == "FIREWIRE") {
-                     setting->addSelection("[ " + result.value(0).toString() + " Port: " +
-                                  result.value(3).toString() + ", Node: " +
-                                  result.value(4).toString() + "]",
-                                  result.value(2).toString());
-            } else {
-                     setting->addSelection("[ " + result.value(0).toString() + " : " +
-                                  result.value(1).toString() + " ]",
-                                  result.value(2).toString());
+            if(query.value(0).toString() == "FIREWIRE") 
+            {
+                setting->addSelection(
+                      "[ " + query.value(0).toString() + " Port: " +
+                             query.value(3).toString() + ", Node: " +
+                             query.value(4).toString() + "]",
+                             query.value(2).toString());
+            } 
+            else 
+            {
+                setting->addSelection(
+                       "[ " + query.value(0).toString() + " : " +
+                              query.value(1).toString() + " ]",
+                              query.value(2).toString());
             }
+        }
+    }   
 }
 
-void CaptureCard::loadByID(QSqlDatabase* db, int cardid) {
+void CaptureCard::loadByID(int cardid) 
+{
     id->setValue(cardid);
-    load(db);
+    load();
 }
 
 CardType::CardType(const CaptureCard& parent)
@@ -1065,6 +1093,7 @@ void CardType::fillSelections(SelectSetting* setting)
                           "DVB");
     setting->addSelection(QObject::tr("FireWire Input"),
                           "FIREWIRE");
+    setting->addSelection(QObject::tr("USB Mpeg-4 Encoder (Plextor ConvertX, etc)"), "GO7007");
 }
 
 class CardID: public SelectLabelSetting, public CISetting {
@@ -1074,13 +1103,13 @@ public:
         setLabel(QObject::tr("Capture device"));
     };
 
-    virtual void load(QSqlDatabase* db) {
-        fillSelections(db);
-        CISetting::load(db);
+    virtual void load() {
+        fillSelections();
+        CISetting::load();
     };
 
-    void fillSelections(QSqlDatabase* db) {
-        CaptureCard::fillSelections(db, this);
+    void fillSelections() {
+        CaptureCard::fillSelections(this);
     };
 };
 
@@ -1092,15 +1121,15 @@ public:
         addSelection(QObject::tr("(None)"), "0");
     };
 
-    virtual void load(QSqlDatabase* db) {
-        fillSelections(db);
-        CISetting::load(db);
+    virtual void load() {
+        fillSelections();
+        CISetting::load();
     };
 
-    void fillSelections(QSqlDatabase* db) {
+    void fillSelections() {
         clearSelections();
         addSelection(QObject::tr("(None)"), "0");
-        VideoSource::fillSelections(db, this);
+        VideoSource::fillSelections(this);
     };
 };
 
@@ -1252,11 +1281,10 @@ public:
         setHelpText("Select the LNB Settings for DVB-S cards. "
                     "For DVB-C and DVB-T you don't need to set these values. ");
     };
-    void save(QSqlDatabase* db) { (void)db; };
-    void load(QSqlDatabase* db) { (void)db; };
+    void save() {};
+    void load() {};
 
 private:
-    QSqlDatabase* db;
 };
 
 class CardInput: public ConfigurationWizard {
@@ -1269,9 +1297,9 @@ public:
         group->addChild(cardid = new CardID(*this));
         group->addChild(inputname = new InputName(*this));
         group->addChild(sourceid = new SourceID(*this));
+        group->addChild(new InputPreference(*this));
         if (!DVB)
         {
-            group->addChild(new InputPreference(*this));
             group->addChild(new ExternalChannelCommand(*this));
             group->addChild(new PresetTuner(*this));
         }
@@ -1292,13 +1320,13 @@ public:
 
     int getInputID(void) const { return id->intValue(); };
 
-    void loadByID(QSqlDatabase* db, int id);
-    void loadByInput(QSqlDatabase* db, int cardid, QString input);
+    void loadByID(int id);
+    void loadByInput(int cardid, QString input);
     QString getSourceName(void) const { return sourceid->getSelectionLabel(); };
 
     void fillDiseqcSettingsInput(QString _pos, QString _port);
 
-    virtual void save(QSqlDatabase* db);
+    virtual void save();
 
 private:
     class ID: virtual public IntegerSetting,
@@ -1320,7 +1348,6 @@ private:
     CardID* cardid;
     InputName* inputname;
     SourceID* sourceid;
-    QSqlDatabase* db;
     DVBLNBChooser *lnbsettings;
     DiseqcPos* diseqcpos;
     DiseqcPort* diseqcport;
@@ -1329,39 +1356,48 @@ private:
     LNBLofHi *lnblofhi;
 };
 
-QString CISetting::whereClause(void) {
+QString CISetting::whereClause(void) 
+{
     return QString("cardinputid = %1").arg(parent.getInputID());
 }
 
-QString CISetting::setClause(void) {
+QString CISetting::setClause(void) 
+{
     return QString("cardinputid = %1, %2 = '%3'")
         .arg(parent.getInputID())
         .arg(getColumn())
         .arg(getValue());
 }
 
-void CardInput::loadByID(QSqlDatabase* db, int inputid) {
+void CardInput::loadByID(int inputid) 
+{
     id->setValue(inputid);
-    load(db);
+    load();
 }
 
-void CardInput::loadByInput(QSqlDatabase* db, int _cardid, QString _inputname) {
-    QString query = QString("SELECT cardinputid FROM cardinput WHERE cardid = %1 AND inputname = '%2';")
-        .arg(_cardid)
-        .arg(_inputname);
-    QSqlQuery result = db->exec(query);
-    if (result.isActive() && result.numRowsAffected() > 0) {
-        result.next();
-        loadByID(db, result.value(0).toInt());
-    } else {
-        load(db); // new
+void CardInput::loadByInput(int _cardid, QString _inputname) 
+{
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("SELECT cardinputid FROM cardinput "
+                 "WHERE cardid = :CARDID AND inputname = :INPUTNAME ;");
+    query.bindValue(":CARDID", _cardid);
+    query.bindValue(":INPUTNAME", _inputname);
+
+    if (query.exec() && query.isActive() && query.size() > 0) 
+    {
+        query.next();
+        loadByID(query.value(0).toInt());
+    } 
+    else 
+    {
+        load(); // new
         cardid->setValue(QString::number(_cardid));
         inputname->setValue(_inputname);
     }
 
-    if (CardUtil::isDVB(db,_cardid))
+    if (CardUtil::isDVB(_cardid))
     {
-        int iVideoDev = CardUtil::videoDeviceFromCardID(db,_cardid);
+        int iVideoDev = CardUtil::videoDeviceFromCardID(_cardid);
         CardUtil::DVB_TYPES dvbType;
         if ((iVideoDev >= 0) &&
            ((dvbType = CardUtil::cardDVBType(iVideoDev))>CardUtil::ERROR_PROBE))
@@ -1373,7 +1409,7 @@ void CardInput::loadByInput(QSqlDatabase* db, int _cardid, QString _inputname) {
                 lnblofswitch->setVisible(true);
                 lnbloflo->setVisible(true);
                 lnblofhi->setVisible(true);
-                if (CardUtil::diseqcType(db,_cardid) == CardUtil::POSITIONER_X)
+                if (CardUtil::diseqcType(_cardid) == CardUtil::POSITIONER_X)
                     diseqcpos->setEnabled(true);
                 else
                     diseqcpos->setEnabled(false);
@@ -1389,40 +1425,54 @@ void CardInput::loadByInput(QSqlDatabase* db, int _cardid, QString _inputname) {
     }
 }
 
-void CardInput::save(QSqlDatabase* db) {
+void CardInput::save() 
+{
+
     if (sourceid->getValue() == "0")
+    {
         // "None" is represented by the lack of a row
-        db->exec(QString("DELETE FROM cardinput WHERE cardinputid = %1;").arg(getInputID()));
+        MSqlQuery query(MSqlQuery::InitCon());
+        query.prepare("DELETE FROM cardinput WHERE cardinputid = :INPUTID ;");
+        query.bindValue(":INPUTID", getInputID());
+        query.exec();
+    }
     else
-        ConfigurationWizard::save(db);
+    {
+        ConfigurationWizard::save();
+    }
 }
 
-void CardInput::fillDiseqcSettingsInput(QString _pos, QString _port) {
+void CardInput::fillDiseqcSettingsInput(QString _pos, QString _port) 
+{
     if (_port != "")
         diseqcport->setValue(_port);
     if (_pos != "")
         diseqcpos->setValue(_pos);
 }
 
-int CISetting::getInputID(void) const {
+int CISetting::getInputID(void) const 
+{
     return parent.getInputID();
 }
 
-int CCSetting::getCardID(void) const {
+int CCSetting::getCardID(void) const 
+{
     return parent.getCardID();
 }
 
-int CaptureCardEditor::exec(QSqlDatabase* db) {
-    while (ConfigurationDialog::exec(db) == QDialog::Accepted)
+int CaptureCardEditor::exec() 
+{
+    while (ConfigurationDialog::exec() == QDialog::Accepted)
         edit();
 
     return QDialog::Rejected;
 }
 
-void CaptureCardEditor::load(QSqlDatabase* db) {
+void CaptureCardEditor::load() 
+{
     clearSelections();
     addSelection(QObject::tr("(New capture card)"), "0");
-    CaptureCard::fillSelections(db, this);
+    CaptureCard::fillSelections(this);
 }
 
 MythDialog* CaptureCardEditor::dialogWidget(MythMainWindow* parent,
@@ -1440,7 +1490,7 @@ void CaptureCardEditor::menu(void)
     if (getValue().toInt() == 0) 
     {
         CaptureCard cc;
-        cc.exec(db);
+        cc.exec();
     } 
     else 
     {
@@ -1460,8 +1510,8 @@ void CaptureCardEditor::edit(void)
 {
     CaptureCard cc;
     if (getValue().toInt() != 0)
-        cc.loadByID(db,getValue().toInt());
-    cc.exec(db);
+        cc.loadByID(getValue().toInt());
+    cc.exec();
 }
 
 void CaptureCardEditor::del(void)
@@ -1473,18 +1523,19 @@ void CaptureCardEditor::del(void)
                                              tr("No, don't"), 2);
     if (val == 0)
     {
-        QSqlQuery query;
+        MSqlQuery query(MSqlQuery::InitCon());
 
-        query = db->exec(QString("DELETE FROM capturecard"
-                                 " WHERE cardid='%1'").arg(getValue()));
-        if (!query.isActive())
+        query.prepare("DELETE FROM capturecard WHERE cardid= :CARDID ;");
+        query.bindValue(":CARDID", getValue());
+        if (!query.exec() || !query.isActive())
             MythContext::DBError("Deleting Capture Card", query);
 
-        query = db->exec(QString("DELETE FROM cardinput"
-                                 " WHERE cardid='%1'").arg(getValue()));
-        if (!query.isActive())
+        query.prepare("DELETE FROM cardinput WHERE cardid= :CARDID ;");
+        query.bindValue(":CARDID", getValue());
+        if (!query.exec() || !query.isActive())
             MythContext::DBError("Deleting Card Input", query);
-        load(db);
+        
+        load();
     }
 }
 
@@ -1498,17 +1549,17 @@ MythDialog* VideoSourceEditor::dialogWidget(MythMainWindow* parent,
     return dialog;
 }
 
-int VideoSourceEditor::exec(QSqlDatabase* db) {
-    while (ConfigurationDialog::exec(db) == QDialog::Accepted)
+int VideoSourceEditor::exec() {
+    while (ConfigurationDialog::exec() == QDialog::Accepted)
         edit();
 
     return QDialog::Rejected;
 }
 
-void VideoSourceEditor::load(QSqlDatabase* db) {
+void VideoSourceEditor::load() {
     clearSelections();
     addSelection(QObject::tr("(New video source)"), "0");
-    VideoSource::fillSelections(db, this);
+    VideoSource::fillSelections(this);
 }
 
 void VideoSourceEditor::menu()
@@ -1516,7 +1567,7 @@ void VideoSourceEditor::menu()
     if (getValue().toInt() == 0) 
     {
         VideoSource vs;
-        vs.exec(db);
+        vs.exec();
     } 
     else 
     {
@@ -1537,9 +1588,9 @@ void VideoSourceEditor::edit()
 {
     VideoSource vs;
     if (getValue().toInt() != 0)
-        vs.loadByID(db,getValue().toInt());
+        vs.loadByID(getValue().toInt());
 
-    vs.exec(db);
+    vs.exec();
 }
 
 void VideoSourceEditor::del() 
@@ -1552,38 +1603,43 @@ void VideoSourceEditor::del()
 
     if (val == 0)
     {
-        QSqlQuery query = db->exec(QString("DELETE FROM videosource"
-                                           " WHERE sourceid='%1'")
-                                           .arg(getValue()));
-        if (!query.isActive())
+        MSqlQuery query(MSqlQuery::InitCon());
+        query.prepare("DELETE FROM videosource WHERE sourceid= :SOURCEID ;");
+        query.bindValue(":SOURCEID", getValue());
+
+        if (!query.exec() || !query.isActive())
             MythContext::DBError("Deleting VideoSource", query);
-        load(db);
+
+        load();
     }
 }
 
-int CardInputEditor::exec(QSqlDatabase* db) {
-    while (ConfigurationDialog::exec(db) == QDialog::Accepted)
-        cardinputs[getValue().toInt()]->exec(db);
+int CardInputEditor::exec() 
+{
+    while (ConfigurationDialog::exec() == QDialog::Accepted)
+        cardinputs[getValue().toInt()]->exec();
 
     return QDialog::Rejected;
 }
 
 
 
-void CardInputEditor::load(QSqlDatabase* db) {
+void CardInputEditor::load() 
+{
     clearSelections();
 
     // We do this manually because we want custom labels.  If
     // SelectSetting provided a facility to edit the labels, we
     // could use CaptureCard::fillSelections
 
-    QString thequery = QString("SELECT cardid, videodevice, cardtype, dvb_diseqc_type, "
-                               "firewire_port, firewire_node "
-                               "FROM capturecard WHERE hostname = \"%1\";")
-                              .arg(gContext->GetHostName());
+    MSqlQuery capturecards(MSqlQuery::InitCon());
+    
+    capturecards.prepare("SELECT cardid, videodevice, cardtype, "
+                         "dvb_diseqc_type, firewire_port, firewire_node "
+                         "FROM capturecard WHERE hostname = :HOSTNAME ;");
+    capturecards.bindValue(":HOSTNAME", gContext->GetHostName());
 
-    QSqlQuery capturecards = db->exec(thequery);
-    if (capturecards.isActive() && capturecards.numRowsAffected() > 0)
+    if (capturecards.exec() && capturecards.isActive() && capturecards.size() > 0)
         while (capturecards.next()) {
             int cardid = capturecards.value(0).toInt();
             QString videodevice(capturecards.value(1).toString());
@@ -1599,7 +1655,7 @@ void CardInputEditor::load(QSqlDatabase* db) {
                 {
                     // IS DVB Check for CardInput class
                     CardInput* cardinput = new CardInput(1);
-                    cardinput->loadByInput(db, cardid, (*it).input);
+                    cardinput->loadByInput(cardid, (*it).input);
 
                     cardinput->fillDiseqcSettingsInput((*it).position,(*it).port);
 
@@ -1622,7 +1678,7 @@ void CardInputEditor::load(QSqlDatabase* db) {
                      i != inputs.end(); ++i)
                 { 
                     CardInput* cardinput = new CardInput();
-                    cardinput->loadByInput(db, cardid, *i);   
+                    cardinput->loadByInput(cardid, *i);   
                     cardinputs.push_back(cardinput);
                     QString index = QString::number(cardinputs.size()-1);
 
@@ -1644,7 +1700,7 @@ void CardInputEditor::load(QSqlDatabase* db) {
                      i != inputs.end(); ++i)
                 {
                     CardInput* cardinput = new CardInput();
-                    cardinput->loadByInput(db, cardid, *i);
+                    cardinput->loadByInput(cardid, *i);
                     cardinputs.push_back(cardinput);
                     QString index = QString::number(cardinputs.size()-1);
 
@@ -1709,6 +1765,28 @@ QValueList<DVBDiseqcInputList> VideoDevice::fillDVBInputsDiseqc(int dvb_diseqc_t
             for (int x=1;x<20;x++)
                 list.append ( DVBDiseqcInputList(QString("DiSEqC v1.3 Input %1").arg(x),QString(""),QString("%1").arg(x)));
             break;
+        case 8:
+            list.append(DVBDiseqcInputList(QString("DiSEqC Switch Input  1"),
+                                           QString("0"), QString("")));
+            list.append(DVBDiseqcInputList(QString("DiSEqC Switch Input  2"),
+                                           QString("1"), QString("")));
+            list.append(DVBDiseqcInputList(QString("DiSEqC Switch Input  3"),
+                                           QString("2"), QString("")));
+            list.append(DVBDiseqcInputList(QString("DiSEqC Switch Input  4"),
+                                           QString("3"), QString("")));
+            list.append(DVBDiseqcInputList(QString("DiSEqC Switch Input  5"),
+                                           QString("4"), QString("")));
+            list.append(DVBDiseqcInputList(QString("DiSEqC Switch Input  6"),
+                                           QString("5"), QString("")));
+            list.append(DVBDiseqcInputList(QString("DiSEqC Switch Input  7"),
+                                           QString("6"), QString("")));
+            list.append(DVBDiseqcInputList(QString("DiSEqC Switch Input  8"),
+                                           QString("7"), QString("")));
+            list.append(DVBDiseqcInputList(QString("DiSEqC Switch Input  9"),
+                                           QString("8"), QString("")));
+            list.append(DVBDiseqcInputList(QString("DiSEqC Switch Input 10"),
+                                           QString("9"), QString("")));
+            break;
         default:
             list.append ( DVBDiseqcInputList(QString("DVBInput"),QString(""),QString("")));
     }
@@ -1901,8 +1979,8 @@ void CaptureCard::execDVBConfigMenu() {
 
         if (val != 0)
             return;
-        save(db);
-        load(db);
+        save();
+        load();
     }
 
     MythPopupBox *popup = new MythPopupBox(gContext->GetMainWindow(),
@@ -1925,7 +2003,7 @@ public:
         VerticalConfigurationGroup* rec = new VerticalConfigurationGroup(false);        rec->setLabel(QObject::tr("Recorder Options"));
         rec->setUseLabel(false);
 
-        rec->addChild(new DVBSwFilter(parent));
+        rec->addChild(new DVBHwDecoder(parent));
         rec->addChild(new DVBRecordTS(parent));
         rec->addChild(new DVBNoSeqStart(parent));
         rec->addChild(new DVBOnDemand(parent));
@@ -1975,11 +2053,11 @@ public:
 void CaptureCard::execAdvConfigWiz()
 {
     DVBAdvConfigurationWizard acw(*this);
-    acw.exec(db);
+    acw.exec();
 }
 
 void CaptureCard::execDiseqcWiz()
 {
     DVBDiseqcConfigurationWizard diseqcWiz;
-    diseqcWiz.exec(db);
+    diseqcWiz.exec();
 }
