@@ -31,11 +31,9 @@ using namespace std;
 #include "remoteutil.h"
 #include "libmyth/mythdbcon.h"
 
-Scheduler::Scheduler(bool runthread, QMap<int, EncoderLink *> *tvList, 
-                     QSqlDatabase *ldb)
+Scheduler::Scheduler(bool runthread, QMap<int, EncoderLink *> *tvList)
 {
     hasconflicts = false;
-    db = ldb;
     m_tvList = tvList;
 
     m_mainServer = NULL;
@@ -75,13 +73,12 @@ void Scheduler::verifyCards(void)
 {
     QString thequery;
 
-    thequery = "SELECT NULL FROM capturecard;";
-
-    MSqlQuery query = db->exec(thequery);
+    MSqlQuery query(MSqlQuery::SchedCon());
+    query.prepare("SELECT NULL FROM capturecard;");
 
     int numcards = -1;
-    if (query.isActive())
-        numcards = query.numRowsAffected();
+    if (query.exec() && query.isActive())
+        numcards = query.size();
 
     if (numcards <= 0)
     {
@@ -90,25 +87,25 @@ void Scheduler::verifyCards(void)
         exit(5);
     }
 
-    thequery = "SELECT sourceid,name FROM videosource ORDER BY sourceid;";
-
-    query = db->exec(thequery);
+    query.prepare("SELECT sourceid,name FROM videosource ORDER BY sourceid;");
 
     int numsources = -1;
-    if (query.isActive())
+    if (query.exec() && query.isActive())
     {
-        numsources = query.numRowsAffected();
+        numsources = query.size();
 
         int source = 0;
 
         while (query.next())
         {
             source = query.value(0).toInt();
+            MSqlQuery subquery(MSqlQuery::SchedCon());
 
             thequery = QString("SELECT cardinputid FROM cardinput WHERE "
                                "sourceid = %1 ORDER BY cardinputid;")
                               .arg(source);
-            MSqlQuery subquery = db->exec(thequery);
+            subquery.prepare(thequery);
+            subquery.exec();
             
             if (!subquery.isActive() || subquery.numRowsAffected() <= 0)
                 cerr << query.value(1).toString() << " is defined, but isn't "
@@ -136,10 +133,14 @@ static bool comp_overlap(ProgramInfo *a, ProgramInfo *b)
         return a->endts < b->endts;
 
     // Note: the PruneOverlaps logic depends on the following
+    if (a->title != b->title)
+        return a->title < b->title;
     if (a->chanid != b->chanid)
         return a->chanid < b->chanid;
     if (a->inputid != b->inputid)
         return a->inputid < b->inputid;
+    if (a->findid != b->findid)
+        return a->findid > b->findid;
     return a->recordid < b->recordid;
 }
 
@@ -151,6 +152,8 @@ static bool comp_recstart(ProgramInfo *a, ProgramInfo *b)
         return a->recendts < b->recendts;
 
     // Note: the PruneRedundants logic depends on the following
+    if (a->title != b->title)
+        return a->title < b->title;
     if (a->recordid != b->recordid)
         return a->recordid < b->recordid;
     if (a->chansign != b->chansign)
@@ -232,8 +235,6 @@ bool Scheduler::FillRecordList(void)
     schedMoveHigher = (bool)gContext->GetNumSetting("SchedMoveHigher");
     schedTime = QDateTime::currentDateTime();
 
-    MythContext::KickDatabase(db);
-
     VERBOSE(VB_SCHEDULE, "PruneOldRecords...");
     PruneOldRecords();
     VERBOSE(VB_SCHEDULE, "AddNewRecords...");
@@ -252,14 +253,11 @@ bool Scheduler::FillRecordList(void)
     VERBOSE(VB_SCHEDULE, "SchedNewRecords...");
     SchedNewRecords();
 
-    if (schedMoveHigher)
-    {
-        VERBOSE(VB_SCHEDULE, "Sort retrylist...");
-        retrylist.sort(comp_retry);
-        VERBOSE(VB_SCHEDULE, "MoveHigherRecords...");
-        MoveHigherRecords();
-        retrylist.clear();
-    }
+    VERBOSE(VB_SCHEDULE, "Sort retrylist...");
+    retrylist.sort(comp_retry);
+    VERBOSE(VB_SCHEDULE, "MoveHigherRecords...");
+    MoveHigherRecords();
+    retrylist.clear();
 
     VERBOSE(VB_SCHEDULE, "ClearListMaps...");
     ClearListMaps();
@@ -274,10 +272,11 @@ bool Scheduler::FillRecordList(void)
 
 void Scheduler::FillRecordListFromDB(void)
 {
-    MSqlQuery query(QString::null, db);
+    MSqlQuery query(MSqlQuery::SchedCon());
     query.prepare("CREATE TEMPORARY TABLE recordmatch "
                   "(recordid int unsigned, chanid int unsigned, "
-                  " starttime datetime, INDEX (recordid));");
+                  " starttime datetime, manualid int unsigned, "
+                  " INDEX (recordid));");
     query.exec();
     if (!query.isActive())
     {
@@ -366,6 +365,7 @@ void Scheduler::UpdateRecStatus(ProgramInfo *pginfo)
         if (p->IsSameProgramTimeslot(*pginfo))
         {
             p->recstatus = pginfo->recstatus;
+            ScheduledRecording::signalChange(0);
             return;
         }
     }
@@ -464,6 +464,7 @@ void Scheduler::BuildListMaps(void)
         {
             cardlistmap[p->cardid].push_back(p);
             titlelistmap[p->title].push_back(p);
+            recordidlistmap[p->recordid].push_back(p);
         }
     }
 }
@@ -472,6 +473,7 @@ void Scheduler::ClearListMaps(void)
 {
     cardlistmap.clear();
     titlelistmap.clear();
+    recordidlistmap.clear();
 }
 
 bool Scheduler::FindNextConflict(RecList &cardlist, ProgramInfo *p, RecIter &j)
@@ -500,10 +502,31 @@ bool Scheduler::FindNextConflict(RecList &cardlist, ProgramInfo *p, RecIter &j)
     return false;
 }
 
-void Scheduler::MarkOtherShowings(RecList &titlelist, ProgramInfo *p)
+void Scheduler::MarkOtherShowings(ProgramInfo *p)
 {
-    RecIter i = titlelist.begin();
-    for ( ; i != titlelist.end(); i++)
+    RecList &showinglist = titlelistmap[p->title];
+
+    MarkShowingsList(showinglist, p);
+
+    if (p->rectype == kFindOneRecord || 
+        p->rectype == kFindDailyRecord ||
+        p->rectype == kFindWeeklyRecord)
+    {
+        showinglist = recordidlistmap[p->recordid];
+        MarkShowingsList(showinglist, p);
+    }
+
+    if (p->rectype == kOverrideRecord && p->findid > 0)
+    {
+        showinglist = recordidlistmap[p->parentid];
+        MarkShowingsList(showinglist, p);
+    }
+}
+
+void Scheduler::MarkShowingsList(RecList &showinglist, ProgramInfo *p)
+{
+    RecIter i = showinglist.begin();
+    for ( ; i != showinglist.end(); i++)
     {
         ProgramInfo *q = *i;
         if (q == p)
@@ -547,13 +570,20 @@ void Scheduler::RestoreRecStatus(void)
     }
 }
 
-bool Scheduler::TryAnotherShowing(RecList &titlelist, ProgramInfo *p)
+bool Scheduler::TryAnotherShowing(ProgramInfo *p)
 {
     if (p->recstatus == rsRecording)
         return false;
 
-    RecIter j = titlelist.begin();
-    for ( ; j != titlelist.end(); j++)
+    RecList &showinglist = titlelistmap[p->title];
+
+    if (p->rectype == kFindOneRecord || 
+        p->rectype == kFindDailyRecord ||
+        p->rectype == kFindWeeklyRecord)
+        showinglist = recordidlistmap[p->recordid];
+
+    RecIter j = showinglist.begin();
+    for ( ; j != showinglist.end(); j++)
     {
         ProgramInfo *q = *j;
         if (q == p)
@@ -562,7 +592,7 @@ bool Scheduler::TryAnotherShowing(RecList &titlelist, ProgramInfo *p)
 
     p->recstatus = rsLaterShowing;
 
-    for (j++; j != titlelist.end(); j++)
+    for (j++; j != showinglist.end(); j++)
     {
         ProgramInfo *q = *j;
         if (q->recstatus != rsEarlierShowing &&
@@ -583,7 +613,7 @@ bool Scheduler::TryAnotherShowing(RecList &titlelist, ProgramInfo *p)
             continue;
 
         q->recstatus = rsWillRecord;
-        MarkOtherShowings(titlelist, q);
+        MarkOtherShowings(q);
         PrintRec(p, "     -");
         PrintRec(q, "     +");
         return true;
@@ -601,12 +631,9 @@ void Scheduler::SchedNewRecords(void)
     for ( ; i != reclist.end(); i++)
     {
         ProgramInfo *p = *i;
-        if (p->recstatus == rsRecording ||
-            p->recstatus == rsWillRecord)
-        {
-            RecList &titlelist = titlelistmap[p->title];
-            MarkOtherShowings(titlelist, p);
-        }
+        if (p->recstatus == rsRecording || p->recstatus == rsWillRecord)
+            MarkOtherShowings(p);
+
         if (p->recstatus != rsUnknown)
             continue;
 
@@ -614,12 +641,11 @@ void Scheduler::SchedNewRecords(void)
         RecIter k = cardlist.begin();
         if (!FindNextConflict(cardlist, p, k))
         {
-            RecList &titlelist = titlelistmap[p->title];
             p->recstatus = rsWillRecord;
-            MarkOtherShowings(titlelist, p);
+            MarkOtherShowings(p);
             PrintRec(p, "  +");
         }
-        else if (schedMoveHigher)
+        else
         {
             for (k++; !FindNextConflict(cardlist, p, k); k++)
                 ;
@@ -643,15 +669,14 @@ void Scheduler::MoveHigherRecords(void)
 
         BackupRecStatus();
         p->recstatus = rsWillRecord;
-        RecList &titlelist = titlelistmap[p->title];
-        MarkOtherShowings(titlelist, p);
+        MarkOtherShowings(p);
 
         RecList &cardlist = cardlistmap[p->cardid];
         RecIter k = cardlist.begin();
         for ( ; FindNextConflict(cardlist, p, k); k++)
         {
-            RecList &ktitlelist = titlelistmap[(*k)->title];
-            if (!TryAnotherShowing(ktitlelist, *k))
+            if ((p->recpriority < (*k)->recpriority && !schedMoveHigher) ||
+                !TryAnotherShowing(*k))
             {
                 RestoreRecStatus();
                 break;
@@ -1076,7 +1101,7 @@ void Scheduler::RunScheduler(void)
                 .arg(nextRecording->sourceid);
             VERBOSE(VB_GENERAL, msg.local8Bit());
             gContext->LogEntry("scheduler", LP_NOTICE, "Schedule Change", 
-                               msg.utf8());
+                               msg);
         }
 
         if (statuschanged)
@@ -1270,11 +1295,113 @@ void *Scheduler::SchedulerThread(void *param)
     return NULL;
 }
 
+void Scheduler::UpdateManuals(int recordid)
+{
+    MSqlQuery query(MSqlQuery::SchedCon());
+
+    query.prepare("SELECT type,title,station,startdate,starttime, "
+                  " enddate,endtime "
+                  "FROM record WHERE recordid = :RECORDID");
+    query.bindValue(":RECORDID", recordid);
+    query.exec();
+    if (!query.isActive() || query.numRowsAffected() != 1)
+    {
+        MythContext::DBError("UpdateManuals", query);
+        return;
+    }
+
+    query.next();
+    RecordingType rectype = RecordingType(query.value(0).toInt());
+    QString title = query.value(1).toString();
+    QString station = query.value(2).toString() ;
+    QDateTime startdt = QDateTime(query.value(3).asDate(),
+                                  query.value(4).asTime());
+    int duration = startdt.secsTo(QDateTime(query.value(5).asDate(),
+                                            query.value(6).asTime())) / 60;
+
+    query.prepare("SELECT chanid from channel "
+                  "WHERE callsign = :STATION");
+    query.bindValue(":STATION", station);
+    query.exec();
+    if (!query.isActive())
+    {
+        MythContext::DBError("UpdateManuals", query);
+        return;
+    }
+
+    QValueList<int> chanidlist;
+    while (query.next())
+        chanidlist.append(query.value(0).toInt());
+
+    int progcount;
+    int skipdays;
+    bool weekday;
+    int weeksoff;
+
+    switch (rectype)
+    {
+    case kSingleRecord:
+    case kOverrideRecord:
+    case kDontRecord:
+        progcount = 1;
+        skipdays = 1;
+        weekday = false;
+        break;
+    case kTimeslotRecord:
+        progcount = 13;
+        skipdays = 1;
+        if (startdt.date().dayOfWeek() < 6)
+            weekday = true;
+        else
+            weekday = false;
+        startdt.setDate(QDate::currentDate());
+        break;
+    case kWeekslotRecord:
+        progcount = 2;
+        skipdays = 7;
+        weekday = false;
+        weeksoff = (startdt.date().daysTo(QDate::currentDate()) + 6) / 7;
+        startdt = startdt.addDays(weeksoff * 7);
+        break;
+    default:
+        VERBOSE(VB_IMPORTANT, QString("Invalid rectype for manual "
+                                      "recordid %1").arg(recordid));
+        return;
+    }
+
+    while (progcount--)
+    {
+        for (int i = 0; i < (int)chanidlist.size(); i++)
+        {
+            if (weekday && startdt.date().dayOfWeek() >= 6)
+                continue;
+
+            query.prepare("INSERT INTO program (chanid,starttime,endtime,"
+                          " title,subtitle,manualid) "
+                          "VALUES (:CHANID,:STARTTIME,:ENDTIME,:TITLE,"
+                          " :SUBTITLE,:RECORDID)");
+            query.bindValue(":CHANID", chanidlist[i]);
+            query.bindValue(":STARTTIME", startdt);
+            query.bindValue(":ENDTIME", startdt.addSecs(duration * 60));
+            query.bindValue(":TITLE", title);
+            query.bindValue(":SUBTITLE", startdt.toString());
+            query.bindValue(":RECORDID", recordid);
+            query.exec();
+            if (!query.isActive())
+            {
+                MythContext::DBError("UpdateManuals", query);
+                return;
+            }
+        }
+        startdt = startdt.addDays(skipdays);
+    }
+}
+
 void Scheduler::BuildNewRecordsQueries(int recordid, QStringList &from, 
                                        QStringList &where)
 {
+    MSqlQuery result(MSqlQuery::SchedCon());
     QString query;
-    MSqlQuery result;
     QString qphrase;
 
     query = QString("SELECT recordid,search,subtitle,description "
@@ -1282,8 +1409,9 @@ void Scheduler::BuildNewRecordsQueries(int recordid, QStringList &from,
                     "(recordid = %2 OR %3 = -1) ")
         .arg(kNoSearch).arg(recordid).arg(recordid);
 
-    result = db->exec(query);
-    if (!result.isActive())
+    result.prepare(query);
+
+    if (!result.exec() || !result.isActive())
     {
         MythContext::DBError("BuildNewRecordsQueries", result);
         return;
@@ -1294,24 +1422,29 @@ void Scheduler::BuildNewRecordsQueries(int recordid, QStringList &from,
         qphrase = result.value(3).toString();
         qphrase.replace("\'", "\\\'");
 
-        if (qphrase == "")
+        RecSearchType searchtype = RecSearchType(result.value(1).toInt());
+
+        if (qphrase == "" && searchtype != kManualSearch)
         {
             VERBOSE(VB_IMPORTANT, QString("Invalid search key in recordid %1")
                                          .arg(result.value(0).toString()));
             continue;
         }
 
-        switch (result.value(1).toInt())
+        switch (searchtype)
         {
         case kPowerSearch:
+            qphrase.remove(QRegExp("^\\s*AND\\s+", false));
             from << result.value(2).toString();
-            where << QString("record.recordid = %1 %2")
+            where << QString("record.recordid = %1 AND "
+                             "program.manualid = 0 AND ( %2 )")
                 .arg(result.value(0).toString())
                 .arg(qphrase);
             break;
         case kTitleSearch:
             from << "";
             where << QString("record.recordid = %1 AND "
+                             "program.manualid = 0 AND "
                              "program.title LIKE '\%%2\%'")
                 .arg(result.value(0).toString())
                 .arg(qphrase);
@@ -1319,6 +1452,7 @@ void Scheduler::BuildNewRecordsQueries(int recordid, QStringList &from,
         case kKeywordSearch:
             from << "";
             where << QString("record.recordid = %1 AND "
+                             "program.manualid = 0 AND "
                              "(program.title LIKE '\%%2\%' OR "
                              " program.subtitle LIKE '\%%3\%' OR "
                              " program.description LIKE '\%%4\%')")
@@ -1328,15 +1462,23 @@ void Scheduler::BuildNewRecordsQueries(int recordid, QStringList &from,
         case kPeopleSearch:
             from << ", people, credits";
             where << QString("record.recordid = %1 AND "
-                             "people.name LIKE '\%%2\%' AND "
+                             "program.manualid = 0 AND "
+                             "people.name LIKE '%2' AND "
                              "credits.person = people.person AND "
                              "program.chanid = credits.chanid AND "
                              "program.starttime = credits.starttime")
                 .arg(result.value(0).toString())
                 .arg(qphrase);
             break;
+        case kManualSearch:
+            UpdateManuals(result.value(0).toInt());
+            from << "";
+            where << QString("record.recordid = %1 AND "
+                             "program.manualid = record.recordid ")
+                .arg(result.value(0).toString());
+            break;
         default:
-            VERBOSE(VB_SCHEDULE, QString("Unknown RecSearchType "
+            VERBOSE(VB_IMPORTANT, QString("Unknown RecSearchType "
                                          "(%1) for recordid %2")
                                          .arg(result.value(1).toInt())
                                          .arg(result.value(0).toString()));
@@ -1349,6 +1491,7 @@ void Scheduler::BuildNewRecordsQueries(int recordid, QStringList &from,
         from << "";
         where << QString("record.search = %1 AND "
                          "(record.recordid = %2 OR %3 = -1) AND "
+                         "program.manualid = 0 AND "
                          "program.title = record.title ")
             .arg(kNoSearch).arg(recordid).arg(recordid);
     }
@@ -1357,14 +1500,23 @@ void Scheduler::BuildNewRecordsQueries(int recordid, QStringList &from,
 void Scheduler::UpdateMatches(int recordid) {
     struct timeval dbstart, dbend;
 
-    MythContext::KickDatabase(db);
-    MSqlQuery query;
-    query.prepare("DELETE FROM recordmatch WHERE "
-                  "recordid = :RECORDID OR :RECORDID = -1;");
+    MSqlQuery query(MSqlQuery::SchedCon());
+    query.prepare("DELETE FROM recordmatch "
+                  "WHERE recordid = :RECORDID OR :RECORDID = -1;");
+
     query.bindValue(":RECORDID", recordid);
-
     query.exec();
+    if (!query.isActive())
+    {
+        MythContext::DBError("UpdateMatches", query);
+        return;
+    }
 
+    query.prepare("DELETE FROM program "
+                  "WHERE manualid = :RECORDID OR "
+                  " (manualid <> 0 AND :RECORDID = -1)");
+    query.bindValue(":RECORDID", recordid);
+    query.exec();
     if (!query.isActive())
     {
         MythContext::DBError("UpdateMatches", query);
@@ -1386,9 +1538,10 @@ void Scheduler::UpdateMatches(int recordid) {
     for (clause = 0; clause < fromclauses.count(); clause++)
     {
         QString query = QString(
-"INSERT INTO recordmatch (recordid, chanid, starttime) "
-"SELECT record.recordid, program.chanid, program.starttime "
-"FROM record, program ") + fromclauses[clause] + QString(
+"INSERT INTO recordmatch (recordid, chanid, starttime, manualid) "
+"SELECT record.recordid, program.chanid, program.starttime, "
+" IF(search = %1, recordid, 0) "
+"FROM record, program ").arg(kManualSearch) + fromclauses[clause] + QString(
 " INNER JOIN channel ON (channel.chanid = program.chanid) "
 "WHERE ") + whereclauses[clause] + QString(" AND "
 "((record.type = %1 " // allrecord
@@ -1433,13 +1586,15 @@ void Scheduler::UpdateMatches(int recordid) {
         VERBOSE(VB_SCHEDULE, QString(" |-- Start DB Query %1...").arg(clause));
 
         gettimeofday(&dbstart, NULL);
-        MSqlQuery result = db->exec(query);
+        MSqlQuery result(MSqlQuery::SchedCon());
+        result.prepare(query);
+        result.exec();
         gettimeofday(&dbend, NULL);
 
         if (!result.isActive())
         {
             MythContext::DBError("UpdateMatches", result);
-            return;
+            continue;
         }
 
         VERBOSE(VB_SCHEDULE, QString(" |-- %1 results in %2 sec.")
@@ -1471,7 +1626,7 @@ void Scheduler::AddNewRecords(void) {
     QMap<int, bool> tooManyMap;
     bool checkTooMany = false;
 
-    MSqlQuery rlist(QString::null, db);
+    MSqlQuery rlist(MSqlQuery::SchedCon());
     rlist.prepare("SELECT recordid,title,maxepisodes,maxnewest FROM record;");
 
     rlist.exec();
@@ -1493,7 +1648,7 @@ void Scheduler::AddNewRecords(void) {
 
         if (maxEpisodes && !maxNewest)
         {
-            MSqlQuery epicnt(QString::null, db);
+            MSqlQuery epicnt(MSqlQuery::SchedCon());
 
             epicnt.prepare("SELECT count(*) FROM recorded "
                            "WHERE title = :TITLE;");
@@ -1541,21 +1696,22 @@ void Scheduler::AddNewRecords(void) {
 "cardinput.preference, "
 "record.dupin, "
 "recorded.endtime IS NOT NULL AND recorded.endtime < NOW() AS recduplicate, "
-"record.type, record.recordid, 0, "
+"oldfind.findid IS NOT NULL AS findduplicate, "
+"record.type, record.recordid, "
 "program.starttime - INTERVAL record.startoffset minute AS recstartts, "
 "program.endtime + INTERVAL record.endoffset minute AS recendts, "
 "program.previouslyshown, record.recgroup, record.dupmethod, "
 "channel.commfree, capturecard.cardid, "
 "cardinput.cardinputid, UPPER(cardinput.shareable) = 'Y' AS shareable, "
 "program.seriesid, program.programid, program.category_type, "
-"program.airdate, program.stars, program.originalairdate, record.inactive, ")
-+ progfindid + QString(
-
+"program.airdate, program.stars, program.originalairdate, record.inactive, "
+"record.parentid, ") + progfindid + QString(
 "FROM recordmatch "
 
 " INNER JOIN record ON (recordmatch.recordid = record.recordid) "
 " INNER JOIN program ON (recordmatch.chanid = program.chanid AND "
-"                        recordmatch.starttime = program.starttime) "
+"                        recordmatch.starttime = program.starttime AND "
+"                        recordmatch.manualid = program.manualid) "
 " INNER JOIN channel ON (channel.chanid = program.chanid) "
 " INNER JOIN cardinput ON (channel.sourceid = cardinput.sourceid) "
 " INNER JOIN capturecard ON (capturecard.cardid = cardinput.cardid) "
@@ -1606,12 +1762,18 @@ void Scheduler::AddNewRecords(void) {
 "          AND program.description = recorded.description)) "
 "      ) "
 "     ) "
-"  ) ");
+"  ) "
+" LEFT JOIN oldfind ON "
+"  (oldfind.recordid = recordmatch.recordid AND "
+"   oldfind.findid = ") + progfindid + QString(") "
+);
 
     VERBOSE(VB_SCHEDULE, QString(" |-- Start DB Query..."));
 
     gettimeofday(&dbstart, NULL);
-    MSqlQuery result = db->exec(query);
+    MSqlQuery result(MSqlQuery::SchedCon());
+    result.prepare(query);
+    result.exec();
     gettimeofday(&dbend, NULL);
 
     if (!result.isActive())
@@ -1651,8 +1813,8 @@ void Scheduler::AddNewRecords(void) {
         p->recpriority = result.value(12).toInt();
         p->dupin = RecordingDupInType(result.value(13).toInt());
         p->dupmethod = RecordingDupMethodType(result.value(22).toInt());
-        p->rectype = RecordingType(result.value(15).toInt());
-        p->recordid = result.value(16).toInt();
+        p->rectype = RecordingType(result.value(16).toInt());
+        p->recordid = result.value(17).toInt();
 
         p->recstartts = result.value(18).toDateTime();
         p->recendts = recendts;
@@ -1669,12 +1831,19 @@ void Scheduler::AddNewRecords(void) {
         p->stars =  result.value(31).toDouble();
 
         if(result.value(32).isNull())
+        {
             p->originalAirDate = p->startts.date();
+            p->hasAirDate = false;
+        }
         else
-            p->originalAirDate = QDate::fromString(result.value(30).toString(), Qt::ISODate);
+        {
+            p->originalAirDate = QDate::fromString(result.value(32).toString(), Qt::ISODate);
+            p->hasAirDate = true;
+        }
 
         bool inactive = result.value(33).toInt();
-        p->findid = result.value(34).toInt();
+        p->parentid = result.value(34).toInt();
+        p->findid = result.value(35).toInt();
 
         if (!recTypeRecPriorityMap.contains(p->rectype))
             recTypeRecPriorityMap[p->rectype] = 
@@ -1733,11 +1902,14 @@ void Scheduler::AddNewRecords(void) {
         {
             if (p->dupin == kDupsNewEpi && p->repeat)
                 p->recstatus = rsRepeat;
-        
+
+            if (result.value(15).toInt())
+                p->recstatus = rsPreviousRecording;
+
             if (((p->dupin & kDupsInOldRecorded) || (p->dupin == kDupsNewEpi)) &&
                 result.value(10).toInt())
                 p->recstatus = rsPreviousRecording;
-            
+
             if (((p->dupin & kDupsInRecorded) || (p->dupin == kDupsNewEpi)) &&
                 result.value(14).toInt())
                 p->recstatus = rsCurrentRecording;
@@ -1783,9 +1955,10 @@ void Scheduler::findAllScheduledPrograms(list<ProgramInfo *> &proglist)
 "GROUP BY recordid "
 "ORDER BY title ASC;");
 
-    MSqlQuery result = db->exec(query);
+    MSqlQuery result(MSqlQuery::SchedCon());
+    result.prepare(query);
 
-    if (result.isActive() && result.numRowsAffected() > 0)
+    if (result.exec() && result.isActive() && result.size() > 0)
         while (result.next()) 
         {
             ProgramInfo *proginfo = new ProgramInfo;
