@@ -2,11 +2,11 @@
 #define SCHEDULER_H_
 
 class ProgramInfo;
-class QSqlDatabase;
 class EncoderLink;
 class MainServer;
 
 #include <qmutex.h>
+#include <qwaitcondition.h>
 #include <qmap.h> 
 #include <list>
 #include <vector>
@@ -22,12 +22,11 @@ typedef RecList::iterator RecIter;
 class Scheduler : public QObject
 {
   public:
-    Scheduler(bool runthread, QMap<int, EncoderLink *> *tvList, 
-              QSqlDatabase *ldb);
-   ~Scheduler();
+    Scheduler(bool runthread, QMap<int, EncoderLink *> *tvList);
+    ~Scheduler();
 
-    bool CheckForChanges(void);
-    bool FillRecordLists(void);
+    void Reschedule(int recordid);
+    void FillRecordListFromDB(void);
     void FillRecordListFromMaster(void);
 
     void FillEncoderFreeSpaceCache(void);
@@ -59,17 +58,21 @@ class Scheduler : public QObject
   private:
     void verifyCards(void);
 
+    bool FillRecordList(void);
+    void UpdateMatches(int recordid);
+    void UpdateManuals(int recordid);
     void PruneOldRecords(void);
     void AddNewRecords(void);
-    void BuildNewRecordsQueries(QStringList &from, QStringList &where);
+    void BuildNewRecordsQueries(int recordid, QStringList &from, QStringList &where);
     void PruneOverlaps(void);
     void BuildListMaps(void);
     void ClearListMaps(void);
     bool FindNextConflict(RecList &cardlist, ProgramInfo *p, RecIter &iter);
-    void MarkOtherShowings(RecList &titlelist, ProgramInfo *p);
+    void MarkOtherShowings(ProgramInfo *p);
+    void MarkShowingsList(RecList &showinglist, ProgramInfo *p);
     void BackupRecStatus(void);
     void RestoreRecStatus(void);
-    bool TryAnotherShowing(RecList &titlelist, ProgramInfo *p);
+    bool TryAnotherShowing(ProgramInfo *p);
     void SchedNewRecords(void);
     void MoveHigherRecords(void);
     void PruneRedundants(void);
@@ -80,12 +83,15 @@ class Scheduler : public QObject
                              bool &blockShutdown);
     void ShutdownServer(int prerollseconds);
 
-    QSqlDatabase *db;
 
+    QValueList<int> reschedQueue;
+    QMutex reschedLock;
+    QWaitCondition reschedWait;
     RecList reclist;
     RecList retrylist;
     RecList schedlist;
     QMap<int, RecList> cardlistmap;
+    QMap<int, RecList> recordidlistmap;
     QMap<QString, RecList> titlelistmap;
 
     QMutex *reclist_lock;
