@@ -41,6 +41,7 @@ class PlaybackBox : public MythDialog
     void cursorUp(bool page = false, bool newview = false);
     void pageDown() { cursorDown(true); }
     void pageUp() { cursorUp(true); }
+    void upcoming();
     void details();
     void selected();
     void playSelected();
@@ -52,8 +53,16 @@ class PlaybackBox : public MythDialog
     void showRecGroupChanger();
     void showRecGroupChooser();
     void showRecGroupPasswordChanger();
+    void showPlayFromPopup();
+    void showRecordingPopup();
+    void showJobPopup();
+    void showStoragePopup();
+    void showPlaylistPopup();
+    void showPlaylistJobPopup();
 
     void doPlay();
+    void doPlayFromBeg();
+    void doPlayListRandom();
 
     void askStop();
     void doStop();
@@ -68,8 +77,11 @@ class PlaybackBox : public MythDialog
 
     void doAutoExpire();
     void noAutoExpire();
+    void doPreserveEpisode();
+    void noPreserveEpisode();
 
     void doCancel();
+    void toggleTitleView();
 
     void exitWin();
 
@@ -81,8 +93,34 @@ class PlaybackBox : public MythDialog
     void changeSetRecGroup(void);
     void changeRecGroupPassword();
     void changeOldPasswordChanged(const QString &newText);
+    void doJobQueueJob(int jobType, int jobFlags = 0);
     void doBeginTranscoding();
     void doBeginFlagging();
+    void doBeginUserJob1();
+    void doBeginUserJob2();
+    void doBeginUserJob3();
+    void doBeginUserJob4();
+    void doPlaylistJobQueueJob(int jobType, int jobFlags = 0);
+    void stopPlaylistJobQueueJob(int jobType);
+    void doPlaylistBeginTranscoding();
+    void stopPlaylistTranscoding();
+    void doPlaylistBeginFlagging();
+    void stopPlaylistFlagging();
+    void doPlaylistBeginUserJob1();
+    void stopPlaylistUserJob1();
+    void doPlaylistBeginUserJob2();
+    void stopPlaylistUserJob2();
+    void doPlaylistBeginUserJob3();
+    void stopPlaylistUserJob3();
+    void doPlaylistBeginUserJob4();
+    void stopPlaylistUserJob4();
+    void doClearPlaylist();
+    void doPlaylistDelete();
+    void doPlaylistChangeRecGroup();
+    void togglePlayListTitle(void);
+    void togglePlayListItem(void);
+    void playSelectedPlaylist(bool random);
+    void doPlayList(void);
   protected:
     void paintEvent(QPaintEvent *);
     void keyPressEvent(QKeyEvent *e);
@@ -94,12 +132,15 @@ class PlaybackBox : public MythDialog
     QString cutDown(QString, QFont *, int);
     QPixmap getPixmap(ProgramInfo *);
     QPainter backup;
-    void play(ProgramInfo *);
+    bool play(ProgramInfo *);
     void stop(ProgramInfo *);
     void remove(ProgramInfo *);
     void expire(ProgramInfo *);
     void showActions(ProgramInfo *);
 
+    void togglePlayListItem(ProgramInfo *pginfo);
+    void randomizePlayList(void);
+
     bool haveGroupInfoSet;
     
     bool skipUpdate;
@@ -122,9 +163,13 @@ class PlaybackBox : public MythDialog
     int titleIndex;
     int progIndex;
     QStringList titleList;
+    QStringList playList;
+    bool onPlaylist;
     QMap<QString, ProgramList> progLists;
 
     ProgramInfo *findMatchingProg(ProgramInfo *);
+    ProgramInfo *findMatchingProg(QString key);
+    ProgramInfo *findMatchingProg(QString chanid, QString startts);
 
     BoxType type;
 
@@ -187,7 +232,8 @@ class PlaybackBox : public MythDialog
     void updateInfo(QPainter *);
     void updateUsage(QPainter *);
     void updateProgramInfo(QPainter *p, QRect& pr, QPixmap& pix);
-    void updateGroupInfo(QPainter *p, QRect& pr, QPixmap& pix);
+    void updateCurGroup(QPainter *p);
+    void updateGroupInfo(QPainter *p, QRect& pr, QPixmap& pix, QString cont_name = "group_info");
     
     QString showDateFormat;
     QString showTimeFormat;
@@ -205,6 +251,7 @@ class PlaybackBox : public MythDialog
     QRect infoRect;
     QRect usageRect;
     QRect videoRect;
+    QRect curGroupRect;
 
     int listsize;
 
@@ -244,6 +291,7 @@ class PlaybackBox : public MythDialog
     int underrectime;
 
     bool playingSomething;
+    bool titleView;
 };
 
 #endif
