@@ -5,17 +5,36 @@
 #include <qvbox.h>
 #include <qgrid.h>
 #include <qregexp.h>
+#include <qhostaddress.h>
 
 #include <unistd.h>
 
+#ifdef __linux__
+#include <sys/statvfs.h>
+#include <sys/sysinfo.h>
+#endif
+#if defined(__FreeBSD__) || defined(CONFIG_DARWIN)
+#include <sys/sysctl.h>
+#include <sys/param.h>
+#include <sys/mount.h>
+#endif
+#ifdef CONFIG_DARWIN
+#include <mach/mach.h>
+#endif
+
 #include <iostream>
+#include <cerrno>
 using namespace std;
 
+#include "config.h"
 #include "statusbox.h"
 #include "mythcontext.h"
 #include "remoteutil.h"
 #include "programinfo.h"
 #include "tv.h"
+#include "jobqueue.h"
+#include "util.h"
+#include "mythdbcon.h"
 
 StatusBox::StatusBox(MythMainWindow *parent, const char *name)
          : MythDialog(parent, name)
@@ -32,14 +51,27 @@ StatusBox::StatusBox(MythMainWindow *parent, const char *name)
   
     icon_list->SetItemText(item_count++, QObject::tr("Listings Status"));
     icon_list->SetItemText(item_count++, QObject::tr("Tuner Status"));
-#ifdef USING_DVB
-    if (gContext->GetNumSetting("DVBMonitorInterval", 0))
-        icon_list->SetItemText(item_count++, QObject::tr("DVB Status"));
-#endif
     icon_list->SetItemText(item_count++, QObject::tr("Log Entries"));
+    icon_list->SetItemText(item_count++, QObject::tr("Job Queue"));
+    icon_list->SetItemText(item_count++, QObject::tr("Machine Status"));
     icon_list->SetItemCurrent(0);
     icon_list->SetActive(true);
 
+    QStringList strlist;
+    strlist << "QUERY_IS_ACTIVE_BACKEND";
+    strlist << gContext->GetHostName();
+
+    gContext->SendReceiveStringList(strlist);
+
+    if (QString(strlist[0]) == "FALSE")
+        isBackend = false;
+    else if (QString(strlist[0]) == "TRUE")
+        isBackend = true;
+    else
+        isBackend = false;
+
+    VERBOSE(VB_NETWORK, QString("QUERY_IS_ACTIVE_BACKEND=%1").arg(strlist[0]));
+
     max_icons = item_count;
     inContent = false;
     contentPos = 0;
@@ -103,7 +135,11 @@ void StatusBox::updateContent()
         list_area->ResetList();
         for (int x = startPos; (x - startPos) <= contentSize; x++)
             if (contentLines.contains(x))
+            {
                 list_area->SetItemText(x - startPos, contentLines[x]);
+                if (contentFont.contains(x))
+                    list_area->EnableForcedFont(x - startPos, contentFont[x]);
+            }
     }
 
     // If we are scrolling, the determine the item to highlight
@@ -127,7 +163,7 @@ void StatusBox::updateContent()
         }
     }
 
-    list_area->SetUpArrow(startPos > 0);
+    list_area->SetUpArrow((startPos > 0) && (contentSize < contentTotalLines));
     list_area->SetDownArrow((startPos + contentSize) < contentTotalLines);
 
     container->Draw(&tmp, 0, 0);
@@ -298,12 +334,11 @@ void StatusBox::keyPressEvent(QKeyEvent *e)
                                  QObject::tr("Yes"), QObject::tr("No"), 0);
                 if (retval == 0)
                 {
-                    QString query = QString("UPDATE mythlog "
-                                            "SET acknowledged = 1 "
-                                            "WHERE priority <= %1")
-                                            .arg(min_level);
-                    QSqlDatabase *db = QSqlDatabase::database();
-                    db->exec(query);
+                    MSqlQuery query(MSqlQuery::InitCon());
+                    query.prepare("UPDATE mythlog SET acknowledged = 1 "
+                                  "WHERE priority <= :PRIORITY ;");
+                    query.bindValue(":PRIORITY", min_level);
+                    query.exec();
                     doLogEntries();
                 }
             }
@@ -356,15 +391,23 @@ void StatusBox::keyPressEvent(QKeyEvent *e)
             update(SelectRect);
             update(ContentRect);
         }
-        else if ((action == "LEFT") &&
-                 (inContent))
+        else if (action == "LEFT")
         {
-            inContent = false;
-            contentPos = 0;
-            list_area->SetActive(false);
-            icon_list->SetActive(true);
-            update(SelectRect);
-            update(ContentRect);
+            if (inContent)
+            {
+                inContent = false;
+                contentPos = 0;
+                list_area->SetActive(false);
+                icon_list->SetActive(true);
+                setHelpText();
+                update(SelectRect);
+                update(ContentRect);
+            }
+            else
+            {
+                if (gContext->GetNumSetting("UseArrowAccels", 1))
+                    accept();
+            }
         }
         else if ((currentItem == QObject::tr("Log Entries")) &&
                  (logNumberKeys.search(action) == 0))
@@ -413,53 +456,150 @@ void StatusBox::setHelpText()
             helptext->SetText(QObject::tr("Log Entries shows any unread log "
                                           "entries from the system if you "
                                           "have logging enabled"));
+        if (currentItem == QObject::tr("Job Queue"))
+            helptext->SetText(QObject::tr("Job Queue shows any jobs currently "
+                                          "in Myth's Job Queue such as a "
+                                          "commercial flagging job."));
+        if (currentItem == QObject::tr("Machine Status"))
+        {
+            QString machineStr = QObject::tr("Machine Status shows "
+                                             "some operating system "
+                                             "statistics of this machine");
+            if (!isBackend)
+                machineStr.append(" " + QObject::tr("and the MythTV server"));
+
+            helptext->SetText(machineStr);
+        }
     }
     update(TopRect);
 }
 
 void StatusBox::clicked()
 {
-    if ((inContent) &&
-        (icon_list->GetItemText(icon_list->GetCurrentItem()) == QObject::tr("Log Entries")))
+    QString currentItem = icon_list->GetItemText(icon_list->GetCurrentItem());
+
+    if (inContent)
     {
-        int retval = MythPopupBox::show2ButtonPopup(my_parent,
+        if (currentItem == QObject::tr("Log Entries"))
+        {
+            int retval;
+
+            retval = MythPopupBox::show2ButtonPopup(my_parent,
                                    QString("AckLogEntry"),
                                    QObject::tr("Acknowledge this log entry?"),
                                    QObject::tr("Yes"), QObject::tr("No"), 0);
-        if (retval == 0)
-        {
-            QString query = QString("UPDATE mythlog SET acknowledged = 1 "
-                                    "WHERE logid = %1")
-                                    .arg(contentData[contentPos]);
-            QSqlDatabase *db = QSqlDatabase::database();
-            db->exec(query);
-            doLogEntries();
+            if (retval == 0)
+            {
+                MSqlQuery query(MSqlQuery::InitCon());
+                query.prepare("UPDATE mythlog SET acknowledged = 1 "
+                              "WHERE logid = :LOGID ;");
+                query.bindValue(":LOGID", contentData[contentPos]);
+                query.exec();
+                doLogEntries();
+            }
         }
-    } else {
-        // Clear all visible content elements here
-        // I'm sure there's a better way to do this but I can't find it
-        content->ClearAllText();
-        list_area->ResetList();
-        contentLines.clear();
-        contentDetail.clear();
-        contentData.clear();
+        else if (currentItem == QObject::tr("Job Queue"))
+        {
+            QStringList msgs;
+            int jobStatus;
+            int retval;
 
-        QString currentItem;
+            jobStatus = JobQueue::GetJobStatus(
+                                contentData[contentPos].toInt());
 
-        currentItem = icon_list->GetItemText(icon_list->GetCurrentItem());
+            if (jobStatus == JOB_QUEUED)
+            {
+                retval = MythPopupBox::show2ButtonPopup(my_parent,
+                                       QString("JobQueuePopup"),
+                                       QObject::tr("Delete Job?"),
+                                       QObject::tr("Yes"),
+                                       QObject::tr("No"), 1);
+                if (retval == 0)
+                {
+                    JobQueue::DeleteJob(contentData[contentPos].toInt());
+                    doJobQueueStatus();
+                }
+            }
+            else if ((jobStatus == JOB_PENDING) ||
+                     (jobStatus == JOB_STARTING) ||
+                     (jobStatus == JOB_RUNNING))
+            {
+                msgs << QObject::tr("Pause");
+                msgs << QObject::tr("Stop");
+                msgs << QObject::tr("No Change");
+                retval = MythPopupBox::showButtonPopup(my_parent,
+                                       QString("JobQueuePopup"),
+                                       QObject::tr("Job Queue Actions:"),
+                                       msgs, 2);
+                if (retval == 0)
+                {
+                    JobQueue::PauseJob(contentData[contentPos].toInt());
+                    doJobQueueStatus();
+                }
+                else if (retval == 1)
+                {
+                    JobQueue::StopJob(contentData[contentPos].toInt());
+                    doJobQueueStatus();
+                }
+            }
+            else if (jobStatus == JOB_PAUSED)
+            {
+                msgs << QObject::tr("Resume");
+                msgs << QObject::tr("Stop");
+                msgs << QObject::tr("No Change");
+                retval = MythPopupBox::showButtonPopup(my_parent,
+                                       QString("JobQueuePopup"),
+                                       QObject::tr("Job Queue Actions:"),
+                                       msgs, 2);
+                if (retval == 0)
+                {
+                    JobQueue::ResumeJob(contentData[contentPos].toInt());
+                    doJobQueueStatus();
+                }
+                else if (retval == 1)
+                {
+                    JobQueue::StopJob(contentData[contentPos].toInt());
+                    doJobQueueStatus();
+                }
+            }
+            else if (jobStatus & JOB_DONE)
+            {
+                retval = MythPopupBox::show2ButtonPopup(my_parent,
+                                       QString("JobQueuePopup"),
+                                       QObject::tr("Requeue Job?"),
+                                       QObject::tr("Yes"),
+                                       QObject::tr("No"), 1);
+                if (retval == 0)
+                {
+                    JobQueue::ChangeJobStatus(contentData[contentPos].toInt(),
+                                              JOB_QUEUED);
+                    doJobQueueStatus();
+                }
+            }
+        }
 
-        if (currentItem == QObject::tr("Listings Status"))
-            doListingsStatus();
-
-        if (currentItem == QObject::tr("Tuner Status"))
-            doTunerStatus();
-
-        if (currentItem == QObject::tr("DVB Status"))
-            doDVBStatus();
-
-        if (currentItem == QObject::tr("Log Entries"))
-            doLogEntries();
+        return;
     }
+    
+    // Clear all visible content elements here
+    // I'm sure there's a better way to do this but I can't find it
+    content->ClearAllText();
+    list_area->ResetList();
+    contentLines.clear();
+    contentDetail.clear();
+    contentFont.clear();
+    contentData.clear();
+
+    if (currentItem == QObject::tr("Listings Status"))
+        doListingsStatus();
+    else if (currentItem == QObject::tr("Tuner Status"))
+        doTunerStatus();
+    else if (currentItem == QObject::tr("Log Entries"))
+        doLogEntries();
+    else if (currentItem == QObject::tr("Job Queue"))
+        doJobQueueStatus();
+    else if (currentItem == QObject::tr("Machine Status"))
+        doMachineStatus();
 }
 
 void StatusBox::doListingsStatus()
@@ -468,17 +608,18 @@ void StatusBox::doListingsStatus()
     QString querytext, DataDirectMessage;
     int DaysOfData;
     QDateTime qdtNow, GuideDataThrough;
-    QSqlDatabase *db = QSqlDatabase::database();
     int count = 0;
 
     contentLines.clear();
     contentDetail.clear();
+    contentFont.clear();
     doScroll = false;
 
     qdtNow = QDateTime::currentDateTime();
-    querytext = QString("SELECT max(endtime) FROM program;");
 
-    QSqlQuery query = db->exec(querytext);
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("SELECT max(endtime) FROM program;");
+    query.exec();
 
     if (query.isActive() && query.numRowsAffected())
     {
@@ -550,12 +691,13 @@ void StatusBox::doTunerStatus()
     int count = 0;
     doScroll = true;
 
-    QString querytext = QString("SELECT cardid FROM capturecard;");
-    QSqlDatabase *db = QSqlDatabase::database();
-    QSqlQuery query = db->exec(querytext);
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("SELECT cardid FROM capturecard;");
+    query.exec();
 
     contentLines.clear();
     contentDetail.clear();
+    contentFont.clear();
 
     if (query.isActive() && query.numRowsAffected())
     {
@@ -568,12 +710,15 @@ void StatusBox::doTunerStatus()
             strlist << "GET_STATE";
 
             gContext->SendReceiveStringList(strlist);
+            int state = strlist[0].toInt();
   
             QString Status = QString(tr("Tuner %1 ")).arg(cardid);
-            if (strlist[0].toInt()==kState_WatchingLiveTV)
+            if (state==kState_Error)
+                Status += tr("is not available");
+            else if (state==kState_WatchingLiveTV)
                 Status += tr("is watching live TV");
-            else if (strlist[0].toInt()==kState_RecordingOnly ||
-                     strlist[0].toInt()==kState_WatchingRecording)
+            else if (state==kState_RecordingOnly ||
+                     state==kState_WatchingRecording)
                 Status += tr("is recording");
             else 
                 Status += tr("is not recording");
@@ -581,7 +726,8 @@ void StatusBox::doTunerStatus()
             contentLines[count] = Status;
             contentDetail[count] = Status;
 
-            if (strlist[0].toInt()==kState_RecordingOnly)
+            if (state==kState_RecordingOnly ||
+                state==kState_WatchingRecording)
             {
                 strlist = QString("QUERY_RECORDER %1").arg(cardid);
                 strlist << "GET_RECORDING";
@@ -589,7 +735,7 @@ void StatusBox::doTunerStatus()
                 ProgramInfo *proginfo = new ProgramInfo;
                 proginfo->FromStringList(strlist, 0);
    
-                Status = proginfo->title;
+                Status += " " + proginfo->title;
                 Status += "\n";
                 Status += proginfo->subtitle;
                 contentDetail[count] = Status;
@@ -601,92 +747,6 @@ void StatusBox::doTunerStatus()
     update(ContentRect);
 }
 
-void StatusBox::doDVBStatus(void)
-{
-    QString querytext;
-    bool doneAnything = false;
-  
-    doScroll = false;
-    int count = 0;
-  
-    contentLines.clear();
-    contentDetail.clear();
- 
-    QString Status = QObject::tr("Details of DVB error statistics for last 48 "
-                                 "hours:\n");
-
-    QString outerqry =
-        "SELECT starttime,endtime FROM recorded "
-        "WHERE starttime >= DATE_SUB(NOW(), INTERVAL 48 HOUR) "
-        "ORDER BY starttime;";
-
-    QSqlDatabase *db = QSqlDatabase::database();
-    QSqlQuery oquery = db->exec(outerqry);
-
-    if (oquery.isActive() && oquery.numRowsAffected())
-    {
-        querytext = QString("SELECT cardid,"
-                            "max(fe_ss),min(fe_ss),avg(fe_ss),"
-                            "max(fe_snr),min(fe_snr),avg(fe_snr),"
-                            "max(fe_ber),min(fe_ber),avg(fe_ber),"
-                            "max(fe_unc),min(fe_unc),avg(fe_unc),"
-                            "max(myth_cont),max(myth_over),max(myth_pkts) "
-                            "FROM dvb_signal_quality "
-                            "WHERE sampletime BETWEEN ? AND ? "
-                            "GROUP BY cardid");
-
-        QSqlQuery query = db->exec(NULL);
-        query.prepare(querytext);
-        
-        while (oquery.next())
-        {
-            QDateTime t_start = oquery.value(0).toDateTime();
-            QDateTime t_end = oquery.value(1).toDateTime();
-
-            query.bindValue(0, t_start);
-            query.bindValue(1, t_end);
-
-            if (!query.exec())
-                cout << query.lastError().databaseText() << "\r\n" 
-                     << query.lastError().driverText() << "\r\n";
-            
-            if (query.isActive() && query.numRowsAffected())
-            {
-                contentLines[count++] =
-                       QObject::tr("Recording period from %1 to %2")
-                                   .arg(t_start.toString())
-                                   .arg(t_end.toString());
-                
-                while (query.next())
-                {
-                    contentLines[count++] =
-                           QObject::tr("Encoder %1 Min SNR: %2 Avg SNR: %3 Min "
-                                       "BER %4 Avg BER %5 Cont Errs: %6 "
-                                       "Overflows: %7")
-                                       .arg(query.value(0).toInt())
-                                       .arg(query.value(5).toInt())
-                                       .arg(query.value(6).toInt())
-                                       .arg(query.value(8).toInt())
-                                       .arg(query.value(9).toInt())
-                                       .arg(query.value(13).toInt())
-                                       .arg(query.value(14).toInt());
-
-                    doneAnything = true;
-                }
-            }
-        }
-    }
-
-    if (!doneAnything)
-    {
-        contentLines[count++] = QObject::tr("There is no DVB signal quality "
-                                            "data available to display.");
-    }
-
-    contentTotalLines = count;
-    update(ContentRect);
-}
-
 void StatusBox::doLogEntries(void)
 {
     QString line;
@@ -696,16 +756,17 @@ void StatusBox::doLogEntries(void)
 
     contentLines.clear();
     contentDetail.clear();
+    contentFont.clear();
     contentData.clear();
 
-    QSqlDatabase *db = QSqlDatabase::database();
-    QString thequery;
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("SELECT logid, module, priority, logdate, host, "
+                  "message, details "
+                  "FROM mythlog WHERE acknowledged = 0 "
+                  "AND priority <= :PRIORITY ORDER BY logdate DESC;");
+    query.bindValue(":PRIORITY", min_level);
+    query.exec();
 
-    thequery = QString("SELECT logid, module, priority, logdate, host, "
-                       "message, details "
-                       "FROM mythlog WHERE acknowledged = 0 AND priority <= %1 "
-                       "ORDER BY logdate DESC;").arg(min_level);
-    QSqlQuery query = db->exec(thequery);
     if (query.isActive())
     {
         while (query.next())
@@ -714,7 +775,7 @@ void StatusBox::doLogEntries(void)
             contentLines[count] = line;
 
             if (query.value(6).toString() != "")
-                line = QString("On %1 %2 from %3.%4\n%5\n%6")
+                line = tr("On %1 %2 from %3.%4\n%5\n%6")
                                .arg(query.value(3).toDateTime()
                                          .toString(dateFormat))
                                .arg(query.value(3).toDateTime()
@@ -722,9 +783,9 @@ void StatusBox::doLogEntries(void)
                                .arg(query.value(4).toString())
                                .arg(query.value(1).toString())
                                .arg(query.value(5).toString())
-                               .arg(query.value(6).toString());
+                               .arg(QString::fromUtf8(query.value(6).toString()));
             else
-                line = QString("On %1 %2 from %3.%4\n%5\nNo other details")
+                line = tr("On %1 %2 from %3.%4\n%5\nNo other details")
                                .arg(query.value(3).toDateTime()
                                          .toString(dateFormat))
                                .arg(query.value(3).toDateTime()
@@ -751,3 +812,314 @@ void StatusBox::doLogEntries(void)
     update(ContentRect);
 }
 
+void StatusBox::doJobQueueStatus()
+{
+    QMap<int, JobQueueEntry> jobs;
+    QMap<int, JobQueueEntry>::Iterator it;
+    int count = 0;
+
+    QString detail;
+
+    JobQueue::GetJobsInQueue(jobs,
+                             JOB_LIST_NOT_DONE | JOB_LIST_ERROR |
+                             JOB_LIST_RECENT);
+
+    doScroll = true;
+
+    contentLines.clear();
+    contentDetail.clear();
+    contentFont.clear();
+    contentData.clear();
+
+    if (jobs.size())
+    {
+        for (it = jobs.begin(); it != jobs.end(); ++it)
+        {
+            QString chanid = it.data().chanid;
+            QDateTime starttime = it.data().starttime;
+            ProgramInfo *pginfo;
+
+            pginfo = ProgramInfo::GetProgramFromRecorded(chanid, starttime);
+
+            if (!pginfo)
+                continue;
+
+            detail = pginfo->title + "\n" +
+                     pginfo->channame + " " + pginfo->chanstr +
+                         " @ " + starttime.toString(timeDateFormat) + "\n" +
+                     tr("Job:") + " " + JobQueue::JobText(it.data().type) +
+                     "     " + tr("Status: ") +
+                     JobQueue::StatusText(it.data().status);
+
+            if (it.data().status != JOB_QUEUED)
+                detail += " (" + it.data().hostname + ")";
+
+            detail += "\n" + it.data().comment;
+
+            contentLines[count] = pginfo->title + " @ " +
+                                  starttime.toString(timeDateFormat);
+
+            contentDetail[count] = detail;
+            contentData[count] = QString("%1").arg(it.data().id);
+
+            if (it.data().status == JOB_ERRORED)
+                contentFont[count] = "error";
+            else if (it.data().status == JOB_ABORTED)
+                contentFont[count] = "warning";
+
+            count++;
+
+            delete pginfo;
+        }
+    }
+    else
+    {
+        contentLines[count++] = QObject::tr("Job Queue is currently empty.");
+        doScroll = false;
+    }
+
+    contentTotalLines = count;
+    update(ContentRect);
+}
+
+// Some helper routines for doMachineStatus() that format the output strings
+
+#define MB (1024*1024)
+
+static const QString usageStr(float total, float used, float free)
+{
+    char  buff1[10];       // QString::arg(double) doesn't allow %0.2f,
+    char  buff2[10];       // so we need to use sprintf() with a buffer
+    char *units  = "MB";
+    char *format = "%0.0f";
+
+    if (total == 0.0)
+        return "unknown";
+
+    if (total > 1024)
+        units = "GB", format = "%0.2f",
+        total /= 1024, used /= 1024, free /= 1024;
+
+    sprintf(buff1, format, total);
+
+    QString s = QString("%1 %2").arg(buff1).arg(units);
+    //QString s = QString("%1 %2 %3")
+                //.arg(buff1).arg(units).arg(QObject::tr("total"));
+
+
+    // If, some some reason, we didn't get the usage stats, shorten output:
+
+    if (used == 0 && free == 0)
+        return s;
+
+
+    sprintf(buff1, format, used);
+    sprintf(buff2, format, free);
+
+    return s + ", " +
+           QString("%1 %2 %3")
+           .arg(buff1).arg(units).arg(QObject::tr("used"))
+           + ", " +
+           QString("%1 %2 %3 (%4%)")
+           .arg(buff2).arg(units).arg(QObject::tr("free"))
+           .arg((int)(100*free/total));
+}
+
+static const QString diskUsageStr(const char *path)
+{
+    double total, free, used;
+
+    if (diskUsage(path, total, used, free))
+        return usageStr (total, used, free);
+    else
+        return QString("%1 - %2").arg(path).arg(strerror(errno));
+}
+
+static const QString uptimeStr(time_t uptime)
+{
+    int     days, hours, min, secs;
+    QString str;
+
+    str = QString("   " + QObject::tr("Uptime") + ": ");
+
+    if (uptime == 0)
+        return str + "unknown";
+
+    days = uptime/(60*60*24);
+    uptime -= days*60*60*24;
+    hours = uptime/(60*60);
+    uptime -= hours*60*60;
+    min  = uptime/60;
+    secs = uptime%60;
+
+    if (days > 0)
+    {
+        char    buff[6];
+        QString dayLabel;
+
+        if (days == 1)
+            dayLabel = QObject::tr("day");
+        else
+            dayLabel = QObject::tr("days");
+
+        sprintf(buff, "%d:%02d", hours, min);
+
+        return str + QString("%1 %2, %3").arg(days).arg(dayLabel).arg(buff);
+    }
+    else
+    {
+        char  buff[9];
+
+        sprintf(buff, "%d:%02d:%02d", hours, min, secs);
+
+        return str + buff;
+    }
+}
+
+void StatusBox::doMachineStatus()
+{
+    int           count(0);
+    int           totalM, usedM, freeM;    // Physical memory
+    int           totalS, usedS, freeS;    // Virtual  memory (swap)
+    time_t        uptime;
+
+    contentLines.clear();
+    contentDetail.clear();
+    contentFont.clear();
+    //doScroll = false;??
+
+    if (isBackend)
+        contentLines[count++] = QObject::tr("System") + ":";
+    else
+        contentLines[count++] = QObject::tr("This machine") + ":";
+
+    // uptime
+    if (!getUptime(uptime))
+        uptime = 0;
+    contentLines[count] = uptimeStr(uptime);
+
+    // weighted average loads
+    contentLines[count].append(".   " + QObject::tr("Load") + ": ");
+
+    double loads[3];
+    if (getloadavg(loads,3) == -1)
+        contentLines[count].append(QObject::tr("unknown") +
+                                   " - getloadavg() " + QObject::tr("failed"));
+    else
+    {
+        char buff[30];
+
+        sprintf(buff, "%0.2lf, %0.2lf, %0.2lf", loads[0], loads[1], loads[2]);
+        contentLines[count].append(QString(buff));
+    }
+    count++;
+
+
+    // memory usage
+    if (getMemStats(totalM, freeM, totalS, freeS))
+    {
+        usedM = totalM - freeM;
+        if (totalM > 0)
+            contentLines[count++] = "   " + QObject::tr("RAM") +
+                                    ": "  + usageStr(totalM, usedM, freeM);
+        usedS = totalS - freeS;
+        if (totalS > 0)
+            contentLines[count++] = "   " +
+                                    QObject::tr("Swap") +
+                                    ": "  + usageStr(totalS, usedS, freeS);
+    }
+
+    if (!isBackend)
+    {
+        contentLines[count++] = QObject::tr("MythTV server") + ":";
+
+        // uptime
+        if (!RemoteGetUptime(uptime))
+            uptime = 0;
+        contentLines[count] = uptimeStr(uptime);
+
+        // weighted average loads
+        contentLines[count].append(".   " + QObject::tr("Load") + ": ");
+        float loads[3];
+        if (RemoteGetLoad(loads))
+        {
+            char buff[30];
+
+            sprintf(buff, "%0.2f, %0.2f, %0.2f", loads[0], loads[1], loads[2]);
+            contentLines[count].append(QString(buff));
+        }
+        else
+            contentLines[count++].append(QObject::tr("unknown"));
+
+        // memory usage
+        if (RemoteGetMemStats(totalM, freeM, totalS, freeS))
+        {
+            usedM = totalM - freeM;
+            if (totalM > 0)
+                contentLines[count++] = "   " + QObject::tr("RAM") +
+                                        ": "  + usageStr(totalM, usedM, freeM);
+            usedS = totalS - freeS;
+            if (totalS > 0)
+                contentLines[count++] = "   " + QObject::tr("Swap") +
+                                        ": "  + usageStr(totalS, usedS, freeS);
+        }
+    }
+
+    // get free disk space
+    contentLines[count++] = QString(QObject::tr("Disk space:"));
+
+    if (isBackend)
+    {
+        QString str;
+
+        contentLines[count] = "   " + QString(QObject::tr("Recordings")) + ": ";
+
+        // Perform the database queries
+        MSqlQuery query(MSqlQuery::InitCon());
+        query.prepare("SELECT * FROM settings "
+                      "WHERE value=\"RecordFilePrefix\" "
+                      "AND hostname = :HOSTNAME ;");
+        query.bindValue(":HOSTNAME", gContext->GetHostName());
+        query.exec();
+
+        // did we get results?
+        if (query.isActive() && query.numRowsAffected())
+        {
+            // skip to first result
+            query.next();
+            str = diskUsageStr(query.value(1).toCString());
+        }
+        else
+            str = QString(QObject::tr("DB error - RecordFilePrefix unknown"));
+        contentLines[count++].append(str);
+
+        // Ditto for the second path
+        contentLines[count] = "   " + QString(QObject::tr("TV buffer")) + ": ";
+
+        query.prepare("SELECT * FROM settings "
+                      "WHERE value=\"LiveBufferDir\" "
+                      "AND hostname = :HOSTNAME ;");
+        query.bindValue(":HOSTNAME", gContext->GetHostName());
+        query.exec();
+        if (query.isActive() && query.numRowsAffected())
+        {
+            query.next();
+            str = diskUsageStr(query.value(1).toCString());
+        }
+        else
+            str = QString(QObject::tr("DB error - LiveBufferDir unknown"));
+        contentLines[count++].append(str);
+    }
+    else
+    {
+        int  total, used;
+        RemoteGetFreeSpace(total, used);
+        if (total)
+            contentLines[count++] = "   " + usageStr(total, used, total - used);
+        else
+            contentLines[count++] = "   " + QString(QObject::tr("Unknown"));
+    }
+
+    contentTotalLines = count;
+    update(ContentRect);
+}
